import { SelectProps } from '@material-ui/core';
import * as React from 'react';

export interface ArgousComplexPaginationProps {
  /**
   * Array de opções para as quantidades de linhas por pagina possiveis
   * 
   * Se receber zero como opção ao invés de um array a tabela tera todas as linahs numa só página
   */ 
  rowsPerPageOptions?: Array<number> | 0, 
  /**
   * Mostra a quantidade total de linhas da tabela
   */ 
  showCount?: boolean
}

export interface ArgousComplexPaginationPrivateProps extends ArgousComplexPaginationProps {
  /**
   * Componente com os botões de mudança de página
   */
  ActionsComponent: (props: {count: number, page: number, rowsPerPage: number, onChangePage: (evento: EventTarget, novaPagina: number) => void}) => JSX.Element,
  /**
   * Componente com o display das páginas
   */
  LabelDisplayedRows: (props: {pages: number, page: number, onChangePage: (evento: EventTarget, novaPagina: number) => void, from: number, to: number, count: number, showCount: boolean}) => JSX.Element,
  /**
   * Classes aplicadas á raiz do componente
   */
  className?: string,
  /**
   * Texto representando os itens da paginação (Linhas por padrão)
   */
  labelRowsPerPage?: string,
  /**
   * Props repassados ao componente de seleção
   */
  SelectProps?: SelectProps,
  /**
   * Quantidade total de linhas
   */
  count: number,
  /**
   * Função chamada ao mudar de página
   * 
   * @param evento - Evento do click
   * @param novaPagina - Número da nova página selecionada
   * 
   * @returns {void} Sem retorno
   */
  onChangePage: (evento: EventTarget, novaPagina: number) => void,
  /**
   *  Função chamada ao mudar a quantidade de itens por página
   * 
   * @param evento - Evento do click
   * @param filho - Elemento que foi selecionado
   * 
   * @returns {void} Sem retorno
   */
  onChangeRowsPerPage: ((evento: React.ChangeEvent<{name?: string | undefined; value: unknown;}>, filho: React.ReactNode) => void) | undefined,
  /**
   * Página atual 
   */
  page: number,
  /**
   * Quantidade de linhas por página
   */
  rowsPerPage: number,
  // backIconButtonProps?,
  // backIconButtonText?: string,
  // nextIconButtonProps?,
  // nextIconButtonText?: string,
}

declare function ArgousComplexPagination(props: ArgousComplexPaginationPrivateProps): JSX.Element

export default ArgousComplexPagination;