import { ArgousComplexPaginationPrivateProps, ArgousComplexPaginationProps } from '../ArgousComplexPagination/ArgousComplexPagination';
import { ArgousComplexTableBodyPrivateProps, ArgousComplexTableBodyProps } from '../ArgousComplexTableBody/ArgousComplexTableBody';
import { ArgousComplexTableHeadPrivateProps, ArgousComplexTableHeadProps } from '../ArgousComplexTableHead/ArgousComplexTableHead';
import { ArgousComplexTableToolBarPrivateProps, ArgousComplexTableToolBarProps } from '../ArgousComplexTableToolbar/ArgousComplexTableToolBar';
import { TableContainerProps, TableProps } from '@material-ui/core';
import React from 'react';

export interface ArgousComplexTableProps extends ArgousComplexTableBodyProps, ArgousComplexPaginationProps, ArgousComplexPaginationProps, ArgousComplexTableHeadProps, ArgousComplexTableToolBarProps {
  /**
   * Componente para substituir a Toolbar padrão
   */ 
  ToolbarComponent?: (props: ArgousComplexTableToolBarPrivateProps) => JSX.Element,
  /**
   * Componente para substituir o Header padrão
   */ 
  HeadComponent?: (props: ArgousComplexTableHeadPrivateProps) => JSX.Element,
  /**
   * Componente para substituir o Body padrão
   */ 
  BodyComponent?: (props: ArgousComplexTableBodyPrivateProps) => JSX.Element,
  /**
   * Componente para substituir a pagination padrão
   */ 
  PaginationComponent?: (props: ArgousComplexPaginationPrivateProps) => JSX.Element,
  /**
   * Adiciona um Footer à tabela
   */ 
  FooterComponent?: React.ReactNode,
  /**
   * Props repassados ao componente de footer
   */ 
  footerProps?: object,
  /**
   * Array de dados fornecido para a tabela
   */ 
  dados?: Array<Array<any> | Object>,
  /**
   * Número de linhas padrão por página
   */ 
  defaultRowsPerPage?: number,
  /**
   * Estilos aplicados à tabela
   */ 
  style?: React.CSSProperties,
  /**
   * Classes aplicadas à tabela
   */ 
  className?: string,
  /**
   * Tamanho da tabela
   */ 
  size?: 'small' | 'normal',
  /**
   * Função chamada ao abrir um item
   * 
   * @param linhaOriginal - Linha original 
   * @param linhaFormatada - Linha formatada
   * 
   * @returns {void} Sem retorno
   */ 
  onOpen?: (linhaOriginal: Array<any> | object, linhaFormatada: Array<any>) => void,
  /**
   * Função chamada ao fechar um item
   * 
   * @param linhaOriginal - Linha original 
   * @param linhaFormatada - Linha formatada
   * 
   * @returns {void} Sem retorno
   */ 
  onClose?: (linhaOriginal: Array<any> | object, linhaFormatada: Array<any>) => void,
  /**
   * Nome de metódo utilizando o padrão de caminhador argous para prover os dados para a tabela
   */ 
  method?: string,
  /**
   * Objeto de parâmetros passados para o método
   */ 
  methodPrms?: object,
  /**
   * Nome da chave do objeto de resposta do método (lista por padrão)
   */ 
  methodResultList?: string,
  /**
   * Função chamada apos o metodo receber uma resposta 
   * 
   * @param response - Resposta
   * 
   * @returns {void} Sem retorno
   */
  onMethodResponse?: (response: Object) => void,
  /**
   * Boolean que define se o erro do método será tratado automaticamente na ServerAction
   */
  handleMethodError?: boolean,
  /**
   * Boolean que permite solicitar o método
   */
  allowMethodRequest?: boolean,
  /**
   * @deprecated
   * 
   * Field contendo os dados a serem providos para a tabela
   */ 
  field?: string,
  /**
   * Função que é chamada sempre que os itens selecionados mudam
   * 
   * @param itensSelecionados - Array com os itens selecionados
   * 
   * @returns {void} Sem retorno
   */ 
  onSelect?: (itensSelecionados: Array<any>) => void,
  /**
   * Referência utilizada para salvar o estado da tabela quando ela é descarregada
   */ 
  //TODO criar interface para o savedstate
  savedState?: React.RefObject<any>,
  /**
   * Referencia passada ao componente
   */
  //TODO criar interface para a referencia
  ref?: React.RefObject<any>,
  /**
   * Props repassados ao componente container
   */
  containerProps?: TableContainerProps,
  /**
   * Props repassados ao componente table
   */
  tableProps?: TableProps,
  /**
   * Props repassados ao componente toolbar
   */
  toolbarProps?: any,
  /**
   * Objeto de controle da paginação/ordenação que será fornecido à tabela e ao método externo
   */
  externalCtrlPgn?: object,
  /**
   * Função que é chamada para setar o externalCtrlPgn sempre que os controles de paginação/ordenação mudarem
   * 
   * @param value - Objeto com parâmetros da paginação/ordenação para ser enviado ao método
   * 
   * @returns {void} Sem retorno
   */ 
  setExternalCtrlPgn?: (value: Object) => void,
  /**
   * Program code (pgCode) que será utilizado junto ao método
   */
  code?: string,
  /**
   * Token que será utilizado junto ao método
   */
  token?: string
}

declare function ArgousComplexTable(props: ArgousComplexTableProps): JSX.Element

export default ArgousComplexTable;