import * as React from 'react';

export interface ArgousComplexTableHeadProps {
  /**
   * Array de títulos para o cabeçalho da tabela
   */ 
  head?: Array<string>, 
  /**
   * Array de estilos para cada item do cabeçalho
   */ 
  headerColumnStyles?: Array<React.CSSProperties>,
  /**
   * Array de booleanos para desabilitar a ordenacao em itens expecificos
   */ 
  disableSorting?: Array<boolean>,
}

export interface ArgousComplexTableHeadPrivateProps extends ArgousComplexTableHeadProps {
  /**
   * Quantidade de itens selecionados
   */
  numSelected: number,
  /**
   * Quantidade de linhas por página
   */
  rowCount: number,
  /**
   * Habilita a seleção de linhas da lista 
   */ 
  enableCheck?: boolean,
  /**
   * Habilita o drag and drop das linhas
   */ 
  enableDrag?: boolean,
  /**
   * Função que decide se a seleção de determinada linha está habilitada
   * 
   * @param linhaOriginal - Linha original, antes de ser formatada pela função line
   * @param linhaFormatada - Linha ja formatada pela função line
   * @param index - Índice da linha
   * 
   * @returns {boolean} Booleano
   */ 
  disableCheckIf?: (linhaOriginal: Array<any> | object, linhaFormatada: Array<any>, index: number) => boolean,
  /**
   * Função que converte a linha original em um array com os dados a serem exibidos
   * 
   * @param linhaOriginal - linha a ser formatada
   * 
   * @returns {Array<any>} Retorna array
   */ 
  line: (linhaOriginal: Array<Array<any> | Object>) => Array<any>, 
  /**
   * Quantidade total de linhas na tabela
   */
  size: number,
  /**
   * Array de booleanos retornado pelo caminhador para habilitar a ordenação
   */ 
  ordCols: Array<boolean>,
  /**
   * Recebe o array com todos os dados e retorna um array somente com os dados mostrados
   * 
   * @param dados - Dados completos
   * 
   * @returns {Array<Array<any> | Object>} Array com dados mostrados
   */
  cortaDados: (dados: Array<Array<any> | Object>) => Array<Array<any> | Object>
}

declare function ArgousComplexTableHead(props: ArgousComplexTableHeadPrivateProps): JSX.Element

export default ArgousComplexTableHead;