import * as React from 'react';
import { IconType } from 'react-icons';

declare interface ActionsFunction {
  /**
   * @param identificador - Array com todos os itens que foram selecionados
   * 
   * @returns {void} Sem retorno
   */ 
  (identificador: Array<any>): void
}

export interface ArgousComplexTableToolBarProps {
  /**
   * Recebe um array com as ações para a tabela, cada item do array deve ser outro array
   * com três itens na seguinte ordem:
   * 
   * @param icone - Icone, do react icons, que ficará visivel na action
   * @param title - Texto que aparecerá ao passar o mouse por cima do icone
   * @param callback - Funcao q será chamada ao clicar no icone, ela pode receber como parametros os itens selecioandos na lista
   */ 
  actions?: Array<Array<IconType | string | ActionsFunction>>,
  /**
   * Titulo que aparecerá acima da tabela
   */ 
  title?: string,
  /**
   * Desabilita o filtro
   */ 
  disableFilter?: boolean,
  /**
   * Label a ser aplicado ao filtro
   */ 
  filterLabel?: string,
  /**
   * Deixa a barra de ferramentas fixa sempre ativa
   */ 
  fixedToolbar?: boolean,
}

export interface ArgousComplexTableToolBarPrivateProps extends ArgousComplexTableToolBarProps {
  /**
   * Quantidade de itens selecionados
   */
  numSelected: number,
  /**
   * Lista com os itens da tabela
   */
  lista: Array<Array<any> | Object>,
  /**
   * Ìndice da linha original a ser utilizado como ID primário na hora das seleções e que será retornado pelas actions
   */ 
  indexToSelect?: number | string,
}

declare function ArgousComplexTableToolBar(props: ArgousComplexTableToolBarPrivateProps): JSX.Element

export default ArgousComplexTableToolBar;