"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@material-ui/core");
var prop_types_1 = __importDefault(require("prop-types"));
var react_1 = __importDefault(require("react"));
var useStyles = core_1.makeStyles(function (theme) { return ({
    progressContainer: {
        '& > div': {
            padding: '0px !important'
        }
    },
    progressBorder: {
        border: function (props) { return "3px solid " + (props.borderColor || props.preset.borderColor); },
        borderRadius: '15px',
    },
    progressBack: {
        height: '20px',
        padding: '2px',
        position: 'relative'
    },
    progressBar: {
        width: '100%',
        height: '100%',
        borderRadius: '10px',
        background: function (props) { return props.percnt === 0 ? theme.palette.background.paper : props.percnt > 100 ? props.fullBarColor || props.preset.fullBarColor : props.barColor || props.preset.barColor; },
    },
    progressCover: {
        height: '100%',
        transition: '300ms',
        background: function (props) { return props.percnt > 99 ? 'transparent' : theme.palette.background.paper; },
        position: 'absolute',
        top: '0px',
        right: '0',
        width: function (props) { return 100 - props.percnt + "%"; },
        minWidth: '5%',
        maxWidth: '95%',
        borderRadius: '0px 10px 10px 0px;',
    },
    topIndicator: {
        zIndex: 1,
        transition: '300ms',
        display: 'block',
        position: 'relative',
        width: 'max-content',
        margin: '0 2px',
        paddingLeft: function (props) { return props.percnt > 94 ? '94%' : props.percnt < 5 ? '0' : props.percnt + "%"; },
        transform: function (props) { return props.percnt === 0 ? "translate(" + (props.compact ? '-25px, 1px' : '-10px, 0px') + ")" : props.percnt > 5 ? "translate(" + (props.compact ? '-30px, 1px' : '-15px, 0px') + ")" : ''; },
        '& p': {
            margin: '0 0 5px',
            border: function (props) { return props.compact ? 0 : "2px solid " + (props.topIndicatorColor || props.preset.indicatorColor); },
            padding: '0px 3px',
            borderRadius: '5px',
            minWidth: '25px',
            textAlign: 'center',
            cursor: 'default',
            lineHeight: '12px',
            height: '24px',
        },
        '& p::before': {
            content: '""',
            position: 'relative',
            bottom: '-22px',
            left: '0px',
            width: '0px',
            height: '0px',
            borderTop: function (props) { return "5px solid " + (props.topIndicatorColor || props.preset.indicatorColor); },
            borderLeft: '10px solid transparent',
            borderRight: '10px solid transparent',
            display: 'block',
            opacity: function (props) { return props.compact ? 0 : 1; }
        },
    },
    rightIndicator: {
        display: 'block',
        position: 'relative',
        width: 'max-content',
        '& p': {
            margin: '0 7px 0',
            border: function (props) { return "2px solid " + (props.rightIndicatorColor || props.preset.indicatorColor); },
            padding: '0px 3px',
            borderRadius: '5px',
            minWidth: '25px',
            textAlign: 'center',
            minHeight: '24px',
            cursor: 'default'
        },
        '& p::before': {
            content: '""',
            position: 'absolute',
            bottom: '2px',
            left: '0px',
            width: '0px',
            height: '0px',
            borderTop: '10px solid transparent',
            borderBottom: '10px solid transparent',
            borderRight: function (props) { return "8px solid " + (props.rightIndicatorColor || props.preset.indicatorColor); },
        },
    },
    collapse: {
        height: function (props) { return props.compact ? '0px !important' : undefined; }
    }
}); });
var selectPreset = function (preset, palette) {
    switch (preset) {
        case 'secondary':
            return { fullBarColor: palette.secondary.dark, borderColor: palette.secondary.dark, indicatorColor: palette.secondary.dark, barColor: "linear-gradient(90deg, " + palette.secondary.light + " 0%, " + palette.secondary.main + " 50%, " + palette.secondary.dark + " 100%)" };
        case 'success':
            return { fullBarColor: palette.success.dark, borderColor: palette.success.dark, indicatorColor: palette.success.dark, barColor: "linear-gradient(90deg, " + palette.success.light + " 0%, " + palette.success.main + " 50%, " + palette.success.dark + " 100%)" };
        case 'info':
            return { fullBarColor: palette.info.dark, borderColor: palette.info.dark, indicatorColor: palette.info.dark, barColor: "linear-gradient(90deg, " + palette.info.light + " 0%, " + palette.info.main + " 50%, " + palette.info.dark + " 100%)" };
        case 'warning':
            return { fullBarColor: palette.warning.dark, borderColor: palette.warning.dark, indicatorColor: palette.warning.dark, barColor: "linear-gradient(90deg, " + palette.warning.light + " 0%, " + palette.warning.main + " 50%, " + palette.warning.dark + " 100%)" };
        case 'danger':
            return { fullBarColor: palette.error.dark, borderColor: palette.error.dark, indicatorColor: palette.error.dark, barColor: "linear-gradient(90deg, " + palette.error.light + " 0%, " + palette.error.main + " 50%, " + palette.error.dark + " 100%)" };
        case 'hot':
            return { fullBarColor: 'rgba(244,67,54,1)', borderColor: '#d84315', indicatorColor: '#f44336', barColor: 'linear-gradient(90deg, rgba(255,235,59,1) 0%, rgba(255,152,0,1) 50%, rgba(244,67,54,1) 100%)' };
        case 'cold':
            return { fullBarColor: '#673ab7', borderColor: '#5e35b1', indicatorColor: '#512da8', barColor: 'linear-gradient(90deg, #2196f3 0%, #3f51b5 50%, #673ab7 100%)' };
        default:
            return { fullBarColor: palette.primary.dark, borderColor: palette.primary.dark, indicatorColor: palette.primary.dark, barColor: "linear-gradient(90deg, " + palette.primary.light + " 0%, " + palette.primary.main + " 50%, " + palette.primary.dark + " 100%)" };
    }
};
function ProgressBar(_a) {
    var valueLabel = _a.valueLabel, valueSuffix = _a.valueSuffix, maxValueSuffix = _a.maxValueSuffix, maxValueLabel = _a.maxValueLabel, value = _a.value, maxValue = _a.maxValue, rightIndicatorColor = _a.rightIndicatorColor, topIndicatorColor = _a.topIndicatorColor, borderColor = _a.borderColor, barColor = _a.barColor, fullBarColor = _a.fullBarColor, preset = _a.preset, compact = _a.compact, className = _a.className;
    var theme = core_1.useTheme();
    var percnt = maxValue ? value * 100 / maxValue : 0;
    var classes = useStyles({ percnt: percnt, rightIndicatorColor: rightIndicatorColor, topIndicatorColor: topIndicatorColor, borderColor: borderColor, barColor: barColor, fullBarColor: fullBarColor, preset: selectPreset(preset, theme.palette), compact: compact });
    return (react_1.default.createElement(core_1.Grid, { container: true, spacing: 2, className: classes.progressContainer + " " + className, style: { marginTop: compact ? '-10px' : undefined } },
        react_1.default.createElement(core_1.Grid, { item: true, xs: 10, style: { maxWidth: '80%', flexBasis: '80%' } },
            react_1.default.createElement(core_1.Collapse, { in: value > 0, className: classes.collapse },
                react_1.default.createElement("div", { className: classes.topIndicator },
                    react_1.default.createElement("p", { title: valueLabel || 'Minutos Realizados' },
                        value,
                        valueSuffix)))),
        react_1.default.createElement(core_1.Grid, { item: true, xs: 2, style: { maxWidth: '20%', flexBasis: '20%' } }),
        react_1.default.createElement(core_1.Grid, { item: true, xs: 10, style: { maxWidth: '80%', flexBasis: '80%' } },
            react_1.default.createElement("div", { item: true, title: "Porcentagem " + percnt.toFixed(2) + "%", className: classes.progressBorder },
                react_1.default.createElement("div", { className: classes.progressBack },
                    react_1.default.createElement("div", { className: classes.progressBar }),
                    react_1.default.createElement("div", { className: classes.progressCover })))),
        react_1.default.createElement(core_1.Grid, { item: true, xs: 2, style: { maxWidth: '20%', flexBasis: '20%' } },
            react_1.default.createElement(core_1.Grow, { in: maxValue > 0 },
                react_1.default.createElement("div", { className: classes.rightIndicator },
                    react_1.default.createElement("p", { title: maxValueLabel || 'Minutos Previstos' },
                        maxValue,
                        maxValueSuffix))))));
}
exports.default = ProgressBar;
ProgressBar.propTypes = {
    value: prop_types_1.default.number,
    maxValue: prop_types_1.default.number,
    rightIndicatorColor: prop_types_1.default.string,
    topIndicatorColor: prop_types_1.default.string,
    borderColor: prop_types_1.default.string,
    barColor: prop_types_1.default.string,
    fullBarColor: prop_types_1.default.string,
    compact: prop_types_1.default.bool,
    preset: prop_types_1.default.oneOf(['default', 'primary', 'secondary', 'success', 'info', 'warning', 'danger', 'hot', 'cold'])
};
