import { TextFieldProps } from '@material-ui/core';
import { AutocompleteProps, AutocompleteChangeDetails } from '@material-ui/lab';
import * as React from 'react';

export interface AutoCompleteFavoritesProps extends Omit<AutocompleteProps<any, false, false, false>, 'getOptionLabel'>{
  /**
   * Valor do campo
   */
  value?: Array<number|string> | object, 
  /**
   * Props repassados ao componente Textfield
   */
  textFieldProps?: TextFieldProps, 
  /**
   * Callback fired when the value changes.
   *
   * @param event — The event source of the callback.
   *
   * @param value — The new value of the component.
   *
   * @param reason — One of "create-option", "select-option", "remove-option", "blur", "clear" ou "only-one-left".
   */
  onChange?: ((event: React.ChangeEvent<{}>, value: any, reason: 'create-option' | 'select-option' | 'remove-option' | 'blur' | 'clear' | 'only-one-left', details?: AutocompleteChangeDetails<any> | undefined) => void) | undefined,
  /**
   * Formata o valor que aparece na listagem
   * 
   * @param option - Opção sendo testada ou selecionada
   * 
   * @param lista - Lista com todas as opções
   */
  getOptionLabel?: ((option: Array<any>, lista: Array<Array<any>>) => string),
  /**
   * Utiliza um pgCode diferente do padrão
   */
  pgCode?: string,
  /**
   * Objeto de busca personalizado
   */
  busca?: string,
  /**
   * Objeto de resposta personalizado
   */
  resposta?: string,
  /**
   * Action personalizada
   */
  action?: string,
  /**
   * Label do campo
   */
  label?: string,
  /**
   * Parametros utilizados no metodo de busca
   */
  searchParams?: object,
  /**
   * Define a cor do loading
   */
  loadingColor?: string,
  /**
    * Parametro que define se terá paginação
    */
  favoritePage?: boolean,
  /**
   * Define a largura do listbox
   */
  widthListBox?: number|string,
  /**
   * Prop para utilizar grupo
   */
  group?: boolean,
  /**
    * Array contendo nome do grupo
    */
  groupName?: Array<string>,
  /**
    * Array contendo a cor do titulo no grupo
    */
  groupTitleColor?: Array<string>,
  /**
    * Array contendo o backgroundColor do header no grupo
    */
  groupHeaderColor?: Array<string>,
  /**
   * Prop que ativa a seleção automática da primeira opção da pesquisa
   */
  autoSelectFirstOption?: boolean,
  /**
   * Função para filtrar a lista de opções
   * 
   * @param valor - Valor setado no componente
   * @param item - Item a ser testado contra o valor
   * 
   * @returns {boolean} Retorna true se item de ser removido da lista
   */
  filterFunction?: (valor: any, item: any) => boolean,
}

declare function AutoCompleteFavorites(props: AutoCompleteFavoritesProps): JSX.Element

export default AutoCompleteFavorites;