"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getDroppedOrSelectedFiles = exports.getDataTransferFiles = void 0;
var DEFAULT_FILES_TO_IGNORE = [
    '.DS_Store',
    'Thumbs.db' // Windows indexing file
];
// map of common (mostly media types) mime types to use when the browser does not supply the mime type
var EXTENSION_TO_MIME_TYPE_MAP = {
    avi: 'video/avi',
    gif: 'image/gif',
    ico: 'image/x-icon',
    jpeg: 'image/jpeg',
    jpg: 'image/jpeg',
    mkv: 'video/x-matroska',
    mov: 'video/quicktime',
    mp4: 'video/mp4',
    pdf: 'application/pdf',
    png: 'image/png',
    zip: 'application/zip'
};
function shouldIgnoreFile(file) {
    return DEFAULT_FILES_TO_IGNORE.indexOf(file.name) >= 0;
}
function copyString(aString) {
    return (" " + aString).slice(1);
}
function traverseDirectory(entry) {
    var reader = entry.createReader();
    // Resolved when the entire directory is traversed
    return new Promise(function (resolveDirectory) {
        var iterationAttempts = [];
        var errorHandler = function () { };
        function readEntries() {
            // According to the FileSystem API spec, readEntries() must be called until
            // it calls the callback with an empty array.
            reader.readEntries(function (batchEntries) {
                if (!batchEntries.length) {
                    // Done iterating this particular directory
                    resolveDirectory(Promise.all(iterationAttempts));
                }
                else {
                    // Add a list of promises for each directory entry.  If the entry is itself
                    // a directory, then that promise won't resolve until it is fully traversed.
                    iterationAttempts.push(Promise.all(batchEntries.map(function (batchEntry) {
                        if (batchEntry.isDirectory) {
                            return traverseDirectory(batchEntry);
                        }
                        return Promise.resolve(batchEntry);
                    })));
                    // Try calling readEntries() again for the same dir, according to spec
                    readEntries();
                }
            }, errorHandler);
        }
        // initial call to recursive entry reader function
        readEntries();
    });
}
// package the file in an object that includes the fullPath from the file entry
// that would otherwise be lost
function packageFile(file, entry) {
    var _a;
    // let fileTypeOverride = '';
    // handle some browsers sometimes missing mime types for dropped files
    // const hasExtension = file.name && file.name.lastIndexOf('.') !== -1;
    // if (hasExtension && !file.type) {
    // const fileExtension = (file.name || '').split('.').pop();
    // fileTypeOverride = EXTENSION_TO_MIME_TYPE_MAP[fileExtension];
    // }
    return {
        fileObject: file,
        fullPath: entry ? (_a = copyString(entry.fullPath)) === null || _a === void 0 ? void 0 : _a.replace(file.name, '') : file.name,
        // lastModified: file.lastModified,
        // lastModifiedDate: file.lastModifiedDate,
        // name: file.name,
        // size: file.size,
        // type: file.type ? file.type : fileTypeOverride,
        // webkitRelativePath: file.webkitRelativePath
    };
}
function getFile(entry) {
    return new Promise(function (resolve) {
        entry.file(function (file) {
            resolve(packageFile(file, entry));
        });
    });
}
function handleFilePromises(promises, fileList) {
    return __awaiter(this, void 0, void 0, function () {
        var files;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, Promise.all(promises)];
                case 1:
                    files = _a.sent();
                    files.forEach(function (file) {
                        if (!shouldIgnoreFile(file)) {
                            fileList.push(file);
                        }
                    });
                    return [2 /*return*/, fileList];
            }
        });
    });
}
/**
 * @param {DataTransfer} dataTransfer
 */
function getDataTransferFiles(dataTransfer) {
    return __awaiter(this, void 0, void 0, function () {
        var dataTransferFiles, folderPromises, filePromises, flatten_1, fileEntries, flattenedEntries;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    dataTransferFiles = [];
                    folderPromises = [];
                    filePromises = [];
                    [].slice.call(dataTransfer.items).forEach(function (listItem) {
                        if (typeof listItem.webkitGetAsEntry === 'function') {
                            var entry = listItem.webkitGetAsEntry();
                            if (entry) {
                                if (entry.isDirectory) {
                                    folderPromises.push(traverseDirectory(entry));
                                }
                                else {
                                    filePromises.push(getFile(entry));
                                }
                            }
                        }
                        else {
                            dataTransferFiles.push(listItem);
                        }
                    });
                    if (!folderPromises.length) return [3 /*break*/, 3];
                    flatten_1 = function (array) { return array.reduce(function (a, b) { return a.concat(Array.isArray(b) ? flatten_1(b) : b); }, []); };
                    return [4 /*yield*/, Promise.all(folderPromises)];
                case 1:
                    fileEntries = _a.sent();
                    flattenedEntries = flatten_1(fileEntries);
                    // collect async promises to convert each fileEntry into a File object
                    flattenedEntries.forEach(function (fileEntry) {
                        filePromises.push(getFile(fileEntry));
                    });
                    return [4 /*yield*/, handleFilePromises(filePromises, dataTransferFiles)];
                case 2: return [2 /*return*/, _a.sent()];
                case 3:
                    if (filePromises.length) {
                        return [2 /*return*/, handleFilePromises(filePromises, dataTransferFiles)];
                    }
                    _a.label = 4;
                case 4: return [2 /*return*/, Promise.resolve(dataTransferFiles)];
            }
        });
    });
}
exports.getDataTransferFiles = getDataTransferFiles;
/**
 * This function should be called from both the onDrop event from your drag/drop
 * dropzone as well as from the HTML5 file selector input field onChange event
 * handler.  Pass the event object from the triggered event into this function.
 * Supports mix of files and folders dropped via drag/drop.
 *
 * Returns: an array of File objects, that includes all files within folders
 *   and subfolders of the dropped/selected items.
 *
 * @param {React.DragEvent<HTMLDivElement>} event
 */
function getDroppedOrSelectedFiles(event) {
    return __awaiter(this, void 0, void 0, function () {
        var dataTransfer, files, dragDropFileList, inputFieldFileList, fileList, i;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    dataTransfer = event.dataTransfer;
                    if (!(dataTransfer && dataTransfer.items)) return [3 /*break*/, 2];
                    return [4 /*yield*/, getDataTransferFiles(dataTransfer)];
                case 1: return [2 /*return*/, _a.sent()];
                case 2:
                    files = [];
                    dragDropFileList = dataTransfer && dataTransfer.files;
                    inputFieldFileList = event.target && event.target.files;
                    fileList = dragDropFileList || inputFieldFileList || [];
                    // convert the FileList to a simple array of File objects
                    for (i = 0; i < fileList.length; i++) {
                        files.push(packageFile(fileList[i]));
                    }
                    return [2 /*return*/, files];
            }
        });
    });
}
exports.getDroppedOrSelectedFiles = getDroppedOrSelectedFiles;
