import { CheckboxProps, OutlinedTextFieldProps, RadioProps, TypographyProps } from '@material-ui/core';
import { ClassNameMap } from '@material-ui/core/styles/withStyles';
import { AutocompleteProps } from '@material-ui/lab';
import { KeyboardDatePickerProps } from '@material-ui/pickers';
import * as React from 'react';

interface TextField extends OutlinedTextFieldProps {
  tp: 'text',
  name: string
}

interface TextSelection extends AutocompleteProps<any, false, false, false> {
  tp: 'text',
  variant: 'select',
  name: string
}

interface MultipleTextSelection extends AutocompleteProps<any, true, false, false> {
  tp: 'text',
  variant: 'select',
  multiple: true
  name: string
}

interface Label extends TypographyProps {
  tp: 'label',
  value: string
}

interface Data extends KeyboardDatePickerProps {
  tp: 'data',
  name: string
}

interface SelectionRadio extends RadioProps{
  tp: 'checkBox',
  multiple?: false,
  name: string,
  options: string[]
}

interface SelectionCheckBox extends CheckboxProps{
  tp: 'checkBox',
  multiple: true
  name: string,
  options: string[]
}

interface Coluna {
  span: number,
  className?: string,
  style?: React.CSSProperties, 
  obj: TextField | TextSelection | MultipleTextSelection | Label | Data | SelectionRadio | SelectionCheckBox
}

interface Linha {
  className?: string,
  colunas: Coluna[],
}

interface Documento {
  idDoc: number,
  nome: string,
  obrigatorio: boolean,
  className?: string,
  linhas: Linha[]
}

interface Fase {
  idFase: number,
  nome: string,
  grupos?: number[],
  usuarios?: number[],
  className?: string,
  documentos: Documento[]
}

interface JSONInterface {
  nome: string,
  classes: ClassNameMap,
  fases: Fase[],
}

export interface FormVisualizerProps {
  json: JSONInterface,
  initialValues?: object[],
  onChange?: (valores: object[]) => void
}

declare function FormVisualizer(props: FormVisualizerProps): JSX.Element

export default FormVisualizer;