import { OverrideProps, OverridableComponent } from '@material-ui/core/OverridableComponent';
import * as React from 'react';
import { IconType } from 'react-icons';

export interface StepperTypeMap<P ={}, D extends React.ElementType = 'div'> {
  props: P & {
    /**
     * Conteudo do Stepper
     */
    children?: React.ReactNode;
    /**
     * Array de passos para o stepper
     */
    steps?: [IconType, string, string?][];
    /**
     * Função chamada ao clicar em algum passo do stepper
     */
    onClick?: (index: number) => (event: React.MouseEvent<HTMLButtonElement, MouseEvent>) => void;
    /**
     * Função chamada ao clicar no botão de avançar
     */
    onNext?: (event: React.MouseEvent<HTMLButtonElement, MouseEvent>) => void;
    /**
     * Função chamada ao clicar no botão de voltar
     */
    onPrevious?: (event: React.MouseEvent<HTMLButtonElement, MouseEvent>) => void;
    /**
     * Função chamada ao clicar no botão de finalizar
     */
    onFinish?: (event: React.MouseEvent<HTMLButtonElement, MouseEvent>) => void;
    /**
     * Passo máximo permitido, utilizado para limitar ate onde o usuario pode ir
     */
    maxStep?: number;
    /**
     * Passo atual selecionado
     */
    activeStep?: number;
    /**
     * Array de booleans que mudam o passo para erro
     */
    errors?: boolean[];
    /**
     * Desabilita ações/botões
     */
    disableActions?: boolean;
    /**
     * Orientação do Stepper
     * default: horizontal
     * opções: horizontal ou vertical
     */
    orientation?: string;
    /**
     * Mostra o progresso percorrido
     */
    showProgress?: boolean;
  };
  defaultComponent: D;
  classKey: StepperClassKey;
}

declare const Stepper: OverridableComponent<StepperTypeMap>;

export type StepperClassKey = 'root' | 'header' | 'caption' | 'selectedCaption' | 'stepper' | 'icon' | 'mobileDivider'

export type StepperProps<D extends React.ElementType = StepperTypeMap['defaultComponent'], P = {}> = OverrideProps<StepperTypeMap<P, D>, D>

export default Stepper;