import { TextFieldProps } from '@material-ui/core';
import { AutocompleteProps, AutocompleteChangeDetails } from '@material-ui/lab';
import * as React from 'react';

export interface UtilAutoCompleteProps extends Omit<AutocompleteProps<any, false, false, false>, 'getOptionLabel'>{
  /**
   * Tipo de autocomplete
   */
  type: 'beneficiarios' | 'empresas' | 'gruposUnimed' |'estados' | 'cidades' | 'unimeds' | 'prestadores', 
  /**
   * Valor do campo
   */
  value?: Array<number|string> | object, 
  /**
   * Props repassados ao componente Textfield
   */
  textFieldProps?: TextFieldProps, 
  /**
   * Callback fired when the value changes.
   *
   * @param event — The event source of the callback.
   *
   * @param value — The new value of the component.
   *
   * @param reason — One of "create-option", "select-option", "remove-option", "blur", "clear" ou "only-one-left".
   */
  onChange?: ((event: React.ChangeEvent<{}>, value: any, reason: 'create-option' | 'select-option' | 'remove-option' | 'blur' | 'clear' | 'only-one-left', details?: AutocompleteChangeDetails<any> | undefined) => void) | undefined, 
  /**
   * Trazer itens excluidos
   */
  excluidos?: boolean,
  /**
   * Filtra por id empresa
   */
  idEmpresa?: number,
  /**
   * Filtra por cd estado 
   */
  cdEstado?: number,
  /**
   * Filtra cd unimed
   */
  cdUnimed?:number,
  /**
   * Formata o valor que aparece na listagem
   * 
   * @param option - Opção sendo testada ou selecionada
   * 
   * @param lista - Lista com todas as opções
   */
  getOptionLabel?: ((option: Array<any>, lista: Array<Array<any>>) => string),
  /**
   * Utiliza um pgCode diferente do padrão
   */
  pgCode?: string,
  /**
   * Objeto de busca personalizado
   */
  busca?: string,
  /**
   * Objeto de resposta personalizado
   */
  resposta?: string,
  /**
   * Action personalizada
   */
  action?: string,
  /**
   * Label do campo
   */
  label?: string,
  /**
   * Parametros utilizados no metodo de busca
   */
  searchParams?: object,
  /**
   * Disponível para leitura apenas
   */
  readOnly?: boolean,
  /**
   * Função para filtrar a lista de opções
   * 
   * @param valor - Valor setado no componente
   * @param item - Item a ser testado contra o valor
   * 
   * @returns {boolean} Retorna true se item de ser removido da lista
   */
  filterFunction?: (valor: any, item: any) => boolean,
  /**
   * Força o componente a atualizar quando o valor dele é alterado por fora
   */
  forceUpdate?: boolean
}

declare function UtilAutoComplete(props: UtilAutoCompleteProps): JSX.Element

export default UtilAutoComplete;