import { BasicModalProps } from './components';
import { Store } from 'redux';

export type argousConfig = {
  ssl: boolean,
  ip: string,
  ipImagePosts?: string,
  codigoCliente: string,
  path?: string
}

export interface UsuarioLogado {
  habGrupo?: boolean,
  habChat?: boolean,
  habProcesso?: boolean,
  habFavorito?: boolean,
  habNotificacao?: boolean,
  email?: string,
  funcao?: string,
  idFornecedorInterno?: number,
  idUsuario?: number,
  nome?: string,
  ramal?: string,
  publTL?: boolean,
  imagem?: string,
  imagemCapa?: string
}

interface PromiseOptions extends BasicModalProps{
  resolve: (value: boolean | PromiseLike<boolean>) => void, 
  reject: (reason?: any) => void
}
export interface ArgousSystemType {
  WSHandler: { onMessage: (params: any) => void },
  headerState: { value?: 'min' },
  pessoasChatOpen: { value: boolean },
  isChatMenuOpen: { value: boolean },
  janelaChatOpen?: {
    idUsuario: number,
    idSala: number,
    titulo: string,
    subtitulo: string,
    online: boolean,
    msgsNaoLidas: number
  },
  systemTopAlert?: Record<string, any>,
  openModal?: PromiseOptions,
  [other: string]: Record<string, any> | undefined
}

export interface FieldsType {
  argousSystem: ArgousSystemType,
  [program: string]: Record<string, any> | undefined
}

export interface ArgousType {
  block: boolean,
  currentProgram: string,
  favorites: any[],
  fields: FieldsType,
  initialPage: string,
  loading: boolean,
  processos: any[],
  sidebar: boolean
  usuarioLogado: UsuarioLogado,
  vldPsw: boolean,
  rightSidebar: boolean,
  addProgramWidget: any,
  programAnnotation: object
}
export interface StoreState {
  argous: ArgousType
}

export type ArgousStore = Store<StoreState, ActionType>

interface setUsuarioLogado {
  type: 'setUsuarioLogado',
  usuarioLogado: UsuarioLogado
} 

interface setLoading {
  type: 'setLoading',
  loading: boolean 
} 

interface setProcesso {
  type: 'setProcesso',
  processo: any 
} 

interface setInitialPage {
  type: 'setInitialPage',
  initialPage: string 
} 

interface setCurrentProgram {
  type: 'setCurrentProgram',
  currentProgram: string 
} 

interface setCurrentField {
  type: 'setCurrentField',
  program: string,
  field: string,
  value: any   
} 

interface clearAllFields {
  type: 'clearAllFields'
} 

interface setFields {
  type: 'setFields',
  fields: any 
} 

interface setFavorites {
  type: 'setFavorites',
  favorites: any[] 
} 

interface setBlockUser {
  type: 'setBlockUser',
  block: boolean 
} 

interface setNotificacao {
  type: 'setNotificacao',
  notificacao: any 
} 

interface setSidebar {
  type: 'setSidebar',
  sidebar: boolean 
}

interface setRightSidebar {
  type: 'setRightSidebar',
  rightSidebar: boolean 
}

interface setMensagensNaoLidas {
  type: 'setMensagensNaoLidas',
  mensagensNaoLidas: number 
}

interface setAddProgramWidget {
  type: 'setAddProgramWidget',
  addProgramWidget: any
}

interface setVldPsw {
  type: 'setVldPsw',
  vldPsw: boolean 
}

interface setProgramAnnotation { 
  type: 'setProgramAnnotation', 
  programAnnotation: object 
}


export type ActionType = setUsuarioLogado|setLoading|setProcesso|setInitialPage|setCurrentProgram|setCurrentField|clearAllFields|setFields|setFavorites|setBlockUser|setNotificacao|setSidebar|setMensagensNaoLidas|setVldPsw|setRightSidebar|setAddProgramWidget|setProgramAnnotation
