import { ActionType, ArgousType } from '../types';
import { ChartOptions } from 'chart.js';
import { BasicModalProps } from 'lib/components';
import React from 'react';
export declare const widthToCenterVertically: (content: number) => number;
export declare const getCookie: (cName: string) => string;
export declare const addExtraZero: (vl: number | string) => string | number;
export declare const formataNome: (value: string) => string;
export declare function downloadBase64(base64: string, fileName: string, fileExtension: string): void;
/**
 * Função faz o download de qualquer url
 *
 * @param {string} url - Url a ser baixada
 * @param {string} fileName - Nome do arquivo baixado
 * @param {string=} fileExtension - Extenção do arquivo baixado
 *
 * @returns {void}
 */
export declare function downloadFromUrl(url: string, fileName: string, fileExtension?: string): Promise<void>;
export declare const printPdf: (URL?: string) => Promise<void>;
export declare const getRandomColor: () => string;
export declare const getInitials: (nome?: string) => string;
export declare const getNumeroFromInitials: (initials: string, length: number) => number;
export declare const usePrevious: (value: any) => undefined;
/**
 * Igual o useEffect, mas não ativa quando o componente monta
 *
 * @param {React.EffectCallback} effect - Função imperativa que pode retornar uma função de limpeza
 * @param {React.DependencyList} dependencies - Dependencias que quando atualizadas rodarão o efeito
 *
 * @returns {void} Sem retorno
 */
export declare function useUpdateEffect(effect: React.EffectCallback, dependencies: React.DependencyList): void;
export declare function formataDinheiro(n: number): string;
export declare const sanitizaHTML: (html?: string) => string;
/**
 * Format bytes as human-readable text.
 *
 * @param {number} bytes - Number of bytes.
 * @param {boolean} si - True to use metric (SI) units, aka powers of 1000. False to use binary (IEC) units, aka powers of 1024.
 * @param {number} dp - Number of decimal places to display.
 *
 * @returns {string} Formatted string.
 */
export declare const humanFileSize: (bytes: number, si?: boolean, dp?: number) => string;
export declare const randomColor: () => string;
/**
 * Hook para padronizar o tema dos gráficos
 *
 * @param {ChartOptions} props - Props customizados
 * @returns {ChartOptions} - retorno com tema ajustado
 */
export declare const useGraph: ({ title, legend: { labels, ...legend }, plugins: { datalabels, ...plugins }, scales: { yAxes: [{ ticks: yticks, gridLines: ygridLines, ...yAxes }, ...y], xAxes: [{ ticks: xticks, gridLines: xgridLines, ...xAxes }, ...x], ...scales }, ...props }?: ChartOptions) => ChartOptions;
export declare const useModal: () => import("lib/components").ModalProviderContextProps;
export declare const useStore: () => ArgousType;
export declare const reduxDispatch: (action: ActionType) => void;
export declare const formataTelefone: (value: string) => string;
export declare const useWebSocket: () => {};
export declare const formataCpf: (value: string) => string;
export declare const numbersOnly: (value: string, type: string) => string;
export declare const currencyMask: (value: string | number) => string;
export declare const toFloat: (value: string) => number | undefined;
export declare const copyToClipboard: (text: string) => void;
export declare const confirm: (props: BasicModalProps | string) => Promise<boolean>;
export declare const storageMeasures: {
    byte: string;
    kilobyte: string;
    megabyte: string;
    gigabyte: string;
    terabyte: string;
    petabyte: string;
    exabyte: string;
};
export declare const bytesTranslator: (bytes?: number, decimals?: number, binary?: boolean) => object;
export declare const formatCpfCnpj: (value: (string | number)) => string;
export declare function dataUrlToBlob(dataUrl: string): Blob | undefined;
export declare function cpfTest(cpf: (string | number)): boolean;
interface PrintScreenParameters {
    /**
     * Event
     */
    event?: Event;
    /**
     * Element id
     */
    id: string;
    /**
     * Auto download
     */
    download: boolean;
}
export declare const printScreen: ({ event, id, download }: PrintScreenParameters) => Promise<object>;
interface GeraLogInterface {
    (
    /**
     * Id do documento
     */
    idDocumento: number, 
    /**
     * Descrição
     */
    descricao: string, 
    /**
     * Tipo de ação
     *
     * 1 - Upload
     * 2 - Alteração
     * 3 - Consulta
     */
    tpAcao: string, 
    /**
     * Token do sistema
     */
    tokenSistema?: string): Promise<void>;
}
export declare const geraLog: GeraLogInterface;
export declare const printUrl: (url: string) => Promise<void>;
declare global {
    interface Window {
        clipboardData: any;
    }
}
export declare function stopPrntScr(e: KeyboardEvent): boolean;
export {};
