"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseXml = exports.xmlStringToJson = void 0;
var xmlStringToJson = function (xml, prefixToRemove) {
    if (prefixToRemove === void 0) { prefixToRemove = ''; }
    return JSON.parse(xml2json(exports.parseXml(xml)).replaceAll(prefixToRemove, ''));
};
exports.xmlStringToJson = xmlStringToJson;
var parseXml = function (xml) {
    return (new DOMParser()).parseFromString(xml, 'text/xml');
};
exports.parseXml = parseXml;
var xml2json = function (xml) {
    var toObj = function (xml) {
        var o = {};
        if (xml.nodeType === 1) { // element node ..
            if (xml.firstChild) { // element has child nodes ..
                var textChild = 0, cdataChild = 0, hasElementChild = false;
                for (var n = xml.firstChild; n; n = n.nextSibling) {
                    if (n.nodeType === 1)
                        hasElementChild = true;
                    else if (n.nodeType === 3 && n.nodeValue.match(/[^ \f\n\r\t\v]/))
                        textChild++; // non-whitespace text
                    else if (n.nodeType === 4)
                        cdataChild++; // cdata section node
                }
                if (hasElementChild) {
                    if (textChild < 2 && cdataChild < 2) { // structured element with evtl. a single text or/and cdata node ..
                        removeWhite(xml);
                        for (var n = xml.firstChild; n; n = n.nextSibling) {
                            if (n.nodeType === 3)
                                o['#text'] = escape(n.nodeValue); // text node
                            else if (n.nodeType === 4)
                                o['#cdata'] = escape(n.nodeValue); // cdata node
                            else if (o[n.nodeName]) { // multiple occurence of element ..
                                if (o[n.nodeName] instanceof Array)
                                    o[n.nodeName][o[n.nodeName].length] = toObj(n);
                                else
                                    o[n.nodeName] = [o[n.nodeName], toObj(n)];
                            }
                            else
                                o[n.nodeName] = toObj(n); // first occurence of element..
                        }
                    }
                    else if (!xml.attributes.length)
                        o = escape(innerXml(xml));
                    else
                        o['#text'] = escape(innerXml(xml));
                }
                else if (textChild) { // pure text
                    if (!xml.attributes.length)
                        o = escape(innerXml(xml));
                    else
                        o['#text'] = escape(innerXml(xml));
                }
                else if (cdataChild) { // cdata
                    if (cdataChild > 1)
                        o = escape(innerXml(xml));
                    else
                        for (var n = xml.firstChild; n; n = n.nextSibling)
                            o['#cdata'] = escape(n.nodeValue);
                }
            }
            if (!xml.attributes.length && !xml.firstChild)
                o = null;
        }
        else if (xml.nodeType === 9) { // document.node
            o = toObj(xml.documentElement);
        }
        else
            console.error('unhandled node type: ' + xml.nodeType);
        return o;
    };
    var toJson = function (o, name, ind) {
        var json = name ? ('"' + name + '"') : '';
        if (o instanceof Array) {
            for (var i = 0, n = o.length; i < n; i++)
                o[i] = toJson(o[i], '', ind + '\t');
            json += (name ? ':[' : '[') + (o.length > 1 ? ('\n' + ind + '\t' + o.join(',\n' + ind + '\t') + '\n' + ind) : o.join('')) + ']';
        }
        else if (!o)
            json += (name && ':') + 'null';
        else if (typeof (o) == 'object') {
            var arr = [];
            for (var m in o)
                if (Object.prototype.hasOwnProperty.call(o, m))
                    arr[arr.length] = toJson(o[m], m, ind + '\t');
            json += (name ? ':{' : '{') + (arr.length > 1 ? ('\n' + ind + '\t' + arr.join(',\n' + ind + '\t') + '\n' + ind) : arr.join('')) + '}';
        }
        else if (typeof (o) == 'string')
            json += (name && ':') + '"' + o.toString() + '"';
        else
            json += (name && ':') + o.toString();
        return json;
    };
    var innerXml = function (node) {
        var s = '';
        if ('innerHTML' in node)
            s = node.innerHTML;
        else {
            var asXml_1 = function (n) {
                var s = '';
                if (n.nodeType === 1) {
                    s += '<' + n.nodeName;
                    for (var i = 0; i < n.attributes.length; i++)
                        s += ' ' + n.attributes[i].nodeName + '="' + (n.attributes[i].nodeValue || '').toString() + '"';
                    if (n.firstChild) {
                        s += '>';
                        for (var c = n.firstChild; c; c = c.nextSibling)
                            s += asXml_1(c);
                        s += '</' + n.nodeName + '>';
                    }
                    else
                        s += '/>';
                }
                else if (n.nodeType === 3)
                    s += n.nodeValue;
                else if (n.nodeType === 4)
                    s += '<![CDATA[' + n.nodeValue + ']]>';
                return s;
            };
            for (var c = node.firstChild; c; c = c.nextSibling)
                s += asXml_1(c);
        }
        return s;
    };
    var escape = function (txt) {
        return txt.replace(/[\\]/g, '\\\\')
            .replace(/["]/g, '\\"')
            .replace(/[\n]/g, '\\n')
            .replace(/[\r]/g, '\\r');
    };
    var removeWhite = function (e) {
        e.normalize();
        for (var n = e.firstChild; n;) {
            if (n.nodeType === 3) { // text node
                if (!n.nodeValue.match(/[^ \f\n\r\t\v]/)) { // pure whitespace text node
                    var nxt = n.nextSibling;
                    e.removeChild(n);
                    n = nxt;
                }
                else
                    n = n.nextSibling;
            }
            else if (n.nodeType === 1) { // element node
                removeWhite(n);
                n = n.nextSibling;
            }
            else
                n = n.nextSibling; // any other node
        }
        return e;
    };
    if (xml.nodeType === 9)
        xml = xml.documentElement; // document node
    var json = toJson(toObj(removeWhite(xml)), xml.nodeName, '\t');
    return '{\n' + json.replace(/\t|\n/g, '') + '\n}';
};
