import { connectArgousWS, getArgousWS, enviaMensagem, disconnect } from './ws';
import * as argous from '../actions';
import { getField, getCurrentProgram } from '../fieldsRedux';
import { sendNotification } from '../utils/api';
import PropTypes from 'prop-types';
import { Component } from 'react';
import { connect } from 'react-redux';

class WebSocketComponent extends Component {

  componentDidMount() {
    window.addEventListener('focus', () => {
      if (this.props.usuarioLogado.nome) {
        this.connect();
      }
    });
  }

  componentDidUpdate(previousProps, previousState) {
    if (!previousProps.usuarioLogado.nome && this.props.usuarioLogado.nome) {
      this.connect();
    }

    if (previousProps.usuarioLogado.nome && !this.props.usuarioLogado.nome) {
      disconnect();
    }
  }

  componentWillUnmount() {
    disconnect();
  }

  connect = () => {
    connectArgousWS();

    const argousWS = getArgousWS();

    argousWS.onopen = (event) => {
      enviaMensagem({ trn: 1 });
    };

    argousWS.onmessage = (event) => {
      const res = JSON.parse(event.data);
      this.resolveTrn(res);
    };
  }

  resolveTrn = (res) => {

    switch (res.trn) {
      case 3:
        this.resolveProcesso(res);
        break;
      case 4:
        this.resolveNotificacao(res);
        break;
      case 5:
        this.resolveMessage(res);
        break;
      default:
    }
  }

  resolveProcesso = (res) => {
    const { setProcesso, processos } = this.props;
    let novoProcesso = true;

    let pAux = processos.map(i => {
      if (i.id === res.id) {
        novoProcesso = false;
        return res;
      } 
      return i;
      
    });

    const pLength = pAux.length;

    if (novoProcesso && pLength !== 0) {
      pAux = pAux.concat(res);
    }

    setProcesso(pLength === 0
      ? [res]
      : pAux);
  }

  resolveNotificacao = (res) => {
    try {
      const item = JSON.parse(res.notificacao[4]);
      if (item.link)
        sendNotification({ id: res.notificacao[0], mensagem: item.mensagem, title: res.notificacao[3], icon: res.notificacao[1], link: item.link, dsLabel: item.dsLabel });
      else throw new Error();
    } catch (error) {
      sendNotification(res.notificacao[4], res.notificacao[3], res.notificacao[1]);
    } finally {
      const { setNotificacao } = this.props;
      setNotificacao(res.notificacao);
    }
  }

  resolveMessage = (res) => {
    const currentProgram = getCurrentProgram();

    if (res.pgCode === 'ChatContext') {
      this.resolveChat(res);
      return;
    }

    if (currentProgram && res.pgCode === currentProgram) {
      const onMessage = getField('WSHandler').onMessage;
      onMessage && onMessage(res);
    } else {
      const onMessage = getField('WSHandler', 'onMessage', res.pgCode);
      onMessage && onMessage(res);
    }
  }

  resolveChat = (res) => {
    const onMessage = getField('WSHandler', 'onMessage', 'argousSystem');
    onMessage && onMessage(res);
  }

  render() {
    return null;
  }
}

WebSocketComponent.propTypes = {
  usuarioLogado: PropTypes.object.isRequired,
  setProcesso: PropTypes.func.isRequired,
  processos: PropTypes.array
};

const mapStateToProps = (state) => {
  return { usuarioLogado: state.argous.usuarioLogado, processos: state.argous.processos };
};

const mapDispatchToProps = (dispatch) => {
  return {
    setNotificacao: (data) => dispatch(argous.setNotificacao(data)),
    setProcesso: (data) => dispatch(argous.setProcesso(data))
  };
};

const WebSocket = connect(mapStateToProps, mapDispatchToProps)(WebSocketComponent);

export default WebSocket;
