import { getCookie } from '../utils';
import { getConfig } from '../utils/config';

export const server = () => `://${getConfig()[/^[a-z]+.+\.com\.br$/.test(window.location.host) ? 'ip' : 'ip']}/${getConfig().path ? getConfig().path : 'jUniIII'}/`;

export const api = () => `${getConfig().ssl ? 'https' : 'http'}${server()}argous/`;

export const headers = (token) => ({ lgTkn: (token) ? token : getCookie('lt') });

/**
 * @deprecated
 * 
 * @param {string} request - Metodo chamado 
 * @param {*} data - Dados Enviados
 * 
 * @returns {*} retorno
 */
export const fetchToArgous = async (request, data) => {
  const res = await fetch(`${api()}${request}.rslvr`, {
    method: 'POST',
    body: JSON.stringify(data),
    headers: headers()
  });
  return await res.json();
};

export const login = async (data) => {
  const res = await fetch(`${api()}aglogin.rslvr`, {
    method: 'POST',
    body: JSON.stringify(data)
  });
  return await res.json();
};

export const logout = async () => {
  const res = await fetch(`${api()}aglogout.rslvr`, {
    method: 'POST',
    headers: headers()
  });
  return await res.json();
};

export const validaToken = async () => {
  const res = await fetch(`${api()}agvldtkn.rslvr`, {
    method: 'POST',
    headers: headers()
  });
  return await res.json();
};

export const getMenu = async () => {
  const res = await fetch(`${api()}agetmenu.rslvr`, {
    method: 'POST',
    headers: headers()
  });
  return await res.json();
};

export const getPostsTimeline = async (data) => {
  const res = await fetch(`${api()}agtimeline.rslvr`, {
    method: 'POST',
    body: JSON.stringify(data),
    headers: headers()
  });
  return await res.json();
};

export const addCommentPost = async (data) => {
  const res = await fetch(`${api()}agpostcomment.rslvr`, {
    method: 'POST',
    body: JSON.stringify(data),
    headers: headers()
  });
  return await res.json();
};

export const removeCommentPost = async (data) => {
  const res = await fetch(`${api()}agrmcomentario.rslvr`, {
    method: 'POST',
    body: JSON.stringify(data),
    headers: headers()
  });
  return await res.json();
};

export const getPostPublicacao = async (data) => {
  const res = await fetch(`${api()}aggetpost.rslvr`, {
    method: 'POST',
    body: JSON.stringify(data),
    headers: headers()
  });
  return await res.json();
};

export const getProgram = async (data) => {
  const res = await fetch(`${api()}agpgm.rslvr`, {
    method: 'POST',
    body: JSON.stringify({ pgCode: data }),
    headers: headers()
  });
  return await res.json();
};

/**
 * @deprecated
 * 
 * @param {*} data - Dados
 * 
 * @returns {object} retorno 
 */
export const agGet = async (data) => {
  const res = await fetch(`${api()}agget.rslvr`, {
    method: 'POST',
    body: JSON.stringify(data),
    headers: headers()
  });
  return await res.json();
};

/**
 * @deprecated
 * 
 * @param {*} data - Dados
 * 
 * @returns {object} retorno 
 */
export const agSet = async (data) => {
  const res = await fetch(`${api()}agset.rslvr`, {
    method: 'POST',
    body: JSON.stringify(data),
    headers: headers()
  });
  return await res.json();
};

export const action = async (data, token) => {
  const res = await fetch(`${api()}agaction.rslvr`, {
    method: 'POST',
    body: JSON.stringify(data),
    headers: headers(token)
  });
  return await res.json();
};

/**
 * @deprecated
 * 
 * @param {*} data - Dados
 * 
 * @returns {object} retorno 
 */
export const method = async (data) => {
  const res = await fetch(`${api()}agmethod.rslvr`, {
    method: 'POST',
    body: JSON.stringify(data),
    headers: headers()
  });
  return await res.json();
};

export const getFavorites = async () => {
  const res = await fetch(`${api()}agfvrts.rslvr`, {
    method: 'POST',
    headers: headers()
  });
  return await res.json();
};

export const toggleFavorite = async (data) => {
  const res = await fetch(`${api()}agchfvrts.rslvr`, {
    method: 'POST',
    body: JSON.stringify(data),
    headers: headers()
  });
  return await res.json();
};

export const callProgram = async (data) => {
  const res = await fetch(`${api()}agcallpgm.rslvr`, {
    method: 'POST',
    body: JSON.stringify(data),
    headers: headers()
  });
  return await res.json();
};

export const getNotificacoes = async (data) => {
  const res = await fetch(`${api()}agnotfctn.rslvr`, {
    method: 'POST',
    body: JSON.stringify(data),
    headers: headers()
  });
  return await res.json();
};

export const readNotificacoes = async (data) => {
  const res = await fetch(`${api()}agnotvista.rslvr`, {
    method: 'POST',
    body: JSON.stringify(data),
    headers: headers()
  });
  return await res.json();
};

export const readNotificacao = async (data) => {
  const res = await fetch(`${api()}agnotlida.rslvr`, {
    method: 'POST',
    body: JSON.stringify(data),
    headers: headers()
  });
  return await res.json();
};

export const recebeThread = async (data) => {
  const res = await fetch(`${api()}agthreads.rslvr`, {
    method: 'POST',
    body: JSON.stringify(data),
    headers: headers()
  });
  return await res.json();
};

export const recebeInfoThread = async (data) => {
  const res = await fetch(`${api()}aginfothread.rslvr`, {
    method: 'POST',
    body: JSON.stringify(data),
    headers: headers()
  });
  return await res.json();
};

export const recebeThreadsUsuario = async (data) => {
  const res = await fetch(`${api()}agthreadsusuario.rslvr`, {
    method: 'POST',
    body: JSON.stringify(data),
    headers: headers()
  });
  return await res.json();
};

export const recebeInfoLoginsAtivos = async () => {
  const res = await fetch(`${api()}agloginsativos.rslvr`, {
    method: 'POST',
    headers: headers()
  });
  return await res.json();
};

export const recebeInfoEstatisticas = async () => {
  const res = await fetch(`${api()}agstatistics.rslvr`, {
    method: 'POST',
    headers: headers()
  });
  return await res.json();
};

export const toggleLikePost = async (data) => {
  const res = await fetch(`${api()}aglike.rslvr`, {
    method: 'POST',
    body: JSON.stringify(data),
    headers: headers()
  });
  return await res.json();
};

export const publicaTimeline = async (data) => {
  const res = await fetch(`${api()}agpost.rslvr`, {
    method: 'POST',
    body: JSON.stringify(data),
    headers: headers()
  });
  return await res.json();
};

export const removePostTimeLine = async (data) => {
  const res = await fetch(`${api()}agerasepost.rslvr`, {
    method: 'POST',
    body: JSON.stringify(data),
    headers: headers()
  });
  return await res.json();
};

export const fixTimelinePost = async (data) => {
  const res = await fetch(`${api()}agfixapost.rslvr`, {
    method: 'POST',
    body: JSON.stringify(data),
    headers: headers()
  });
  return await res.json();
};

export const enviaCompartilhamentoDePrograma = async (data) => {
  const res = await fetch(`${api()}agsharepost.rslvr`, {
    method: 'POST',
    body: JSON.stringify(data),
    headers: headers()
  });
  return await res.json();
};

export const inicFileUpload = async (data) => { //inicia o processo de envio, retornando um id de protocolo e o tamanho de particionamento
  const res = await fetch(`${api()}getProtocol.filepart`, {
    method: 'POST',
    body: JSON.stringify(data),
    headers: headers()
  });
  return await res.json();
};

export const sendFilePart = async (data) => { // Envia um pedaco do do arquivo, retorna se foi sucedido ou nao
  const res = await fetch(`${api()}setFilePart.filepart`, {
    method: 'POST',
    body: JSON.stringify(data),
    headers: headers()
  });
  return await res.json();
};

export const endFileUpload = async (data) => { //Finaliza o processo de envio de arquivo 
  const res = await fetch(`${api()}setEndFileUpload.filepart`, {
    method: 'POST',
    body: JSON.stringify(data),
    headers: headers()
  });
  return await res.json();
};

export const cancelFileUpload = async (data) => { //Cancela o envio do arquivo
  const res = await fetch(`${api()}setFileUploadCancel.filepart`, {
    method: 'POST',
    body: JSON.stringify(data),
    headers: headers()
  });
  return await res.json();
};

export const getFileInfo = async (data) => { //Busca informaçoes sobre o estado do arquivo
  const res = await fetch(`${api()}getFileUploadInfo.filepart`, {
    method: 'POST',
    body: JSON.stringify(data),
    headers: headers()
  });
  return await res.json();
};

export const searchUsers = async (data) => {
  const res = await fetch(`${api()}aggetlistausuarios.rslvr`, {
    method: 'POST',
    body: JSON.stringify(data),
    headers: headers()
  });
  return await res.json();
};

export const getDocsUrl = async (data) => {
  const res = await fetch(`${api()}agdocsurl.rslvr`, {
    method: 'POST',
    body: JSON.stringify(data),
    headers: headers()
  });
  return await res.json();
};

export const prepareFile = async (file) => {
  const { status, docsUrl } = await getDocsUrl();
  if (!status) return;
  const formData = new FormData();
  formData.append('blob', file);
  const res = await fetch(`${docsUrl}/argous/docs/upload.agdoc`, {
    method: 'POST',
    headers: headers(),
    body: formData
  });
  return { ...await res.json(), 
    lastModified: file.lastModified,
    lastModifiedDate: file.lastModifiedDate,
    name: file.name,
    size: file.size,
    type: file.type,
    webkitRelativePath: file.webkitRelativePath,
    arrayBuffer: await file.arrayBuffer()
  };
};

export const createDocument = async ({
  idUnidade, cdUnidade, idDocPai, path,
  dsKeyWords, dtValidade, idTipoDocumento,
  FileName, title, idRecebimento,
  tpPrivacidade, description, dsClassificacao,
  dsCategoria, dsSubCategoria, snSensivel
}) => {
  const { status, docsUrl } = await getDocsUrl();
  if (!status) return;
  return (await fetch(`${docsUrl}argous/docs/upload.agdoc`, {
    method: 'POST',
    headers: headers(),
    body: JSON.stringify({
      idUnidade, cdUnidade, idDocPai, path,
      dsKeyWords, dtValidade, idTipoDocumento,
      FileName, title, idRecebimento,
      tpPrivacidade, description, dsClassificacao,
      dsCategoria, dsSubCategoria, snSensivel
    })
  })).json();
};

export const getFileFromLink = async ({ URL: link = '', token = '' } = {}) => {
  if (!link) return '';
  const file = await (await fetch(link, {
    method: 'post',
    headers: headers(token)
  })).blob();
  return URL.createObjectURL(file);
};

export const getLink = async (idDocumento, nrRevisao, timeout) => {
  if (!idDocumento) return '';
  return await (await fetch(`${api()}docs/getURL.agdoc`, {
    method: 'post',
    headers: headers(),
    body: JSON.stringify({ idDocumento, nrRevisao, timeout })
  })).json();
};

export const scanDocument = async (idDocumento, nrRevisao) => {
  const { status, docsUrl } = await getDocsUrl();
  if (!status) return;
  const res = await fetch(`${docsUrl}argous/docs/scan.agdoc`, {
    method: 'post',
    headers: headers(),
    body: JSON.stringify({ idDocumento, nrRevisao })
  });
  return await res.json();
};

export const getWidgets = async (data, abortController) => {
  const res = await fetch(`${api()}agetwidgets.rslvr`, {
    method: 'post',
    headers: headers(),
    body: JSON.stringify(data),
    signal: abortController?.signal
  });
  return await res.json();
};

export const getWidgetInfo = async (pgCode) => {
  const res = await fetch(`${api()}agetwidgetinfo.rslvr`, {
    method: 'post',
    headers: headers(),
    body: JSON.stringify({ pgCode })
  });
  return await res.json();
};

export const sendWidgets = async (widgets) => {
  const res = await fetch(`${api()}agsendwidgets.rslvr`, {
    method: 'post',
    headers: headers(),
    body: JSON.stringify({ widgets })
  });
  return await res.json();
};

export const getPublicDoc = async (data) => {
  const res = await fetch(`${api()}agpublicdoc.rslvr`, {
    method: 'post',
    headers: headers(),
    body: JSON.stringify(data)
  });
  return await res.json();
};

export const getProgramAnnotations = async (data, token) => {
  const res = await fetch(`${api()}aggetpgannotation.rslvr`, {
    method: 'post',
    headers: headers(token),
    body: JSON.stringify(data)
  });
  return await res.json();
};

export const saveProgramAnnotation = async (data, token) => {
  const res = await fetch(`${api()}agsavepgannotation.rslvr`, {
    method: 'post',
    headers: headers(token),
    body: JSON.stringify(data)
  });
  return await res.json();
};

export const sendProgramAnnotation = async (data, token) => {
  const res = await fetch(`${api()}agsendpgannotation.rslvr`, {
    method: 'post',
    headers: headers(token),
    body: JSON.stringify(data)
  });
  return await res.json();
};