import { fetchToArgous } from '../../api';
import { getNumeroFromInitials, getInitials } from '../../utils';
import { Avatar, makeStyles, colors } from '@material-ui/core';
import PropTypes from 'prop-types';
import React, { useEffect, useState } from 'react';

const useStyles = makeStyles(theme => ({
  orange: {
    color: theme.palette.getContrastText(colors.deepOrange[500]),
    backgroundColor: colors.deepOrange[500],
  },
  purple: {
    color: theme.palette.getContrastText(colors.deepPurple[500]),
    backgroundColor: colors.deepPurple[500],
  },
  blue: {
    color: theme.palette.getContrastText(colors.blue[500]),
    backgroundColor: colors.blue[500],
  },
  red: {
    color: theme.palette.getContrastText(colors.red[500]),
    backgroundColor: colors.red[500],
  },
  rosa: {
    color: theme.palette.getContrastText(colors.pink[500]),
    backgroundColor: colors.pink[500],
  },
  cinza: {
    color: theme.palette.getContrastText(colors.grey[500]),
    backgroundColor: colors.grey[500],
  },
  green: {
    color: theme.palette.getContrastText(colors.green[500]),
    backgroundColor: colors.green[500],
  },
  amarelo: {
    color: theme.palette.getContrastText(colors.yellow[500]),
    backgroundColor: colors.yellow[500],
  },
  marrom: {
    color: theme.palette.getContrastText(colors.brown[500]),
    backgroundColor: colors.brown[500],
  },
  preto: {
    color: theme.palette.getContrastText(colors.grey['A400']),
    backgroundColor: colors.grey['A400'],
  },
  img: {
    border: '2px solid #FFF'
  },
}));

const ArgousAvatar = React.forwardRef(({ nome, src, className, idUsuario, forwardedBlockSearch, ...props }, ref) => {
  const classes = useStyles();
  const cores = [classes.orange, classes.purple, classes.blue, classes.rosa, classes.cinza, classes.red, classes.amarelo, classes.marrom, classes.preto, classes.green];
  const [userImage, setUserImage] = useState(undefined);

  useEffect(() => {
    setUserImage(localStorage.getItem(`imgUsuario-${idUsuario}`) || '');
    if (!idUsuario || userImage || localStorage.getItem(`imgUsuario-${idUsuario}`) || forwardedBlockSearch?.return) return;
    fetchToArgous('aguserimage', { idUsuario })
      .then(({ userImage }) => {
        if (userImage) {
          setUserImage(userImage);
          localStorage.setItem(`imgUsuario-${idUsuario}`, userImage);
        } else localStorage.setItem(`imgUsuario-${idUsuario}`, '');
      });
  }, [idUsuario]);

  return <Avatar ref={ref} className={`${className} ${src || userImage ? classes.img : cores[getNumeroFromInitials(getInitials(nome), cores.length)]}`} {...props} src={src || userImage}>{getInitials(nome)}</Avatar>;
});

ArgousAvatar.propTypes = {
  nome: PropTypes.string,
  src: PropTypes.string,
  className: PropTypes.string
};

export default ArgousAvatar;