import { TableContext } from '../ArgousComplexTable/ArgousComplexTable';
import { TableBody, TableRow, Checkbox, Collapse, TableCell, makeStyles, Typography, CircularProgress, alpha, Tooltip, IconButton } from '@material-ui/core';
import clsx from 'clsx';
import { isArray } from 'lodash';
import React, { Fragment, useContext, memo } from 'react';
import { MdDragHandle } from 'react-icons/md';
import { Container, Draggable } from 'react-trello/dist/';

const useStyles = makeStyles(theme => ({
  dragHandle: {
    cursor: 'n-resize'
  },
  hover: {
    '&:hover': {
      backgroundColor: alpha(theme.palette.secondary.main, theme.palette.action.selectedOpacity * 2)
    }
  },
  open: {
    backgroundColor: alpha(theme.palette.primary.main, theme.palette.action.selectedOpacity),
    '&:hover': {
      backgroundColor: alpha(theme.palette.primary.main, theme.palette.action.selectedOpacity * 2),
    }
  },
  semDados: {
    padding: theme.spacing(1),
    paddingLeft: theme.spacing(2),
    fontSize: '1rem',
  },
  group: {
    '&>td': {
      padding: '0px',
      // '&:hover':{
      //   backgroundColor: theme.palette.action.selected
      // }
    }
  },
  loading: {
    textAlign: 'center',
    padding: '7px'
  }
}));

const ArgousComplexTableBody = memo(({
  children, 
  line, 
  lineStyle,
  handleOpenClose,
  lineClass,
  enableCheck,
  disableCheckIf,
  columnStyles, 
  head,
  onClick,
  onDoubleClick,
  indexToSelect,
  open,
  dadosMostrados,
  columnOptions,
  lineTitle,
  customKey,
  columnProps,
  rowProps,
  enableDrag,
  onDrop
}) => {
  const classes = useStyles();

  const { lista, selected, dispatch, loading } = useContext(TableContext);

  const handleClick = name => {
    const selectedIndex = selected.indexOf(name);
    let newSelected = [];
    if (selectedIndex === -1) newSelected = newSelected.concat(selected, name);
    else if (selectedIndex === 0) newSelected = newSelected.concat(selected.slice(1));
    else if (selectedIndex === selected.length - 1) newSelected = newSelected.concat(selected.slice(0, -1));
    else if (selectedIndex > 0) newSelected = newSelected.concat(
      selected.slice(0, selectedIndex),
      selected.slice(selectedIndex + 1),
    );
    dispatch({ type: 'setSelected', value: newSelected });
  };

  const isSelected = name => selected.indexOf(name) !== -1;

  const getCellContent = (coluna, linha, index, i) => {
    if (columnOptions) {
      const item = columnOptions.filter(item => item.alias === index);
      if (item.length > 0) return item[0].customJsx({ index: i, itemFormatado: line(linha)[index], linhaFormatada: line(linha), linhaOriginal: linha });
    }
    return coluna;
  };

  return (
    <Container dragHandleSelector={`.${classes.dragHandle}`} onDrop={onDrop} render={containerRef => (
      <TableBody innerRef={containerRef} /*component={Container} dragHandleSelector={`.${classes.dragHandle}`} onDrop={onDrop}*/>
        {loading ? (
          <TableRow><TableCell className={classes.loading} colSpan={dadosMostrados?.length && line(dadosMostrados[0])?.length + (enableCheck ? 1 : 0) + (enableDrag ? 1 : 0)}><CircularProgress disableShrink/></TableCell></TableRow>
        ) : dadosMostrados.length ? dadosMostrados.map((linha, i) => {
          const isItemSelected = isSelected(linha[indexToSelect]);
          const isItemOpen = i === open;

          const { title, ...rowProperties } = rowProps({ index: i, linhaOriginal: linha, linhaFormatada: line(linha), itensSelecionados: selected }) || {};
          return <Draggable key={customKey ? lista[i][customKey] : i} render={draggableRef => (
            //TODO criar proprio draggable pois esse remove classes do filho
            <Fragment>
              <CustomTooltip title={lineTitle || title}>
                <TableRow
                  component={Draggable}
                  {...rowProperties}
                  className={clsx({ [classes.hover]: isItemSelected, [classes.open]: isItemOpen && !!children }, lineClass(line(linha), linha, i), rowProperties.className)}
                  hover
                  role='checkbox'
                  tabIndex={-1}
                  selected={isItemSelected}
                  style={lineStyle ? lineStyle(linha, line(linha), i) : undefined}
                  onClick={onClick ? e => onClick(e, linha, line(linha), i) : children ? () => handleOpenClose(i) : undefined}
                  onDoubleClick={onDoubleClick ? e => onDoubleClick(e, linha, line(linha), i) : undefined}
                  innerRef={draggableRef} 
                >
                  {enableDrag && <TableCell padding='checkbox'>
                    <Tooltip title='Arrastar'>
                      <IconButton size='small' className={classes.dragHandle}>
                        <MdDragHandle/>
                      </IconButton>
                    </Tooltip>
                  </TableCell>}
                  {enableCheck && <TableCell padding='checkbox'>
                    <Checkbox disabled={disableCheckIf && disableCheckIf(linha, line(linha), i)} onClick={e => {
                      e.stopPropagation();
                      handleClick(linha[indexToSelect]);
                    }} checked={isItemSelected} />
                  </TableCell>}
                  {isArray(line(linha)) && line(linha).map((coluna, j) => (
                    <TableCell key={j} {...columnProps(j, coluna, linha, line(linha))} style={columnStyles ? columnStyles[j] : undefined}>{getCellContent(coluna, dadosMostrados[i], j, i)}</TableCell>
                  ))}
                </TableRow>
              </CustomTooltip>
              {children && (
                <TableRow className={classes.group}>
                  <TableCell colSpan={dadosMostrados?.length && line(dadosMostrados[0])?.length + (enableCheck ? 1 : 0) + (enableDrag ? 1 : 0)} style={{ padding: open === i ? 8 : 0 }} className='argous-complex-table-group-cell'>
                    <Collapse in={open === i} mountOnEnter unmountOnExit>
                      {typeof children === 'function' ? children({ linha: line(linha), linhaOriginal: linha, index: i }) : children}                  
                    </Collapse>
                  </TableCell>
                </TableRow>
              )}
            </Fragment>
          )}/>;
        }) : <TableRow><Typography className={classes.semDados} component='td' variant='h6' colSpan={head.length + (enableCheck ? 1 : 0)}>Nenhum dado a ser exibido</Typography></TableRow> }
      </TableBody>
    )}/>
  );
});

export default ArgousComplexTableBody;

export const CustomTooltip = ({ title, children }) => {
  if (title) return <Tooltip arrow title={title}>
    {children}
  </Tooltip>;
  return children; 
};