import { TableContext } from '../ArgousComplexTable/ArgousComplexTable';
import { TableHead, TableRow, Checkbox, TableCell, TableSortLabel, makeStyles, alpha } from '@material-ui/core';
import clsx from 'clsx';
import { isArray } from 'lodash';
import React, { useContext, memo, useMemo } from 'react';

const useStyles = makeStyles(theme => ({
  highlight: {
    color: theme.palette.type === 'light' ? theme.palette.secondary.main : theme.palette.text.primary,
    backgroundColor: alpha(theme.palette.secondary.main, theme.palette.action.selectedOpacity * 2)   
  },
  lighthighlight: {
    background: alpha(theme.palette.secondary.main, theme.palette.action.selectedOpacity)   
  },
  visuallyHidden: {
    border: 0,
    clip: 'rect(0 0 0 0)',
    height: 1,
    margin: -1,
    overflow: 'hidden',
    padding: 0,
    position: 'absolute',
    top: 20,
    width: 1,
  }
}));

const ArgousComplexTableHead = memo(({ 
  numSelected, 
  rowCount, 
  head, 
  enableCheck,
  disableCheckIf,
  line, 
  size, 
  headerColumnStyles, 
  disableSorting,
  ordCols,
  cortaDados,
  enableDrag
}) => {
  const classes = useStyles();
  const { selected, order, orderBy, dispatch, lista, indexToSelect, method, externalCtrlPgn } = useContext(TableContext);

  const handleRequestSort = property => {
    const isAsc = orderBy === property && order === 'asc';
    dispatch({ type: 'setOrder', value: isAsc ? 'desc' : 'asc' });
    dispatch({ type: 'setOrderBy', value: property });
  };

  const length = useMemo(() => disableCheckIf ? lista.reduce((acc, linha, i) => !disableCheckIf(linha, line(linha), i) ? acc + 1 : acc, 0) : size, [disableCheckIf, lista, size, line]);

  const handleSelectAllClick = event => {
    if (event.target.checked) {
      if (!method && !externalCtrlPgn && (selected.length === 0)) dispatch({ type: 'setSelected', value: cortaDados(lista).filter((linha, i) => disableCheckIf ? !disableCheckIf(linha, line(linha), i) : true).map((n) => n[indexToSelect]) });
      else if (selected.length < length) dispatch({ type: 'setSelected', value: lista.filter((linha, i) => disableCheckIf ? !disableCheckIf(linha, line(linha), i) : true).map((n) => n[indexToSelect]) });
      else dispatch({ type: 'setSelected', value: [] });
    } else dispatch({ type: 'setSelected', value: [] });
  };

  return (
    <TableHead>
      <TableRow className={clsx({ 
        [classes.highlight]: (((numSelected === rowCount) || (numSelected === length) || (numSelected === lista?.length)) && (rowCount && length)), 
        [classes.lighthighlight]: ((numSelected > 0) && ((method || externalCtrlPgn) ? (numSelected < lista?.length) : (numSelected < rowCount))) 
      })}>
        {enableDrag && <TableCell padding='checkbox'/>}
        {enableCheck && <TableCell padding='checkbox'>
          <Checkbox
            disabled={!length}
            indeterminate={(method || externalCtrlPgn) ? ((numSelected > 0) && (numSelected < lista?.length)) : ((numSelected > 0) && (numSelected < length))}
            checked={(method || externalCtrlPgn) ? ((rowCount > 0) && ((numSelected === rowCount) || (numSelected === lista?.length))) : ((rowCount > 0) && length && (numSelected === length))}
            onChange={handleSelectAllClick}
          />
        </TableCell>}
        {isArray(head) && head.map((item, i) => (
          <TableCell key={i} style={headerColumnStyles ? headerColumnStyles[i] : undefined}>
            {(disableSorting[i] || (ordCols && !ordCols[i])) ? 
              item 
              : 
              <TableSortLabel
                active={orderBy === i}
                direction={orderBy === i ? order : 'asc'}
                onClick={() => handleRequestSort(i)}
              >{item}{orderBy === i ? (
                  <span className={classes.visuallyHidden}>
                    {order === 'desc' ? 'sorted descending' : 'sorted ascending'}
                  </span>
                ) : null}
              </TableSortLabel>}
          </TableCell>
        ))}
      </TableRow>
    </TableHead>
  );
});

export default ArgousComplexTableHead;
