import { Grid, makeStyles, Collapse, Grow, useTheme } from '@material-ui/core';
import PropTypes from 'prop-types';
import React from 'react';

const useStyles = makeStyles(theme => ({
  progressContainer: {
    '& > div': {
      padding: '0px !important'
    }
  },
  progressBorder: {
    border: props => `3px solid ${props.borderColor || props.preset.borderColor}`,
    borderRadius: '15px',
  },
  progressBack: {
    height: '20px',
    padding: '2px',
    position: 'relative'
  },
  progressBar: {
    width: '100%',
    height: '100%',
    borderRadius: '10px',
    background: props => props.percnt === 0 ? theme.palette.background.paper : props.percnt > 100 ? props.fullBarColor || props.preset.fullBarColor : props.barColor || props.preset.barColor,
  },
  progressCover: {
    height: '100%',
    transition: '300ms',
    background: props => props.percnt > 99 ? 'transparent' : theme.palette.background.paper,
    position: 'absolute',
    top: '0px',
    right: '0',
    width: props => `${100 - props.percnt}%`,
    minWidth: '5%',
    maxWidth: '95%',
    borderRadius: '0px 10px 10px 0px;',
  },
  topIndicator: {
    zIndex: 1,
    transition: '300ms',
    display: 'block',
    position: 'relative',
    width: 'max-content',
    margin: '0 2px',
    paddingLeft: props => props.percnt > 94 ? '94%' : props.percnt < 5 ? '0' : `${props.percnt}%`,
    transform: props => props.percnt === 0 ? `translate(${props.compact ? '-25px, 1px' : '-10px, 0px'})` : props.percnt > 5 ? `translate(${props.compact ? '-30px, 1px' : '-15px, 0px'})` : '',
    '& p': {
      margin: '0 0 5px',
      border: props => props.compact ? 0 : `2px solid ${props.topIndicatorColor || props.preset.indicatorColor}`,
      padding: '0px 3px',
      borderRadius: '5px',
      minWidth: '25px',
      textAlign: 'center',
      cursor: 'default',
      lineHeight: '12px',
      height: '24px',
    },
    '& p::before': {
      content: '""',
      position: 'relative',
      bottom: '-22px',
      left: '0px',
      width: '0px',
      height: '0px',
      borderTop: props => `5px solid ${props.topIndicatorColor || props.preset.indicatorColor}`,
      borderLeft: '10px solid transparent',
      borderRight: '10px solid transparent',
      display: 'block',
      opacity: props => props.compact ? 0 : 1
    },
  },
  rightIndicator: {
    display: 'block',
    position: 'relative',
    width: 'max-content',
    '& p': {
      margin: '0 7px 0',
      border: props => `2px solid ${props.rightIndicatorColor || props.preset.indicatorColor}`,
      padding: '0px 3px',
      borderRadius: '5px',
      minWidth: '25px',
      textAlign: 'center',
      minHeight: '24px',
      cursor: 'default'
    },
    '& p::before': {
      content: '""',
      position: 'absolute',
      bottom: '2px',
      left: '0px',
      width: '0px',
      height: '0px',
      borderTop: '10px solid transparent',
      borderBottom: '10px solid transparent',
      borderRight: props => `8px solid ${props.rightIndicatorColor || props.preset.indicatorColor}`,
    },
  },
  collapse: {
    height: props => props.compact ? '0px !important' : undefined
  }
}));

const selectPreset = (preset, palette) => {
  switch (preset) {
    case 'secondary':
      return { fullBarColor: palette.secondary.dark, borderColor: palette.secondary.dark, indicatorColor: palette.secondary.dark, barColor: `linear-gradient(90deg, ${palette.secondary.light} 0%, ${palette.secondary.main} 50%, ${palette.secondary.dark} 100%)` };
    case 'success':
      return { fullBarColor: palette.success.dark, borderColor: palette.success.dark, indicatorColor: palette.success.dark, barColor: `linear-gradient(90deg, ${palette.success.light} 0%, ${palette.success.main} 50%, ${palette.success.dark} 100%)` };
    case 'info':
      return { fullBarColor: palette.info.dark, borderColor: palette.info.dark, indicatorColor: palette.info.dark, barColor: `linear-gradient(90deg, ${palette.info.light} 0%, ${palette.info.main} 50%, ${palette.info.dark} 100%)` };
    case 'warning':
      return { fullBarColor: palette.warning.dark, borderColor: palette.warning.dark, indicatorColor: palette.warning.dark, barColor: `linear-gradient(90deg, ${palette.warning.light} 0%, ${palette.warning.main} 50%, ${palette.warning.dark} 100%)` };
    case 'danger':
      return { fullBarColor: palette.error.dark, borderColor: palette.error.dark, indicatorColor: palette.error.dark, barColor: `linear-gradient(90deg, ${palette.error.light} 0%, ${palette.error.main} 50%, ${palette.error.dark} 100%)` };
    case 'hot':
      return { fullBarColor: 'rgba(244,67,54,1)', borderColor: '#d84315', indicatorColor: '#f44336', barColor: 'linear-gradient(90deg, rgba(255,235,59,1) 0%, rgba(255,152,0,1) 50%, rgba(244,67,54,1) 100%)' };
    case 'cold':
      return { fullBarColor: '#673ab7', borderColor: '#5e35b1', indicatorColor: '#512da8', barColor: 'linear-gradient(90deg, #2196f3 0%, #3f51b5 50%, #673ab7 100%)' };
    default:
      return { fullBarColor: palette.primary.dark, borderColor: palette.primary.dark, indicatorColor: palette.primary.dark, barColor: `linear-gradient(90deg, ${palette.primary.light} 0%, ${palette.primary.main} 50%, ${palette.primary.dark} 100%)` };
  }
};

export default function ProgressBar({ valueLabel, valueSuffix, maxValueSuffix, maxValueLabel, value, maxValue, rightIndicatorColor, topIndicatorColor, borderColor, barColor, fullBarColor, preset, compact, className }) {
  const theme = useTheme();
  const percnt = maxValue ? value * 100 / maxValue : 0;
  const classes = useStyles({ percnt, rightIndicatorColor, topIndicatorColor, borderColor, barColor, fullBarColor, preset: selectPreset(preset, theme.palette), compact });
  return (
    <Grid container spacing={2} className={`${classes.progressContainer} ${className}`} style={{ marginTop: compact ? '-10px' : undefined }}>
      <Grid item xs={10} style={{ maxWidth: '80%', flexBasis: '80%' }}>
        <Collapse in={value > 0} className={classes.collapse}>
          <div className={classes.topIndicator}>
            <p title={valueLabel || 'Minutos Realizados'}>{value}{valueSuffix}</p>
          </div>
        </Collapse>
      </Grid>
      <Grid item xs={2} style={{ maxWidth: '20%', flexBasis: '20%' }}></Grid>
      <Grid item xs={10} style={{ maxWidth: '80%', flexBasis: '80%' }}>
        <div item title={`Porcentagem ${percnt.toFixed(2)}%`} className={classes.progressBorder}>
          <div className={classes.progressBack}>
            <div className={classes.progressBar}></div>
            <div className={classes.progressCover}></div>
          </div>
        </div>
      </Grid>
      <Grid item xs={2} style={{ maxWidth: '20%', flexBasis: '20%' }}>
        <Grow in={maxValue > 0}>
          <div className={classes.rightIndicator}>
            <p title={maxValueLabel || 'Minutos Previstos'}>{maxValue}{maxValueSuffix}</p>
          </div>
        </Grow>
      </Grid>
    </Grid>
  );
}

ProgressBar.propTypes = {
  value: PropTypes.number,
  maxValue: PropTypes.number,
  rightIndicatorColor: PropTypes.string,
  topIndicatorColor: PropTypes.string,
  borderColor: PropTypes.string,
  barColor: PropTypes.string,
  fullBarColor: PropTypes.string,
  compact: PropTypes.bool,
  preset: PropTypes.oneOf(['default', 'primary', 'secondary', 'success', 'info', 'warning', 'danger', 'hot', 'cold'])
};