import { Input, SimpleTable } from './../';
import { createField, getField, setField } from '../../fieldsRedux';
import { serverAction } from '../../utils/api';
import PropTypes from 'prop-types';
import React, { Component } from 'react';
import { Scrollbars } from 'react-custom-scrollbars';

/**
 * @deprecated 
 */
class AutoComplete extends Component {
  state = {
    typeMessage: -1,
    resultHeight: 0
  }

  componentDidUpdate(prevProps, prevState) {
    if (prevState.typeMessage !== 0 && this.state.typeMessage === 0) {
      const resultsDiv = document.getElementById(`results-${this.props.field}`);
      if (resultsDiv)
        this.setState({ resultHeight: resultsDiv.clientHeight });
    }
  }

  componentWillUnmount() {
    this.props.clearField(`results-${this.props.field}`);
    clearTimeout(this.searchTimeout);
    clearTimeout(this.blurTimeout);
  }

  changeTypeMessage = (value) => {
    this.buscarServidorEMensagensFeedback(value);
    this.removerSelecionadoSeHouverEdicao();
  }

  removerSelecionadoSeHouverEdicao = () => {
    const selected = this.props.getField().selected;

    if (selected) {
      const { onChangeSelected } = this.props;

      this.props.setField({ selected: undefined });

      if (onChangeSelected)
        onChangeSelected({});
    }
  }

  buscarServidorEMensagensFeedback = (value) => {

    const { dados = {} } = this.props;

    clearTimeout(this.searchTimeout);

    setField(`results-${this.props.field}`, { value: undefined });
    this.setState({ typeMessage: 2, resultHeight: 50 });

    this.searchTimeout = setTimeout(() => {

      serverAction(this.props.method, { value, ...dados }).then(res => {

        if (res.result) {
          setField(`results-${this.props.field}`, { value: res.result });
          this.setState({ typeMessage: 0, resultHeight: res.result.length * 38 });
        } else {
          this.setState({ typeMessage: 3, resultHeight: 50 });
        }

      });
    }, 350);
  }

  focusTypeMessage = (e) => {
    const thisField = this.props.getField();
    const resultsData = getField(`results-${this.props.field}`, 'value');

    if (!thisField.selected) {
      if (resultsData.length === 0) {
        this.setState({ typeMessage: 1, resultHeight: 50 });
      } else {
        this.setState({ typeMessage: 0, resultHeight: resultsData.length * 38 });
      }
    }

    if (thisField.selected) {
      e.target.select();
    }
  }

  blurTypeMessage = (e) => {
    const { onBlur } = this.props;

    if (onBlur && typeof onBlur === 'function') {
      onBlur(e);
    }

    this.blurTimeout = setTimeout(() => {
      this.setState({ typeMessage: -1, resultHeight: 0 });
    }, 250);
  }

  selectResult = (value) => {
    const { alias, aliasInputValue, onChangeSelected } = this.props;

    let v = '';

    for (let a of aliasInputValue || alias) {
      if (typeof value[a] !== 'undefined')
        v += `${value[a].toString().trim()} `;
    }

    setField(this.props.field, 'value', v);
    setField(this.props.field, 'selected', value);

    if (onChangeSelected)
      onChangeSelected(value);
  }

  render() {
    const { typeMessage, resultHeight } = this.state;
    const { field, placeholder, alias, inputClassName, dropdownClassName, dropdownMaxHeight, head, showOnlyResults } = this.props;

    return (
      <table className='argous-groupselect reset-border'>
        <tbody>
          <tr>
            <td>
              <Input
                field={field}
                onFocus={this.focusTypeMessage}
                onBlur={this.blurTypeMessage}
                onChange={this.changeTypeMessage}
                fieldListener={this.fieldListener}
                placeholder={placeholder}
                className={inputClassName}
              />
            </td>
          </tr>
          <tr>
            <td>
              {typeMessage > -1 &&

                <div className={dropdownClassName ? dropdownClassName : 'argous-list-dropdown'} style={{ height: !showOnlyResults ? resultHeight : typeMessage === 0 ? resultHeight : 0, maxHeight: dropdownMaxHeight }}>
                  <Scrollbars>
                    {typeMessage > 0 && !showOnlyResults && (
                      <div className='padding-top-sm padding-bottom-sm color999 text-center'>
                        {typeMessage === 1 && 'Escreva para começar a buscar'}
                        {typeMessage === 2 && 'Pesquisando...'}
                        {typeMessage === 3 && 'Nenhum resultado foi encontrado'}
                      </div>
                    )}

                    <SimpleTable
                      id={`results-${field}`}
                      field={`results-${field}`}
                      alias={alias}
                      head={typeMessage === 0 ? head : undefined}
                      onClick={this.selectResult}
                      className='reset-margin'
                    />
                  </Scrollbars>
                </div>
              }
            </td>
          </tr>
        </tbody>
      </table>
    );
  }
}

AutoComplete.propTypes = {
  field: PropTypes.string.isRequired,
  alias: PropTypes.array.isRequired,
  method: PropTypes.string.isRequired,
  head: PropTypes.array,
  aliasInputValue: PropTypes.array,
  placeholder: PropTypes.string,
  inputClassName: PropTypes.string,
  dropdownClassName: PropTypes.string,
  fieldListener: PropTypes.func,
  destroyOnUnmount: PropTypes.bool,
  dropdownMaxHeight: PropTypes.number,
  onChangeSelected: PropTypes.func,
  showOnlyResults: PropTypes.bool,
  dados: PropTypes.object
};

export default createField(AutoComplete, { disableInitField: true });
