import { serverAction } from '../../utils/api';
import { CircularProgress, Fade, IconButton, ListItem, ListItemText, ListSubheader, makeStyles, Popper, Slide, TextField, Tooltip, withStyles } from '@material-ui/core';
import { Autocomplete } from '@material-ui/lab';
import clsx from 'clsx';
import { isArray, isObject, isString } from 'lodash';
import React, { forwardRef, useCallback, useEffect, useRef, useState } from 'react';
import { IoIosSearch } from 'react-icons/io';

const useStyles = makeStyles((theme) => ({
  subHeader: {
    display: 'flex',
    alignItems: 'center',
    justifyContent: 'space-between',
    backgroundColor: theme.palette.background.default,
    height: 25
  },
  divButton: {
    display: 'flex',
    alignItems: 'center',
    justifyContent: 'center',
  },
  areaButton: {
    padding: '0px 8px 0px 8px'
  },
  button: theme.palette.type === 'light' ? {
    width: 15,
    height: 15,
    display: 'flex',
    alignItems: 'center',
    justifyContent: 'center',
    cursor: 'pointer',
    color: 'rgba(0, 0, 0, 0.4)',
    '&:hover': {
      fontWeight: 500,
      color: '#000000',
    }
  } : {
    width: 15,
    height: 15,
    display: 'flex',
    alignItems: 'center',
    justifyContent: 'center',
    cursor: 'pointer',
    color: 'rgba(255, 255, 255, 0.4)',
    '&:hover': {
      fontWeight: 500,
      color: '#ffffff',
    }
  },
  buttonActive: theme.palette.type === 'light' ? {
    color: 'rgba(0, 0, 0, 0.8)',
    '&:hover': {
      fontWeight: 600,
    }
  } : {
    color: 'rgba(255, 255, 255, 0.8)',
    '&:hover': {
      fontWeight: 600,
    }
  },
  listItem: {
    padding: 0,
    width: '100%',
    cursor: 'pointer',
    '&:hover': {
      background: theme.palette.type === 'light' ? 'rgba(0, 0, 0, 0.05)' : 'rgba(255, 255, 255, 0.08)'
    }
  },
  hoverListItem: {
    background: theme.palette.type === 'light' ? 'rgba(0, 0, 0, 0.05)' : 'rgba(255, 255, 255, 0.08)'
  },
  listItemText: {
    margin: 0,
    width: '100%',
    direction: 'ltr',
    overflow: 'visible',
    whiteSpace: 'nowrap',
    textOverflow: 'ellipsis',
    padding: '0px 0px 0px 6px',
  },
  box: {
    padding: 0,
    overflowX: 'hidden',
    '&::-webkit-scrollbar': {
      width: 5
    },
    '&::-webkit-scrollbar-track': {
      background: '#0000'
    },
    '&::-webkit-scrollbar-thumb': {
      background: theme.palette.type === 'light' ? 'rgba(0, 0, 0, 0.1)' : 'rgba(255, 255, 255, 0.1)'
    }
  },
  searchButton: {
    color: theme.palette.type === 'light' ? 'rgba(0, 0, 0, 0.4)' : 'rgba(255, 255, 255, 0.5)',
    '&:hover': {
      color: theme.palette.type === 'light' ? 'rgba(0, 0, 0, 0.8)' : 'rgba(255, 255, 255, 0.9)',
      background: '#0000'
    }
  },
  popper: {
    zIndex: 1300,
    width: ({ widthListBox }) => widthListBox ? `${widthListBox}${testWidth(widthListBox.toString()) ? 'px' : ''} !important` : '',
  }
}));

const testWidth = value => value.replace(/[0-9]/g, '')?.length ? false : true;

const CustomPopper = (props, classes) => <Popper {...props} className={classes.popper} />;

export const CustomTooltip = withStyles((theme) => ({
  tooltip: theme.palette.type === 'light' ? {
    fontSize: 15,
    fontWeight: 400,
    color: '#FFF',
    backgroundColor: '#333333e3',
  } : {
    fontSize: 15,
    fontWeight: 400,
    color: '#000',
    backgroundColor: '#fbfbfbd1',
  },
  arrow: {
    color: theme.palette.type === 'light' ? '#333333e3' : '#fbfbfbd1'
  }
}))(Tooltip);

const ListboxFavorites = forwardRef((props, ref) => {

  const classes = useStyles();
  const { children, favoritePage, dados, setDados, group, groupHeaderColor, groupTitleColor } = props;
  const { pageNumber, favoritosLength, direction, slide, cursor } = dados;

  function retornaInicio() {
    switch (pageNumber) {
      case 1:
        return 0;
      case 2:
        return 7;
      case 3:
        return 14;
      default: return;
    }
  }

  function retornaFim() {
    switch (pageNumber) {
      case 1:
        return 7;
      case 2:
        return 14;
      case 3:
        return favoritosLength;
      default: return;
    }
  }

  function retornaArray(sectionIndex, section) {
    if (sectionIndex === 0 && favoritePage && favoritosLength > 7) return section.props.children[1].props.children.slice(retornaInicio(), retornaFim());
    return section.props.children[1].props.children;
  }

  function retornaEstrutura(sectionIndex, item, itemIndex) {
    const index = itemIndex ? itemIndex : sectionIndex;
    const content = <CustomTooltip title={item.props.children} arrow enterDelay='1000' enterNextDelay='1000' placement='top-end' TransitionComponent={Fade} TransitionProps={{ timeout: { enter: 100, exit: 500 } }}>
      <ListItem {...item.props} id='listItem' key={item.key} className={clsx(classes.listItem, { [classes.hoverListItem]: cursor?.index === index && cursor?.page === pageNumber })}>
        <ListItemText primary={item.props.children} className={classes.listItemText} />
      </ListItem>
    </CustomTooltip>;
    if (sectionIndex === 0 && favoritePage && favoritosLength > 7) {
      return <Slide in={slide} direction={direction}>
        {content}
      </Slide>;
    } return content;
  }

  function buttonClick(evento, value) {
    evento.preventDefault();
    evento.stopPropagation();
    if (value !== pageNumber) {
      verificaSaidaSlide(value);
      setDados(v => ({ ...v, slide: false }));
      setTimeout(() => {
        setDados(v => ({ ...v, pageNumber: value, direction: retornaDirection(value) }));
        setTimeout(() => setDados(v => ({ ...v, slide: true })), 100);
      }, 200);
    }
  }

  function verificaSaidaSlide(value) {
    let direction = '';
    if ((value === 2 || value === 3) && pageNumber === 1) direction = 'right';
    else if (value === 3 && pageNumber === 2) direction = 'right';
    else if (value === 1 && pageNumber === 2) direction = 'left';
    direction && setDados(v => ({ ...v, direction: direction }));
  }

  function retornaDirection(value) {
    if (value < pageNumber) return 'right';
    return 'left';
  }

  return <ul {...props} id='listBox' className={clsx('MuiAutocomplete-listbox', classes.box)}>
    {children.map((section, sectionIndex) => (group ?
      <React.Fragment>
        <ul {...props} id='listBox' className={clsx('MuiAutocomplete-listbox', classes.box)}>
          <ListSubheader className={classes.subHeader} style={{ backgroundColor: groupHeaderColor?.[sectionIndex], color: groupTitleColor?.[sectionIndex] }}>
            {section.props.children[0].props.children}
            <Fade in={sectionIndex === 0 && favoritePage && favoritosLength > 7}>
              <div className={classes.divButton}>
                <div className={classes.areaButton}>
                  <div onClick={e => buttonClick(e, 1)} className={clsx({ [classes.buttonActive]: pageNumber === 1 }, classes.button)}>1</div>
                </div>
                <div className={classes.areaButton}>
                  <div onClick={e => buttonClick(e, 2)} className={clsx({ [classes.buttonActive]: pageNumber === 2 }, classes.button)}>2</div>
                </div>
                {favoritosLength > 14 &&
                  <React.Fragment>
                    <div className={classes.areaButton}>
                      <div onClick={e => buttonClick(e, 3)} className={clsx({ [classes.buttonActive]: pageNumber === 3 }, classes.button)}>3</div>
                    </div>
                  </React.Fragment>
                }
              </div>
            </Fade>
          </ListSubheader>
          {retornaArray(sectionIndex, section).map((item, itemIndex) => retornaEstrutura(sectionIndex, item, itemIndex))}
        </ul>
      </React.Fragment>
      : retornaEstrutura(sectionIndex, section)
    ))}
  </ul>;
});

const AutoCompleteFavorites = ({ 
  action, 
  busca, 
  resposta, 
  autoSearch, 
  label,
  value,
  pgCode,
  favoritePage,
  group = false,
  groupName,
  groupHeaderColor,
  groupTitleColor,
  loadingColor,
  widthListBox,
  filterFunction,
  getOptionLabel,
  searchParams,
  autoSelectFirstOption,
  onChange = e => e,
  textFieldProps = {}, 
  ...props 
}) => {
  
  const classes = useStyles({ widthListBox });
  const [dados, setDados] = useState({
    lista: [],
    slide: true,
    cursor: { index: '', page: '' },
    direction: '',
    favoritosLength: '',
    pageNumber: favoritePage ? 1 : undefined,
  });
  const [loading, setLoading] = useState(false);
  const [inputValue, setInputValue] = useState('');

  const interval = useRef(null);
  const [seconds, setSeconds] = useState(1);

  const startCounter = () => interval.current = setInterval(() => setSeconds(1), 1000);
  const stopCounter = () => {
    clearInterval(interval.current);
    setSeconds(0);
  };

  const setInput = value => {
    if (loading) return;
    stopCounter();
    setInputValue(value);
    autoSearch && startCounter();
  };

  async function carimbaLista(lista) {
    let completo = [];
    lista.map((arr, index) => {
      index === 0 && favoritePage && setDados(v => ({ ...v, favoritosLength: arr.length }));
      return arr.map(itemArr => itemArr.length < itemArr.length + 1 && itemArr.push(groupName[index]));
    });
    lista.map(v => v.map(d => completo.push(d)));
    setDados(v => ({ ...v, lista: completo }));
  }

  const buscar = useCallback(async valor => {
    stopCounter();
    setLoading(true);
    const res = await serverAction(action, { [busca]: valor || inputValue, ...searchParams }, true, pgCode);
    if (res[resposta || 'result']?.length > 1 && group) await carimbaLista(res[resposta || 'result']);
    else if (res[resposta || 'result']) setDados(v => ({ ...v, lista: res[resposta || 'result'] || [] }));
    setLoading(false);
  // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [inputValue, searchParams]);

  useEffect(() => autoSearch && seconds === 1 && buscar(), [autoSearch, seconds, buscar]);
  // eslint-disable-next-line react-hooks/exhaustive-deps
  useEffect(() => isString(value) && buscar(value), []);

  useEffect(() => (value !== dados.cursor?.linha) && setDados(v => ({ ...v, cursor: { index: '', page: '' } })), [value, dados.cursor?.linha]);

  // eslint-disable-next-line react-hooks/exhaustive-deps
  useEffect(() => autoSelectFirstOption && !!inputValue.length && onChange(undefined, dados?.lista?.[0] || null), [dados.lista]);

  const keyDown = e => {
    ((e.key === 'Enter' || e.key === 'Tab') && !loading) && buscar();
    (e.key === 'ArrowDown' || e.key === 'ArrowUp') && selectListboxItem(e.key);
  };

  const keyUp = e => (e.key === 'Enter' || e.key === 'Tab') && e.stopPropagation();

  const numbersOnly = value => value.replace(/[^0-9]/g, '');

  const getFinder = v => {
    const elInnerText = v.innerText;
    const endIndexCode = elInnerText.indexOf('-') - 1;
    const code = elInnerText.substring(0, endIndexCode);
    const finder = numbersOnly(code)?.length ? code : elInnerText;
    return finder;
  };

  function validateLine(value, finder, el) {
    let retorno = false;
    if (el) {
      const elIndex = parseInt(el?.attributes?.[4]?.value);
      const itemSelecionado = dados.lista?.[elIndex];
      if (isObject(itemSelecionado)) retorno = JSON.stringify(value) === JSON.stringify(itemSelecionado) ? true : false;
      else if (isArray(itemSelecionado)) retorno = value.equals(itemSelecionado) ? true : false;
    } else if (isArray(value)) {
      value.forEach(v => {
        if (retorno) return;
        if (v && v.toString() === finder) retorno = true;
      });
    } else if (isObject(value)) {
      Object.values(value).forEach(v => {
        if (retorno) return;
        if (v && v.toString() === finder) retorno = true;
      });
    }
    return retorno;
  }

  function selectListboxItem(key) {
    const directionIndex = key === 'ArrowDown' ? 1 : key === 'ArrowUp' ? -1 : 0;
    const itens = document.querySelectorAll('#listItem');
    const array = Array.from(itens);

    let el = '';
    let linha = '';
    let indexCapture = 0;

    if (value) {
      array.forEach((v, i) => {
        const finder = getFinder(v);
        if (linha) return;
        if (validateLine(value, finder, v)) {
          const nextIndex = i + directionIndex;
          if (nextIndex < array.length) indexCapture = nextIndex > -1 ? nextIndex : i;
          else indexCapture = i;
          linha = dados.lista[indexCapture];
        }
      });
      el = array[indexCapture];
    }
    if (!linha)linha = dados.lista[indexCapture];

    let elTop = 0;
    let elTotalHeight = 0;
    let lastScrollTop = 0;
    const elListBox = document.getElementById('listBox');
    
    if (el) {
      elTop = el?.getBoundingClientRect()?.top;
      elTotalHeight = el.offsetHeight + el.offsetTop;
    }
    if (elListBox) lastScrollTop = elListBox.scrollTop;

    linha && setDados(v => ({ ...v, cursor: { index: indexCapture, page: v.pageNumber, linha: linha, elTop: elTop, elTotalHeight: elTotalHeight, lastScrollTop: lastScrollTop } }));
    if (linha !== value) onChange(undefined, linha);
  }

  const scrollToSelected = useCallback(() => {
    const elListBox = document.getElementById('listBox');
    if (elListBox) {
      const clientHeight = elListBox?.clientHeight;
      if (clientHeight < dados.cursor?.elTotalHeight) {
        let valueScrollTop = dados.cursor?.elTop - clientHeight;
        if (dados.cursor?.lastScrollTop) valueScrollTop += dados.cursor?.lastScrollTop;
        elListBox.scrollTop = valueScrollTop;
      }
    }
  }, [dados]);
  useEffect(() => dados.lista?.length && setTimeout(() => scrollToSelected(), 0), [dados.lista, scrollToSelected]);

  return (
    <Autocomplete
      {...props}
      {...{ value, onChange }}
      autoComplete
      autoHighlight
      openText='Abrir'
      clearText='Limpar'
      filterSelectedOptions
      onInputChange={e => setInput(e?.target?.value || '')}
      getOptionSelected={(o, v) => o[0] === v}
      getOptionLabel={getOptionLabel && (option => getOptionLabel(option, dados?.lista))}
      noOptionsText={inputValue.length < 2 ? 'Digite para pesquisar' : 'Não Encontrado'}
      options={isObject(value) ? dados?.lista.filter(v => filterFunction ? filterFunction(v, value) : true || v[0] !== value[0]) : dados?.lista}
      renderInput={params => (
        <TextField
          margin='dense'
          variant='outlined'
          label={label}
          {...params}
          {...textFieldProps}
          onKeyUp={e => keyUp(e)}
          inputProps={{
            ...params.inputProps,
            onKeyDown: e => keyDown(e)
          }}
          InputProps={{
            ...textFieldProps.InputProps,
            ...params.InputProps,
            endAdornment: (
              <React.Fragment>
                {params.InputProps.endAdornment}
                <Fade in={loading}><CircularProgress disableShrink color={loadingColor || 'inherit'} size={20} /></Fade>
                {!autoSearch && <IconButton size='small' onClick={e => buscar()} className={classes.searchButton}><IoIosSearch size={20} /></IconButton>}
              </React.Fragment>
            )
          }}
        />
      )}
      ListboxComponent={ListboxFavorites}
      ListboxProps={{ dados, setDados, favoritePage, group, groupHeaderColor, groupTitleColor }}
      PopperComponent={props => CustomPopper(props, classes)}
    />
  );
};

export default AutoCompleteFavorites;