import { Compartilhar } from '..';
import ThreadLog from '../../components/ThreadLog';
import { usePrevious } from '../../utils';
import { Box, CircularProgress, ClickAwayListener, Collapse, Divider, IconButton, makeStyles, MenuItem, MenuList, Paper, Popper, Toolbar, Tooltip, Typography, withStyles } from '@material-ui/core';
import clsx from 'clsx';
import { isArray, isNumber, isString, isEqual, isFunction } from 'lodash';
import PropTypes from 'prop-types';
import React, { useEffect, useRef, useState } from 'react';
import { AiOutlineAppstoreAdd } from 'react-icons/ai';
import { FaStar } from 'react-icons/fa';
import { MdShare, MdFormatListBulleted, MdArrowBack, MdEdit, MdDelete, MdSave, MdKeyboardArrowRight, MdFirstPage, MdKeyboardArrowLeft, MdLastPage, MdEditNote } from 'react-icons/md';


const useStyles = makeStyles(theme => ({
  root: {
    zIndex: 0,
    margin: '-11px',
    marginBottom: 0,
    overflow: 'auto',
    width: 'calc(100% + 22px)',
    paddingRight: theme.spacing(2), 
    paddingLeft: theme.spacing(2),
    '&::-webkit-scrollbar': {
      height: 5
    },
    '&::-webkit-scrollbar-track': {
      background: '#0000'
    },
    '&::-webkit-scrollbar-thumb': {
      background: '#0000'
    },
    '&:hover': {
      '&::-webkit-scrollbar-thumb': {
        background: theme.palette.type === 'light' ? 'rgba(0, 0, 0, 0.05)' : 'rgba(255, 255, 255, 0.05)'
      }
    }
  },
  menuButton: {
    marginRight: theme.spacing(2),
  },
  menuSelected: {
    backgroundColor: theme.palette.action.selected
  },
  title: {
    flexGrow: ({ habNavegador }) => habNavegador ? 0 : 1,
  },
  navigator: {
    flexGrow: 1,
    marginLeft: theme.spacing(2)
  }
}));

const ToolbarTooltip = withStyles({
  popper: {
    zIndex: 1300
  }
})(Tooltip);


const CustomTooltip = ({ disabled, arrow, placement, title, children }) => {
  const [open, setOpen] = useState(false);
  useEffect(() => (disabled && open) && setOpen(false), [disabled, open]);
  return (
    <ToolbarTooltip 
      {...{ arrow, placement, title }} 
      onOpen={() => (!disabled) && setOpen(true)} 
      onClose={() => setOpen(false)} 
      open={open}
    >
      {children}
    </ToolbarTooltip>
  );
};

const ButtonLoadingFeedback = ({ size = 24, color = 'secondary' }) => <CircularProgress {...{ size, color }} />;


function BarraFerramentas ({
  titulo,
  abas = [],
  ferramentas = [],
  habFavorito = true,
  habCrud = false,
  habNavegador = false,
  checkIfIsFavorite,
  addFavorite,
  favoriteAnimationInsert,
  favoriteAnimationRemove,
  nmFuncaoVoltar,
  nmCompartilhar,
  linkCompartilhamento = '',
  threads,
  currentProgram,
  nmSalva,
  nmExclui,
  executaFuncao = prm => void(prm),
  habWidget = false,
  addWidget = () => '',
  snAnotacaoPrograma = false,
  openAnnotations = () => '',
  toolbarLoading = {}
}) {

  const classes = useStyles({ habNavegador });
  const anchorMenu = useRef(null);
  const anchorCompartilhar = useRef(null);
  const [open, setOpen] = useState(false);
  const [abaAtual, setAbaAtual] = useState(undefined);
  const [habEditar, setHabEditar] = useState(true);
  const [habSalvar, setHabSalvar] = useState(false);
  const [habExcluir, setHabExcluir] = useState(false);
  const [openCompartilhar, setOpenCompartilhar] = useState(false);
  const [disableTooltip, setDisableTooltip] = useState(false);

  useEffect(() => {
    if (anchorMenu.current) anchorMenu.current.focus();
  }, [open]);

  const previousAbas = usePrevious(abas);
  useEffect(() => {
    if (isEqual(abas, previousAbas)) return;
    if (!abas.length) setAbaAtual(undefined);
    else for (let i = 0; i < abas?.length; i++) {
      const aba = abas[i];
      if (aba[1] === 'active') {
        setAbaAtual(aba[0]);
        break;
      }
    }
  }, [abas, previousAbas]);

  const toggleAbas = () => {
    setOpen(prevOpen => !prevOpen);
  };

  const toggleCompartilhar = e => {
    e.stopPropagation();
    setOpenCompartilhar(o => !o);
    executaFuncao(nmCompartilhar);
  };

  const closeCompartilhar = event => {
    if (anchorCompartilhar.current && anchorCompartilhar?.current?.contains(event.target)) return;
    setOpenCompartilhar(false);
  };

  const selecionaAba = (index, item, alias) => event => {
    if (anchorMenu.current && anchorMenu?.current?.contains(event.target)) return;
    if (isString(item)) setAbaAtual(item);
    if (isNumber(index)) executaFuncao(alias || `T${index}`);
    setOpen(false);
  };

  function handleListKeyDown(event) {
    if (event.key === 'Tab') {
      event.preventDefault();
      setOpen(false);
    }
  }

  function handleAnnotationsClick(event) {
    setDisableTooltip(true);
    openAnnotations(event);
    setTimeout(() => setDisableTooltip(false), 0);
  }

  const getFerramentas = (item, index) => {
    return (
      <CustomTooltip key={index} title={item[1] || ''}>
        <IconButton onClick={() => isFunction(item[2]) ? item[2]() : executaFuncao(item[2] || 'F' + index)}>
          {typeof item[0] === 'string' ? (
            <i style={{ color: '#999' }} className={item[0]}></i>
          ) : (
            item[0]
          )}
        </IconButton>
      </CustomTooltip>
    );
  };

  return <>
    <Toolbar className={classes.root} /*disableGutters*/ variant='dense'>
      {(isArray(abas) && abas.length > 0) && (
        <IconButton id='botaoAbasBarraFerramentas' ref={anchorMenu} onClick={toggleAbas} className={classes.menuButton}>
          <MdFormatListBulleted />
        </IconButton>
      )}
      <Popper open={open} anchorEl={anchorMenu.current} role={undefined} transition keepMounted style={{ zIndex: 2, marginLeft: 10 }}>
        {({ TransitionProps }) => (
          <Collapse in={TransitionProps.in} timeout={100}>
            <Paper style={{ maxHeight: '80vh', overflowY: 'auto' }}>
              <ClickAwayListener onClickAway={selecionaAba()}>
                <MenuList autoFocusItem={open} onKeyDown={handleListKeyDown}>
                  {isArray(abas) && abas.map((item, key) => (
                    <MenuItem className={clsx({ [classes.menuSelected]: abaAtual === item[0] })} key={key} onClick={selecionaAba(key, item[0], item[2])}>{item[0]}</MenuItem>
                  ))}
                </MenuList>
              </ClickAwayListener>
            </Paper>
          </Collapse>
        )}
      </Popper>
      <Typography component='div' variant='h6' className={classes.title}>
        {(abaAtual) ? (
          <Typography variant='subtitle1'>{abaAtual} <Typography variant='caption'>( {titulo} )</Typography></Typography>
        ) : (
          <Typography variant='subtitle1'>{titulo}</Typography>
        )}
      </Typography>

      {(habNavegador) && (
        <Box className={classes.navigator}>
          <CustomTooltip title='Primeiro registro'>
            <IconButton size='small' onClick={() => executaFuncao('NF')}>
              <MdFirstPage />
            </IconButton>
          </CustomTooltip>
          <CustomTooltip title='Registro anterior'>
            <IconButton size='small' onClick={() => executaFuncao('NP')}>
              <MdKeyboardArrowLeft/>
            </IconButton>
          </CustomTooltip>
          <CustomTooltip title='Próximo registro'>
            <IconButton size='small' onClick={() => executaFuncao('NN')}>
              <MdKeyboardArrowRight/>
            </IconButton>
          </CustomTooltip>
          <CustomTooltip title='Último registro'>
            <IconButton size='small' onClick={() => executaFuncao('NL')}>
              <MdLastPage />
            </IconButton>
          </CustomTooltip>
        </Box>
      )}

      {(isArray(ferramentas)) && ferramentas.map(g => g.map((item, index) => getFerramentas(item, index)))}

      {(nmFuncaoVoltar) && (
        <CustomTooltip title='Voltar'>
          <IconButton onClick={() => executaFuncao(nmFuncaoVoltar)}>
            <MdArrowBack/>
          </IconButton>
        </CustomTooltip>
      )}
      {(habCrud) && <>
        {(habEditar) ? (
          <CustomTooltip title='Editar'>
            <IconButton onClick={() => {
              setHabEditar(false);
              setHabSalvar(true);
              setHabExcluir(true); 
            }}>
              <MdEdit/>
            </IconButton>
          </CustomTooltip>
        ) : (
          <ClickAwayListener onClickAway={() => {
            setHabEditar(true);
            setHabSalvar(false);
            setHabExcluir(false); 
          }}>
            <Box>
              {(habSalvar) && (
                <CustomTooltip title='Salvar Registro'>
                  <IconButton onClick={() => executaFuncao(nmSalva)}>
                    <MdSave/>
                  </IconButton>
                </CustomTooltip>
              )}
              {(habExcluir) && (
                <CustomTooltip title='Excluir Registro'>
                  <IconButton onClick={() => executaFuncao(nmExclui)}>
                    <MdDelete/>
                  </IconButton>
                </CustomTooltip>
              )}
            </Box>
          </ClickAwayListener>
        )}
      </>}
      {(nmCompartilhar) && (
        <CustomTooltip title='Compartilhar'>
          <IconButton ref={anchorCompartilhar} onClick={toggleCompartilhar}>
            <MdShare />
          </IconButton>
        </CustomTooltip>
      )}
      <Popper open={openCompartilhar} anchorEl={anchorCompartilhar.current} role={undefined} transition keepMounted style={{ zIndex: 2, marginRight: 10 }}>
        {({ TransitionProps }) => (
          <Collapse in={TransitionProps.in} timeout={100}>
            <ClickAwayListener onClickAway={closeCompartilhar}>
              <Compartilhar {...{ linkCompartilhamento }} />
            </ClickAwayListener>
          </Collapse>
        )}
      </Popper>
      {(isArray(threads)) && threads.map((thread, key) => <ThreadLog {...{ key, thread, currentProgram }} />)}
      {(habWidget) && (
        <CustomTooltip title='Adicionar widget'>
          <IconButton onClick={addWidget}>
            <AiOutlineAppstoreAdd />
          </IconButton>
        </CustomTooltip>
      )}
      {(snAnotacaoPrograma) && (
        <CustomTooltip title='Anotações' disabled={disableTooltip}>
          <IconButton disabled={toolbarLoading?.annotations} onClick={handleAnnotationsClick}>
            {(toolbarLoading?.annotations) ? <ButtonLoadingFeedback /> : <MdEditNote />}
          </IconButton>
        </CustomTooltip>
      )}
      {(habFavorito && checkIfIsFavorite) && (
        <CustomTooltip title='Favorito'>
          <IconButton onClick={addFavorite}>
            <FaStar className={`pg-fav-start ${checkIfIsFavorite()}`}></FaStar>
            {favoriteAnimationInsert && <FaStar className='pg-fav-start active insertion-star-fav' />}
            {favoriteAnimationRemove && <FaStar className='pg-fav-start active removing-star-fav' />}
          </IconButton>
        </CustomTooltip>
      )}
    </Toolbar>
    <Divider style={{ marginBottom: 10 }}/>
  </>;
}

BarraFerramentas.propTypes = {
  titulo: PropTypes.string,
  ferramentas: PropTypes.array,
  abas: PropTypes.array,
  executaFuncao: PropTypes.func,
  nmFuncaoVoltar: PropTypes.string,
  nmCompartilhar: PropTypes.string,
  habFavorito: PropTypes.bool,
  habNavegador: PropTypes.bool,
  habCrud: PropTypes.bool,
  nmSalva: PropTypes.string,
  nmExclui: PropTypes.string,
  addFavorite: PropTypes.func,
  favoriteAnimationInsert: PropTypes.bool,
  favoriteAnimationRemove: PropTypes.bool,
  linkCompartilhamento: PropTypes.string,
  threads: PropTypes.array,
  currentProgram: PropTypes.string,
  habWidget: PropTypes.bool,
  addWidget: PropTypes.func
};

export default BarraFerramentas;