import { createField, initField, getField } from '../../fieldsRedux';
import { addExtraZero } from '../../utils';
import { getMonthName } from '../../utils/date';
import PropTypes from 'prop-types';
import React, { Component } from 'react';
import { Calendar as CalDep } from 'react-date-range';

/**
 * @deprecated 
 */
class Calendar extends Component {
  state = {
    open: false,
    month: 0
  }

  componentDidMount() {
    const { field, defaultValue } = this.props;
    const defValHandled = this.checkDefaultValue(defaultValue);

    initField(field, defValHandled).then(value => {
      this.setMonthState(value);
    });
  }

  componentWillUnmount() {
    document.removeEventListener('click', this.handleClickCalendar);
  }

  openCalendar = (e) => {
    e.stopPropagation();
    this.setState({ open: true });
    setTimeout(() => document.addEventListener('click', this.handleClickCalendar), 0);
  }

  handleClickCalendar = (e) => {

    const eCls = e.target.className;
    const eClsP = e.target.parentNode.className;

    const prev = eCls !== 'rdr-MonthAndYear-button next' && eClsP !== 'rdr-MonthAndYear-button next';
    const next = eCls !== 'rdr-MonthAndYear-button prev' && eClsP !== 'rdr-MonthAndYear-button prev';

    if (next && prev) {
      this.setState({ open: false });
      document.removeEventListener('click', this.handleClickCalendar);
    } else {
      const { month } = this.state;

      let newMonth = next
        ? month + 1
        : month - 1;

      newMonth = newMonth < 0
        ? 11
        : newMonth > 11
          ? 0
          : newMonth;

      this.setState({ month: newMonth });
      this.setMonthName(null, newMonth);
    }

  }

  checkDefaultValue = (defaultValue) => {
    if (defaultValue && defaultValue.length === 10) {
      if (!defaultValue.indexOf('-')) {
        const newDefVal = `${defaultValue.substring(8, 10)}/${defaultValue.substring(5, 7)}/${defaultValue.substring(0, 4)}`;
        return newDefVal;
      }
    }
    return defaultValue;
  }

  setMonthState = (value) => {
    let month;

    if (value) {
      if (value.length === 10) {
        month = value.substring(3, 5);
      } else {
        const { field } = this.props;
        console.error(`Valor da data inválido no campo ${field}. Formatos aceitos são 'dd/mm/aaaa' ou 'aaaa-mm-dd'`);
      }
    } else {
      month = new Date().getMonth();
    }

    this.setState({ month });
  }

  handleSelect = (date) => {
    const { onChange } = this.props;

    const day = date._d.getDate();
    const month = (date._d.getMonth() + 1);
    const year = date._d.getFullYear();

    const formattedDate = `${addExtraZero(day)}/${addExtraZero(month)}/${year}`;

    if (onChange) {
      onChange(year, month, day);
    }
    
    this.props.handleUpdates(formattedDate);
    this.setState({ open: false });
  }

  setMonthName = (date, month) => {

    if (date) {
      this.setState({ month: date._d.getMonth() });
    }

    const name = getMonthName(typeof month !== 'undefined'
      ? month
      : date._d.getMonth());

    const monthDescription = document.getElementsByClassName('rdr-MonthAndYear-month')[0];
    monthDescription.innerHTML = name;
  }

  render() {
    const { openCalendar, handleSelect, setMonthName } = this;
    const { open } = this.state;
    const { field } = this.props;

    const date = getField(field, 'value');

    return (
      <div className='argous-calendar'>
        <span onMouseUp={openCalendar}>{date ? date : 'dd/mm/aaaa'}</span>
        {open && (<CalDep date={date} onChange={handleSelect} onInit={setMonthName} />)}
      </div>
    );
  }
}

Calendar.propTypes = {
  field: PropTypes.string.isRequired,
  defaultValue: PropTypes.oneOfType([PropTypes.number, PropTypes.string, PropTypes.object, PropTypes.array]),
  ajax: PropTypes.bool,
  action: PropTypes.string,
  dados: PropTypes.object,
  sendUpdates: PropTypes.array,
  receiveUpdates: PropTypes.array,
  onFieldSuccess: PropTypes.func,
  onActionSuccess: PropTypes.func,
  onChange: PropTypes.func,
  onAllUpdated: PropTypes.func,
  fieldListener: PropTypes.func,
  destroyOnUnmount: PropTypes.bool
};

export default createField(Calendar, { disableInitField: true });
