import { ArgousAvatar } from '../';
import { setAnotherProgramField } from '../../fieldsRedux';
import { useStore } from '../../utils';
import { serverAction } from '../../utils/api';
import { Card, Typography, Grid, Button, Portal, Popper } from '@material-ui/core';
import React, { useState, useEffect, useRef, useCallback } from 'react';
import { IoIosCall } from 'react-icons/io';
import { MdMail, MdMessage } from 'react-icons/md';

function CartaoUsuario({ anchor, idUsuario }) {
  const tempo = useRef();
  const deixaMostrar = useRef(false);
  const limpar = useRef(true);
  const { usuarioLogado: { idUsuario: idUsuarioLogado } } = useStore();

  const [localAnchor, setLocalAnchor] = useState(anchor);
  const [localIdUsuario, setLocalIdUsuario] = useState(idUsuario);
  const [dados, setDados] = useState({});
  const [mostrar, setMostrar] = useState(false);

  useEffect(() => {
    if (anchor && idUsuario) {
      setLocalAnchor(anchor);
      setLocalIdUsuario(idUsuario);
      deixaMostrar.current = true;
    } else {
      clearTimeout(tempo.current);
      deixaMostrar.current = false;
      setTimeout(() => {
        if (!limpar.current) return;
        setDados({});
        setMostrar(false);
      }, 200);
    }
  }, [anchor, idUsuario, mostrar]);

  const busca = useCallback(async () => {
    if (!localIdUsuario || !localAnchor) {
      setMostrar(false);
      clearTimeout(tempo.current);
      return;
    }
    const res = await serverAction('getInfoUsuario', { idUsuario: localIdUsuario }, 'Argous.MntUsuarios', true);
    tempo.current = setTimeout(() => {
      if (deixaMostrar.current) {
        setDados(res);
        setMostrar(true);
      }
    }, 500);
  }, [localAnchor, localIdUsuario]);
  useEffect(() => void(busca()), [busca]);

  const retornaUser = () => {
    return { idUsuario, titulo: dados.dsNomeUsuario, subTitulo: dados.dsFuncaoUsuario, msgsNaoLidas: 0, online: false };
  };

  function mouseLeave() {
    setLocalAnchor(null);
    setLocalIdUsuario(null);
    clearTimeout(tempo.current);
    limpar.current = true;
  }

  function mouseEnter() {
    clearTimeout(limpar.current);
    limpar.current = false;
  }

  if (localIdUsuario && mostrar) return (
    <Portal>
      <Popper onClick={e => e.stopPropagation()} style={{ zIndex: 5 }} onMouseLeave={mouseLeave} onMouseEnter={mouseEnter} open={(!!anchor || !!localAnchor) && !!dados.dsNomeUsuario} anchorEl={anchor || localAnchor}>
        <Card variant='outlined' style={{ padding: '10px', maxWidth: '350px' }}>
          <Grid container spacing={1}>
            <Grid item xs='auto'>
              <ArgousAvatar style={{ width: '50px', height: '50px', margin: '10px' }} nome={dados.dsNomeUsuario} idUsuario={idUsuario || localIdUsuario} />
            </Grid>
            <Grid item xs='auto'>
              <Typography variant='h6' style={{ overflow: 'hidden', width: '250px', textOverflow: 'ellipsis', whiteSpace: 'nowrap' }}>{dados.dsNomeUsuario}</Typography>
              <Typography variant='body2' style={{ width: '250px' }}>{dados.dsFuncaoUsuario}</Typography>
              <Typography variant='caption' style={{ width: '250px' }}>{dados.dsSetor}</Typography>
            </Grid>
            {dados.dtExclusaoUsuario ? 
              <Typography variant='h6'>Usuário excluido em {dados.dtExclusaoUsuario}</Typography> 
              : 
              ((idUsuario && (idUsuarioLogado !== idUsuario)) || (localIdUsuario && (idUsuarioLogado !== localIdUsuario))) &&
              <React.Fragment>
                {dados.nrRamalUsuario &&
                  <Grid item xs>
                    <Button fullWidth variant='contained' href={`tel: ${dados.nrRamalUsuario}`} title={`Ramal: ${dados.nrRamalUsuario}`}><IoIosCall/>&nbsp;{dados.nrRamalUsuario}</Button>
                  </Grid>
                }
                <Grid item xs>
                  <Button fullWidth variant='contained' href={`mailto: ${dados.dsEmailUsuario}`} title={dados.dsEmailUsuario}><MdMail/>&nbsp;Email</Button>
                </Grid>
                <Grid item xs>
                  <Button fullWidth variant='contained' onClick={() => setAnotherProgramField('janelaChatOpen', retornaUser(), 'argousSystem')} title='Abrir chat de conversa'><MdMessage/>&nbsp;Mensagem</Button>
                </Grid>
              </React.Fragment>
            }
          </Grid>
        </Card>
      </Popper>
    </Portal>
  );
  return null;
}

export default CartaoUsuario;