import { enviaCompartilhamentoDePrograma, searchUsers } from '../../api/index';
import { formataNome, setError } from '../../utils/index';
import { IconButton, Grid, TextField, Button, InputAdornment, Typography, Collapse, Chip, makeStyles, Paper } from '@material-ui/core';
import { Autocomplete } from '@material-ui/lab';
import React, { useState, useEffect, forwardRef } from 'react';
import { MdLink } from 'react-icons/md';

const useStyles = makeStyles({
  fundoCompartilhar: {
    width: '500px',
    padding: '20px',
    marginTop: 5,
  }
});

const Compartilhar = forwardRef((props, ref) => {

  const [metodoCompartilhar, setMetodoCompartilhar] = useState('');
  const [mensagemEmail, setMensagemEmail] = useState('');
  const [email, setEmail] = useState([]);
  const [stringEmail, setStringEmail] = useState('');

  const [notificacao, setNotificacao] = useState([]);
  const [stringNotificacao, setStringNotificacao] = useState('');

  const [mensagemNotificação, setMensagemNotificacao] = useState('');
  const link = props.linkCompartilhamento;

  const [copiado, setCopiado] = useState(false);
  const [enviado, setEnviado] = useState(false);

  const [options, setOptions] = useState([]);

  const erroVazio = { email: '', stringEmail: '', notificacao: '', stringNotificacao: '' };
  const [erro, setErro] = useState(erroVazio);

  const classes = useStyles();

  function copiarParaAreaDeTransferencia() {
    let link = document.getElementById('link');
    link.select();
    link.setSelectionRange(0, 99999);
    document.execCommand('copy');
    setCopiado(true);
    setTimeout(() => {
      setCopiado(false);
    }, 3500);
  }

  useEffect(() => {
    if (stringEmail.length < 3 && stringNotificacao.length < 3) return;
    (async () => {
      const res = await searchUsers({ dsFiltroNome: metodoCompartilhar === 'email' ? stringEmail : stringNotificacao });
      if (res.status) {
        setOptions(res.users);
      } else setError(res.dsErro);
    })();
  }, [stringEmail, stringNotificacao, metodoCompartilhar]);

  async function enviaCompartilhamento(tpComp) {
    let temp = false;
    if (tpComp === 'E') {
      if (email.length === 0 && mensagemEmail === '') {
        setErro({ ...erro, email: 'Campo Obrigatório', stringEmail: 'Campo Obrigatório' });
        temp = true;
      } else {
        if (email.length === 0) {
          setErro({ ...erro, email: 'Campo Obrigatório' });
          temp = true;
        }
        if (mensagemEmail === '') {
          setErro({ ...erro, stringEmail: 'Campo Obrigatório' });
          temp = true;
        }
      }
    } else if (notificacao.length === 0 && mensagemNotificação === '') {
      setErro({ ...erro, notificacao: 'Campo Obrigatório', stringNotificacao: 'Campo Obrigatório' });
      temp = true;
    } else if (notificacao.length === 0) {
      setErro({ ...erro, notificacao: 'Campo Obrigatório' });
      temp = true;
    } else if (mensagemNotificação === '') {
      setErro({ ...erro, stringNotificacao: 'Campo Obrigatório' });
      temp = true;
    }

    if (temp) return;

    const not = notificacao.map(item => item[0]);
    const mail = email.map(item => item[0]);

    const dados = {
      mensagem: mensagemNotificação || mensagemEmail,
      usuarios: not.length > 0 ? not : mail,
      tpMensagem: tpComp,
      link: link
    };
    const res = await enviaCompartilhamentoDePrograma(dados);
    if (res.status) {
      setNotificacao([]);
      setEmail([]);
      setMensagemEmail('');
      setStringEmail('');
      setStringNotificacao('');
      setMensagemNotificacao('');
      setEnviado(true);
      setTimeout(() => {
        setEnviado(false);
      }, 3500);
    }
  }

  return (
    <Grid innerRef={ref} className={classes.fundoCompartilhar} component={Paper} elevation={2} onClick={e => e.stopPropagation()}>
      <Grid container spacing={2} style={{ position: 'relative', padding: '6px 12px', display: 'flex' }} justifyContent='center'>
        <Typography component='h3' style={{ fontSize: '24px', textAlign: 'center', marginTop: '0px' }}>Compartilhar</Typography>
        <Grid item xs={12} className='row'>
          <TextField
            label='Link'
            value={link}
            readOnly
            id='link'
            fullWidth
            variant='outlined'
            margin='dense'
            InputProps={{
              endAdornment: (
                <InputAdornment position='end'>
                  <IconButton onClick={() => copiarParaAreaDeTransferencia()} title='Copiar Link'>
                    <MdLink />
                  </IconButton>
                </InputAdornment>
              ),
            }} />
          <Collapse in={copiado}>
            <Typography variant='body2' component='div' color='primary'>Link copiado para a área de transferência!</Typography>
          </Collapse>
        </Grid>
        <Grid item xs={6} style={{ display: 'flex', justifyContent: 'center' }}>
          <IconButton size='medium' title='Enviar link por notificação' onClick={() => {
            setErro(erroVazio); setEmail([]); setStringEmail(''); setMetodoCompartilhar(metodoCompartilhar === 'notificacao' ? '' : 'notificacao'); 
          }} >
            <Typography component='span' variant='body1' className='icon-linecons-megaphone icon-lg'
              style={{
                fontSize: '45px',
                width: '75px',
                height: '75px',
                color: 'white',
                background: '#009C6A',
                borderRadius: '50px',
                border: '5px solid #33b088',
              }}
            />
          </IconButton>
        </Grid>
        <Grid item xs={6} style={{ display: 'flex', justifyContent: 'center' }}>
          <IconButton size='medium' title='Enviar link por email' onClick={() => {
            setErro(erroVazio); setNotificacao([]); setStringNotificacao(''); setMetodoCompartilhar(metodoCompartilhar === 'email' ? '' : 'email'); 
          }} >
            <Typography component='span' variant='body1' className='glyph-icon icon-envelope icon-lg'
              style={{
                fontSize: '45px',
                width: '75px',
                height: '75px',
                color: 'white',
                background: '#009C6A',
                borderRadius: '50px',
                border: '5px solid #33b088',
              }}
            />
          </IconButton>
        </Grid>
        <Grid item xs={6} style={{ display: 'flex', justifyContent: 'center', paddingTop: 0, marginBottom: '5px' }}>
          <Typography component='label' variant='body1'>Notificação</Typography>
        </Grid>
        <Grid item xs={6} style={{ display: 'flex', justifyContent: 'center', paddingTop: 0, marginBottom: '5px' }}>
          <Typography component='label' variant='body1'>Email</Typography>
        </Grid>
        <Grid item xs={12} style={{ padding: 0 }}>
          <Collapse in={metodoCompartilhar === 'notificacao'}>
            <Grid container spacing={2}>
              <Grid item xs={12}>
                <Autocomplete
                  multiple
                  value={notificacao}
                  options={options}
                  getOptionLabel={option => formataNome(option[1])}
                  size='small'
                  limitTags={1}
                  onChange={(_e, value) => setNotificacao(value)}
                  disableCloseOnSelect
                  disablePortal
                  noOptionsText={stringEmail.length < 3 && stringNotificacao.length < 3 ? 'Digite para começar a pesquisar' : 'Não Encontrado'}
                  renderTags={(value, getTagProps) => (
                    value.map((option, index) => (
                      <Chip
                        variant='outlined'
                        label={option[1] || option}
                        {...getTagProps({ index })} />
                    ))
                  )}
                  renderInput={params => (
                    <TextField
                      variant='outlined'
                      margin='dense'
                      label='Contatos:'
                      onChange={e => setStringNotificacao(e.target.value)}
                      placeholder={notificacao.length === 0 ? 'Selecionar' : undefined}
                      InputLabelProps={{ shrink: true }}
                      error={!!erro.notificacao}
                      helperText={erro.notificacao}
                      {...params}
                    />
                  )}
                />
              </Grid>
              <Grid item xs={12}>
                <TextField
                  variant='outlined'
                  margin='dense'
                  label='Mensagem:'
                  placeholder='Mensagem da Notificação'
                  fullWidth
                  value={mensagemNotificação}
                  onChange={e => setMensagemNotificacao(e.target.value)}
                  error={!!erro.stringNotificacao}
                  helperText={erro.stringNotificacao} />
              </Grid>
              <Grid container style={{ display: 'flex ' }} direction='row-reverse'>
                <Grid item xs={2}>
                  <Button color='primary' variant='contained' onClick={() => enviaCompartilhamento('N')}>Enviar</Button>
                </Grid>
                <Grid item xs={10}>
                  <Collapse in={enviado}>
                    <Typography variant='body2' component='div' color='primary'>Link enviado com sucesso!</Typography>
                  </Collapse>
                </Grid>
              </Grid>
            </Grid>
          </Collapse>
        </Grid>
        <Grid item xs={12} style={{ padding: 0 }}>
          <Collapse in={metodoCompartilhar === 'email'}>
            <Grid container spacing={2}>
              <Grid item xs={12}>
                <Autocomplete
                  multiple
                  freeSolo
                  value={email}
                  options={options}
                  getOptionLabel={option => option[2]}
                  size='small'
                  limitTags={1}
                  onChange={(_e, value) => setEmail(value)}
                  disableCloseOnSelect
                  disablePortal
                  noOptionsText={stringEmail.length < 3 && stringNotificacao.length < 3 ? 'Digite para começar a pesquisar' : 'Não Encontrado'}
                  renderTags={(value, getTagProps) => (
                    value.map((option, index) => (
                      <Chip
                        variant='outlined'
                        label={option[2] || option}
                        {...getTagProps({ index })} />
                    ))
                  )}
                  renderInput={params => (
                    <TextField
                      variant='outlined'
                      margin='dense'
                      label='Emails:'
                      onChange={e => setStringEmail(e.target.value)}
                      placeholder={email.length === 0 ? 'Selecionar' : undefined}
                      InputLabelProps={{ shrink: true }}
                      error={!!erro.email}
                      helperText={erro.email}
                      {...params}
                    />
                  )}
                />
              </Grid>
              <Grid item xs={12}>
                <TextField
                  multiline
                  label='Mensagem:'
                  minRows='2'
                  margin='dense'
                  variant='outlined'
                  fullWidth
                  placeholder='Mensagem do Email'
                  value={mensagemEmail}
                  onChange={e => setMensagemEmail(e.target.value)}
                  style={{ resize: 'vertical', height: '50px' }}
                  error={!!erro.stringEmail}
                  helperText={erro.stringEmail} />
              </Grid>
              <Grid container style={{ display: 'flex' }} direction='row-reverse'>
                <Grid item xs={2}>
                  <Button color='primary' variant='contained' onClick={() => enviaCompartilhamento('E')}>Enviar</Button>
                </Grid>
                <Grid item xs={10}>
                  <Collapse in={enviado}>
                    <Typography variant='body2' component='div' color='primary'>Link enviado com sucesso!</Typography>
                  </Collapse>
                </Grid>
              </Grid>
            </Grid>
          </Collapse>
        </Grid>
      </Grid>
    </Grid >
  );
});

export default Compartilhar;