import { Dialog, DialogTitle, DialogContent, DialogContentText, DialogActions, Button, DialogProps } from '@material-ui/core';
import React from 'react';

export interface BasicModalProps {
  /**
   * Label no botao de confirmar
   */
  okLabel: React.ReactNode,
  /**
   * Label no botao de cancelar
   */
  cancelLabel: React.ReactNode,
  /**
   * Titulo do modal
   */
  title: React.ReactNode,
  /**
   * Descricao do modal
   */
  description: React.ReactNode,
  /**
   * Props repassados ao componente dialog
   */
  dialogProps: Omit<DialogProps, 'open'>,
  /**
   * @ignore
   */
  proceed: Function,
  /**
   * @ignore
   */
  cancel: Function,
  /**
   * @ignore
   */
  open: boolean,
}

const BasicModal = ({
  okLabel = 'Ok',
  cancelLabel = 'Cancelar',
  title = '',
  description = '',
  open,
  proceed,
  dialogProps,
  cancel
}: BasicModalProps) => {
  return (
    <Dialog open={open} onClose={() => cancel(false)} {...dialogProps}>
      <DialogTitle>{title}</DialogTitle>
      <DialogContent>
        <DialogContentText component='div'>{description}</DialogContentText>
      </DialogContent>
      <DialogActions>
        <Button variant='contained' color='secondary' onClick={() => proceed(false)}>{cancelLabel}</Button>
        <Button variant='contained' color='primary' onClick={() => proceed(true)}>{okLabel}</Button>
      </DialogActions>
    </Dialog>
  );
};

export default BasicModal;
