import { CreateModalType, ModalProviderContextProps } from '.';
import BasicModal from './BasicModal';
import { setAnotherProgramField } from '../../fieldsRedux';
import { ArgousType } from '../../types';
import React, { createContext, useCallback, useEffect, useMemo, useState } from 'react';
import { connect } from 'react-redux';
import { useImmer } from 'use-immer';

// @ts-ignore
export const ModalProviderContext = createContext<ModalProviderContextProps>();

const ModalProvider = ({ children, argousSystem }: any) => {

  const [modals, setCurrentModal] = useImmer<unknown[]>([]);

  const close = useCallback((unmountDelay: number, index: number) => () => {
    setTimeout(() => setCurrentModal(modais => void(modais.splice(index, 1))), unmountDelay);
  }, [setCurrentModal]);

  const createModal = useCallback<CreateModalType>((Component, unmountDelay = 0) => (props) => {
    const promise = new Promise((resolve, reject) => {
      setCurrentModal(modais => void(modais.push(<ModalWrapper Component={Component} close={close(unmountDelay, modals.length)} resolve={resolve} reject={reject} {...props} />)));
    });

    return promise.then((result) => {
      close(unmountDelay, modals.length);
      return result;
    }, (result) => {
      close(unmountDelay, modals.length);
      return Promise.reject(result);
    });
  }, [close, modals.length, setCurrentModal]);

  const openModal = useMemo(() => argousSystem?.openModal?.value, [argousSystem]); 

  useEffect(() => {
    if (!openModal) return;
    const { reject, resolve, ...props } = openModal;
    createModal(BasicModal)(props).then(resolve as any, reject);
    setAnotherProgramField('openModal', undefined, 'argousSystem');
    //eslint-disable-next-line
  }, [openModal]);

  const context = useMemo(() => ({ createModal }), [createModal]);

  return (
    <ModalProviderContext.Provider value={context}>
      {children}
      {modals.map(Modal => Modal)}
    </ModalProviderContext.Provider>
  );
};

const mapStateToProps = (state: { argous: ArgousType }) => ({
  argousSystem: state.argous.fields.argousSystem, 
});

export default connect(mapStateToProps)(ModalProvider);

const ModalWrapper = ({ Component, reject, resolve, close, ...props }) => {
  const [show, setShow] = useState(true);

  const cancel = useCallback((reject: (reason?: any) => void) => <V extends unknown>(value: V) => {
    setShow(false);
    reject(value);
    close();
  }, [close]);

  const proceed = useCallback((resolve: (value: unknown) => void) => <V extends unknown>(value: V) => {
    setShow(false);
    resolve(value);
    close();
  }, [close]);

  return <Component
    cancel={cancel(reject)}
    proceed={proceed(resolve)}
    open={show}
    {...props}
  />;
};