import Button from '../MaterialButton/MaterialButton';
import { Dialog, DialogActions, DialogContent, DialogProps, DialogTitle } from '@material-ui/core';
import React from 'react';

export interface WrapperModalProps {
  okLabel: React.ReactNode,
  cancelLabel: React.ReactNode,
  title: React.ReactNode,
  dialogProps: Omit<DialogProps, 'open'>,
  /**
   * @ignore
   */
  proceed: Function,
  /**
   * @ignore
   */
  open: boolean,
}

const WrapperModal = (Component: (props: any) => JSX.Element) => ({ proceed, open, title, cancelLabel = 'Cancelar', okLabel = 'Ok', dialogProps, ...props }: WrapperModalProps) => {
  return <Dialog open={open} onClose={() => proceed(false)} {...dialogProps}>
    {title && <DialogTitle>{title}</DialogTitle>}
    <DialogContent>
      <Component {...props}/>
    </DialogContent>
    <DialogActions>
      {cancelLabel && <Button variant='contained' color='secondary' onClick={() => proceed(false)}>{cancelLabel}</Button>}
      {okLabel && <Button variant='contained' color='primary' onClick={() => proceed(true)}>{okLabel}</Button>}
    </DialogActions>
  </Dialog>;
};

export default WrapperModal;
