import { getFileFromLink, getLink } from '../../api';
import { CircularProgress } from '@material-ui/core';
import React, { useCallback, useEffect, useMemo, useState } from 'react';

const reqSvgs = require.context ('./icons', true, /\.svg$/);

const loadSvg = async (svg, path) => ({ [path.replace(/.\/|.svg/g, '')]: URL.createObjectURL(await (await fetch(svg?.default || svg)).blob()) });

const iconClasses = {
  'application/pdf': 'pdf',
  'application/msword': 'doc',
  'application/vnd.ms-word': 'doc',
  'application/vnd.oasis.opendocument.text': 'doc',
  'application/vnd.openxmlformats-officedocument.wordprocessingml': 'doc',
  'application/vnd.ms-excel': 'xls',
  'application/vnd.openxmlformats-officedocument.spreadsheetml': 'xls',
  'application/vnd.oasis.opendocument.spreadsheet': 'xls',
  'application/vnd.ms-powerpoint': 'ppt',
  'application/vnd.openxmlformats-officedocument.presentationml': 'ppt',
  'application/vnd.oasis.opendocument.presentation': 'ppt',
  'text/rtf': 'rtf',
  'text/plain': 'txt',
  'text/html': 'html',
  'text/css': 'css',
  'text/csv': 'csv',
  'application/json': 'json',
  'image/vnd.adobe.photoshop': 'psd',
  'image/photoshop': 'psd',
  'image/x-photoshop': 'psd',
  'image/psd': 'psd',
  'application/photoshop': 'psd',
  'application/psd': 'psd',
  'zz-application/zz-winassoc-psd': 'psd',
  'video/x-msvideo': 'avi',
  'application/gzip': 'zip1',
  'application/zip': 'zip',
  'application/acad': 'dwg',
  'image/vnd.dwg': 'dwg',
  'image/x-dwg': 'dwg',
  'application/octet-stream': 'exe',
  'audio/flac': 'fla',
  'image/jpeg': 'jpg',
  'image/jpg': 'jpg',
  'text/javascript': 'js',
  'audio/mpeg': 'mp3',
  'video/mpeg': 'mp4',
  'audio/mp4': 'mp4',
  'image/png': 'png',
  'application/xml': 'xml',
  'application/x-7z-compressed': 'zip1',
  'application/x-compressed-tar': 'zip1',
  'application/vnd.rar': 'zip1',
  'modelo/contrato': 'mod',
  image: 'image',
  audio: 'audio',
  video: 'video'
};

let icones;

(async () => {
  icones = await (await Promise.all(reqSvgs.keys()
    .map(path => loadSvg(reqSvgs(path), path))
  )).reduce((acc, item) => ({ ...acc, ...item }), {});
})();

// eslint-disable-next-line valid-jsdoc
/**
 * @typedef {'ai'|'audio'|'avi'|'css'|'csv'|'dbf'|'doc'|'dwg'|'exe'|'file'|'fla'|'html'|'image'|'iso'|'jpg'|'js'|'json'|'mod'|'mp3'|'mp4'|'pdf'|'png'|'ppt'|'psd'|'rtf'|'search'|'svg'|'txt'|'video'|'xls'|'xml'|'zip'|'zip1'} Extencoes
 * 
 * @typedef {object} Tipos
 * @property {Extencoes} type - Tipos de arquivos suportados
 * @property {string} mimeType - MimeType do arquivo
 * @property {string} fileName - Nome do arquivo
 * 
 * @param {React.ImgHTMLAttributes<HTMLImageElement> & Tipos} props - props 
 * @returns {JSX.Element} retorno
 */

export const IconsComponent = ({ type, fileName, mimeType, idDocumento, link, ...props }) => {
  /** @type {[FileIcons, React.Dispatch<FileIcons>]} */
  const [state, setState] = useState(icones);
  const [preview, setPreview] = useState();


  const setarState = useCallback(() => {
    if (state) return;
    if (icones) setState(icones);
    else setTimeout(setarState, 200);
  }, [state]);

  useEffect(setarState, [setarState]);

  const selectedImage = useMemo(() => {
    if (type) return type;
    /** @type {Extencoes} */
    const extension = fileName?.match(/\.([^.]*)$/)?.at(1);
    if (!state) return '';
    if (Object.keys(state).includes(extension)) return extension;
    for (let key in iconClasses) 
      if (Object.prototype.hasOwnProperty.call(iconClasses, key)) 
        if (mimeType?.search(key) === 0) 
          return iconClasses[key];
    return 'file';
  }, [type, mimeType, fileName, state]);

  const buscaImagemPreview = useCallback(async () => {
    if (idDocumento && (mimeType?.includes('image'))) {
      const file = await getFileFromLink(await getLink(idDocumento));
      setPreview(file);
    }
  }, [mimeType, idDocumento]);
  useEffect(() => void(buscaImagemPreview()), [buscaImagemPreview]);

  if (!state) return <CircularProgress disableShrink/>;
  return <>
    {(preview) ? (
      <img src={preview} alt={type} {...props} style={{ position: 'absolute', left: 0, right: 0, top: 0, bottom: 0, width: '100%', height: '100%', objectFit: 'contain' }} />
    ) : (
      <img src={state[selectedImage]} alt={type} {...props} />
    )}
  </>;
};

/**
 * @typedef {object} FileIcons - Objeto de icones
 * @property {JSX.Element} ai -Icone de arquivos do Ilustrator
 * @property {JSX.Element} audio -Icone genérico de Audio
 * @property {JSX.Element} avi -Icone de videos AVI
 * @property {JSX.Element} css -Icone de arquivos CSS
 * @property {JSX.Element} csv -Icone de arquivos CSV
 * @property {JSX.Element} dbf -Icone de arquivos DBF
 * @property {JSX.Element} doc -Icone de arquivos Word
 * @property {JSX.Element} dwg -Icone de arquivos DWG
 * @property {JSX.Element} exe -Icone de arquivos EXE
 * @property {JSX.Element} file -Icone de arquivos genéricos
 * @property {JSX.Element} fla -Icone de arquivos FLA
 * @property {JSX.Element} html -Icone de arquivos HTML
 * @property {JSX.Element} image -Icone genérico de Imagens
 * @property {JSX.Element} iso -Icone de imagens ISO
 * @property {JSX.Element} jpg -Icone de imagens JPG
 * @property {JSX.Element} js -Icone de arquivos Javascript
 * @property {JSX.Element} json -Icone de arquivos JSON
 * @property {JSX.Element} mod -Icone de modelos de documentos
 * @property {JSX.Element} mp3 -Icone de músicas MP3
 * @property {JSX.Element} mp4 -Icone de vídeos MP4
 * @property {JSX.Element} pdf -Icone de arquivos PDP
 * @property {JSX.Element} png -Icone de imagens PNG
 * @property {JSX.Element} ppt -Icone de arquivos do Power Point
 * @property {JSX.Element} psd -Icone de arquivos do Photoshop
 * @property {JSX.Element} rtf -Icone de arquivos RTF
 * @property {JSX.Element} search -Icone genérico de Busca
 * @property {JSX.Element} svg -Icone de arquivos SVG
 * @property {JSX.Element} txt -Icone de arquivos de Texto
 * @property {JSX.Element} video -Icone genérico de Vídeo
 * @property {JSX.Element} xls -Icone de arquivos do Excel
 * @property {JSX.Element} xml -Icone de arquivos XML
 * @property {JSX.Element} zip -Icone de arquivos ZIP
 * @property {JSX.Element} zip1 -Icone genérico de arquivos comprimidos
 */

/** @type {FileIcons}  */
export const Icons = reqSvgs.keys().reduce((images, path) => {
  const key = path.replace(/.\/|.svg/g, '');
  images[key] = <IconsComponent type={key}/>;
  return images;
}, {});

export default Icons;