import { serverAction, setError } from '../../utils/api';
import { Box, CircularProgress, makeStyles } from '@material-ui/core';
import React, { useEffect, useRef, useState } from 'react';


const useStyles = makeStyles(theme => ({
  iframe: {
    width: '100%',
    height: '100%',
    border: 'none',
    background: theme.palette.background.default,
    '&:focus': {
      outline: 'none'
    },
    '&[seamless]': {
      display: 'block'
    }
  },
  restricSize: {
    maxWidth: '100%',
    maxHeight: '100%'
  }
}));

let onchange = () => undefined;


const Visualizacao = ({ idDocumento, nrRevisao, setVisualizando, proceed, URL: documento, linkPronto, tokenSistema, ariaLabel }) => {

  const classes = useStyles();
  const blockLoop = useRef(null);
  const [{ type }, setArquivo] = useState({});
  const tagType = type?.match(/^[^/]+/)?.at(0);
  const [link, setLink] = useState(linkPronto);

  const preparaLink = async () => {
    if (!linkPronto && ((!idDocumento && !documento) || (documento === blockLoop.current))) {
      return;
    }
 
    const objGetLinkDocumento = { idDocumento, nrRevisao, dsToken: tokenSistema?.toString() };
    const { link: res } = linkPronto ? { link: { URL: linkPronto } } : documento ? { link: { URL: documento } } : await serverAction('getLinkDocumento', objGetLinkDocumento, 'CtrlDocumentos', true);
    try {
      const file = await (await fetch(res?.URL, {
        method: 'post',
        headers: { lgTkn: tokenSistema }
      })).blob();
      const url = URL.createObjectURL(file);
      setVisualizando(v => ({ ...v, URL: url }));
      blockLoop.current = url;
      setLink(url);
      setArquivo({ ...res, type: file?.type });
    } catch (error) {
      setError('Não foi possível visualizar este documento.');
    }
  };

  // eslint-disable-next-line react-hooks/exhaustive-deps
  useEffect(() => preparaLink(), []);

  useEffect(() => {
    onchange = function(evt) {
      let v = 'visible', h = 'hidden', evtMap = { focus: v, focusin: v, pageshow: v, blur: h, focusout: h, pagehide: h };
      evt = evt || window.event;
      if (evt.type in evtMap) document.body.style.opacity = evtMap[evt.type];
      else document.body.style.opacity = this[hidden] ? 0 : 1;
    };
    let hidden = 'hidden';
    if (hidden in document) document.addEventListener('visibilitychange', onchange);
    else if ((hidden = 'mozHidden') in document) document.addEventListener('mozvisibilitychange', onchange);
    else if ((hidden = 'webkitHidden') in document) document.addEventListener('webkitvisibilitychange', onchange);
    else if ((hidden = 'msHidden') in document) document.addEventListener('msvisibilitychange', onchange);
    else if ('onfocusin' in document) document.onfocusin = document.onfocusout = onchange;
    else window.onpageshow = window.onpagehide = window.onfocus = window.onblur = onchange;
    if (document[hidden] !== undefined) onchange({ type: document[hidden] ? 'blur' : 'focus' });

    window.onkeydown = e => {
      if (e.key === 'Escape') proceed(undefined);
      if (e.key === 'F12') return false;
      if (e.ctrlKey && e.shiftKey && e.key === 'I') return false;
      if (e.ctrlKey && e.shiftKey && e.key === 'C') return false;
      if (e.ctrlKey && e.shiftKey && e.key === 'J') return false;
      if (e.ctrlKey && e.key === 'u') return false;
    };
    return () => {
      window.onkeydown = undefined;
      onchange = () => undefined;
    };
  }, [proceed]);

  switch (tagType) {
    case 'audio':
      return <audio className={classes.restricSize} aria-label={ariaLabel} src={link} controls controlsList='nodownload' />;
    case 'image':
      return <img className={classes.restricSize} alt={ariaLabel || 'Imagem'} aria-label={ariaLabel} src={link} onClick={e => e.preventDefault()}/>;
    case 'video':
      return <video className={classes.restricSize} aria-label={ariaLabel} controls controlsList='nodownload' autoPlay src={link}/>;
    default:
      return link ? <iframe className={classes.iframe} aria-label={ariaLabel} title='Iframe' src={`${link}#toolbar=0`}/> : <Box justifyContent='center' alignItems='center' display='flex'><CircularProgress disableShrink/></Box>;
  }
};


export default Visualizacao;