import { getDroppedOrSelectedFiles } from './Html5FileSelector';
import { FilePreparer } from '../';
import { Box, CircularProgress } from '@material-ui/core';
import React, { useState, Children, isValidElement, cloneElement, forwardRef } from 'react';
import { MdFileUpload } from 'react-icons/md';

const DropArea = ({ children, onChange, style, ...props }, ref) => {
  
  const [abrir, setAbrir] = useState(false);
  const [loading, setLoading] = useState(false);

  /** @param {React.DragEvent<HTMLDivElement>} e */
  const dropArquivo = async e => {
    e.preventDefault();
    setLoading(true);

    const finalArray = await getDroppedOrSelectedFiles(e);

    // console.log(temp, temp.reduce((acc, { fileObject, fullPath }) => {
    //   const fileName = fullPath.match(/\/([^\/]*)$/)?.at(1);
    //   const objectPath = fullPath.replace(fileName, '').substring(1).slice(0, -1);
    //   return set(acc, objectPath.length ? [...objectPath.split('/'), fileName] : fileName, fileObject)
    // }, {}));

    // const length = e.dataTransfer.items.length, 
    //     i = 0, itens = e.dataTransfer.items, finalArray = [];

    // for (i; i < length; i++) {
    //   const item = itens[i];
    //   if (item.kind === 'file') finalArray.push(item.getAsFile())
    // }

    // onChange && onChange(finalArray)
    // if (ref?.current) ref.current.preparaArquivo(finalArray)

    onChange && onChange(finalArray);
    if (ref?.current) ref.current.preparaArquivo(finalArray);

    setLoading(false);
    setAbrir(false);
  };

  return (
    <div onDragEnter={() => setAbrir(true)} style={{ position: 'relative', ...style }} {...props}>
      {abrir && (!ref?.current || (ref.current && !ref.current.loading)) && (
        <div 
          onDrop={dropArquivo} 
          onDragOver={e => e.preventDefault()} 
          onDragLeave={() => setAbrir(false)} 
          style={{ position: 'absolute', border: '2px dashed', background: 'rgba(0, 0, 0, 0.5)', zIndex: 5, width: '100%', height: '100%', display: 'grid', placeItems: 'center' }}
        >
          {(loading) ? 
            <Box display='flex' justifyContent='center' alignItems='center' height='100%'>
              <CircularProgress disableShrink />
            </Box>
            :
            <MdFileUpload style={{ width: '100%', height: '99%' }} />
          }
        </div>
      )} 
      {deepMap(children, (filho) => filho.type === FilePreparer ? cloneElement(filho, { ...filho.props, ref }) : filho)}
    </div>
  );
};

export default forwardRef(DropArea);


const deepMap = (children, deepMapFn) => {
  return Children.toArray(children).map((child, index, mapChildren) => {
    if (isValidElement(child) && hasComplexChildren(child)) {
      return deepMapFn(
        cloneElement(child, {
          ...child.props,
          children: deepMap(child.props.children, deepMapFn),
        }),
      );
    }
    return deepMapFn(child, index, mapChildren);
  },
  );
};

const hasComplexChildren = (element) =>
  isValidElement(element) &&
  hasChildren(element) &&
  Children.toArray(element.props.children).reduce((response, child) => response || isValidElement(child), false);

const hasChildren = (element) => isValidElement(element) && element.props.children;