import { ImageResize } from '../';
import { Button } from '@material-ui/core';
import React, { forwardRef, useImperativeHandle, useRef, useState } from 'react';

const ImageResizeButton = forwardRef(({ children, ImageResizeProps, onImage, Component = Button, ...props }, ref) => {
  const [enviandoFoto, setEnviandoFoto] = useState(undefined);

  const inputRef = useRef();

  const handleClick = () => inputRef.current && inputRef.current.click();
  
  const handleChange = e => setEnviandoFoto(e.target.files[0]);  

  const handleImage = image => {
    onImage(dataURLtoFile(image, enviandoFoto.name), image);
    handleClose();
  };

  const handleClose = () => {
    setEnviandoFoto(undefined);
    clearInputFile(inputRef.current);
  };

  useImperativeHandle(ref, () => ({
    handleChange
  }), []);

  return <>
    <Component onClick={handleClick} {...props}>{children}</Component>
    <input ref={inputRef} type='file' style={{ display: 'none' }} onChange={handleChange} accept='image/*'/>
    <ImageResize imagem={enviandoFoto} open={!!enviandoFoto} onImage={handleImage} onClose={handleClose} { ...ImageResizeProps }/>
  </>;
});

export default ImageResizeButton;


function clearInputFile(f) {
  if (f?.value) {
    try {
      f.value = '';
    } catch (e) { 
      let form = document.createElement('form'),
          parentNode = f.parentNode, ref = f.nextSibling;
      form.appendChild(f);
      form.reset();
      parentNode.insertBefore(f, ref);
    }
  }
}


function dataURLtoFile(dataurl, filename) {
  var arr = dataurl.split(','),
      mime = arr[0].match(/:(.*?);/)[1],
      bstr = atob(arr[1]), 
      n = bstr.length, 
      u8arr = new Uint8Array(n);
            
  while (n--) {
    u8arr[n] = bstr.charCodeAt(n);
  }
        
  return new File([u8arr], filename, { type: mime });
}