import { createField, getField } from '../../fieldsRedux';
import PropTypes from 'prop-types';
import React, { Component } from 'react';
import MaskedInput from 'react-maskedinput';

/**
 * @deprecated 
 */
class Input extends Component {
  confs = '';

  componentWillUnmount() {
    if (this.props.stopWriting) {
      clearTimeout(this.stopWritingTimeout);
    }
  }

  onChange = (event) => {
    const { onChange } = this.props;

    let value = this.handleValue(event.target);

    value = this.handleWithTypeValues(value);

    if (onChange) {
      onChange(value);
    }

    this.props.handleUpdates(value);
    this.stopWritingHandler(value);
  }

  stopWritingHandler = (value) => {
    const { stopWriting, stopWritingTime = 2000 } = this.props;

    if (stopWriting) {
      clearTimeout(this.stopWritingTimeout);
      this.stopWritingTimeout = setTimeout(() => {
        stopWriting(value);
      }, stopWritingTime);
    }
  }

  handleWithTypeValues = (value) => {
    const { field, type, optionValue } = this.props;

    if (type === 'checkbox') {
      const fieldValue = getField(field, 'value') || [];
      const index = fieldValue.indexOf(optionValue);

      if (index < 0) {
        fieldValue.push(optionValue);
      } else {
        fieldValue.splice(index, 1);
      }

      value = fieldValue;
    }

    if (type === 'radio') {
      value = optionValue;
    }

    return value;
  }

  handleValue = (target) => {
    const confs = this.stateOptions();

    return confs.alias === 'check'
      ? target.checked
      : target.value;
  }

  handleClassName = () => {
    const {
      className = ''
    } = this.props;
    const confs = this.stateOptions();

    let cn = '';

    if (confs.alias === 'text' && !className) {
      cn += ' form-control ';
    }

    cn += className;

    return cn;
  }

  checkInputProps = () => {
    const {
      field,
      id,
      type = 'text',
      name = '',
      placeholder,
      min,
      max,
      step,
      maxLength,
      minLength,
      autoFocus,
      disabled,
      readOnly,
      required,
      style = {},
      optionValue
    } = this.props;

    const confs = this.stateOptions();

    let value = getField(field, 'value');

    let inputProps = {};

    switch (confs.alias) {
      case 'button':

        break;
      case 'check':
        value = type === 'radio' ? value.toString() : value;
        inputProps['checked'] = value ? value.indexOf(optionValue) < 0 ? false : true : false;
        break;
      case 'color':
        inputProps['value'] = value;
        break;
      case 'hidden':
        inputProps['value'] = value;
        break;
      default:
        inputProps['min'] = min;
        inputProps['max'] = max;
        inputProps['step'] = step;
        inputProps['maxLength'] = maxLength;
        inputProps['minLength'] = minLength;
        inputProps['value'] = value;
        inputProps['placeholder'] = placeholder;
    }

    inputProps['readOnly'] = readOnly;
    inputProps['required'] = required;
    inputProps['disabled'] = disabled;
    inputProps['autoFocus'] = autoFocus;
    inputProps['id'] = id;
    inputProps['type'] = type;
    inputProps['name'] = name;
    inputProps['className'] = this.handleClassName();
    inputProps['style'] = style;

    inputProps = {
      ...inputProps,
      ...this.events()
    };

    return inputProps;
  }

  render() {
    const { checkInputProps } = this;
    const { mask } = this.props;

    const inputProps = checkInputProps();

    return !mask
      ? (<input {...inputProps} />)
      : (<MaskedInput mask={mask} {...inputProps} />);
  }

  stateOptions = () => {
    const { confs } = this;
    const { type } = this.props;

    if (confs) {
      return confs;
    }

    const confsAux = {};

    switch (type) {
      case 'time':
        confsAux['alias'] = 'text';
        break;
      case 'date':
        confsAux['alias'] = 'text';
        break;
      case 'datetime-local':
        confsAux['alias'] = 'text';
        break;
      case 'email':
        confsAux['alias'] = 'text';
        break;
      case 'url':
        confsAux['alias'] = 'text';
        break;
      case 'month':
        confsAux['alias'] = 'text';
        break;
      case 'week':
        confsAux['alias'] = 'text';
        break;
      case 'number':
        confsAux['alias'] = 'text';
        break;
      case 'password':
        confsAux['alias'] = 'text';
        break;
      case 'search':
        confsAux['alias'] = 'text';
        break;
      case 'button':
        confsAux['alias'] = 'button';
        break;
      case 'submit':
        confsAux['alias'] = 'button';
        break;
      case 'file':
        confsAux['alias'] = 'button';
        break;
      case 'checkbox':
        confsAux['alias'] = 'check';
        break;
      case 'radio':
        confsAux['alias'] = 'check';
        break;
      case 'color':
        confsAux['alias'] = 'color';
        break;
      case 'hidden':
        confsAux['alias'] = 'hidden';
        break;
      case 'range':
        confsAux['alias'] = 'range';
        break;
      default:
        confsAux['alias'] = 'text';
    }

    this.confs = confsAux;

    return confsAux;
  }

  events = () => {
    const {
      onFocus,
      onBlur,
      onKeyDown,
      onKeyPress,
      onKeyUp,
      onMouseEnter,
      onMouseOver,
      onMouseMove,
      onMouseDown,
      onMouseUp,
      onClick,
      onDoubleClick,
      onMouseLeave,
      onMouseOut
    } = this.props;

    const onChange = this.onChange;

    return {
      onChange,
      onFocus,
      onBlur,
      onKeyDown,
      onKeyPress,
      onKeyUp,
      onMouseEnter,
      onMouseOver,
      onMouseMove,
      onMouseDown,
      onMouseUp,
      onClick,
      onDoubleClick,
      onMouseLeave,
      onMouseOut
    };
  }
}

Input.propTypes = {
  field: PropTypes.string.isRequired,
  defaultValue: PropTypes.oneOfType([PropTypes.number, PropTypes.string, PropTypes.object, PropTypes.array]),
  ajax: PropTypes.bool,
  onChange: PropTypes.func,
  action: PropTypes.string,
  dados: PropTypes.object,
  receiveUpdates: PropTypes.array,
  sendUpdates: PropTypes.array,
  onFieldSuccess: PropTypes.func,
  onActionSuccess: PropTypes.func,
  onAllUpdated: PropTypes.func,
  fieldListener: PropTypes.func,
  destroyOnUnmount: PropTypes.bool,
  mask: PropTypes.string,
  type: PropTypes.oneOf([
    'time',
    'date',
    'datetime-local',
    'email',
    'url',
    'month',
    'week',
    'number',
    'password',
    'search',
    'button',
    'submit',
    'file',
    'checkbox',
    'radio',
    'color',
    'hidden',
    'range',
    'text'
  ]),
  placeholder: PropTypes.string,
  className: PropTypes.string,
  style: PropTypes.object,
  id: PropTypes.string,
  name: PropTypes.string,
  min: PropTypes.string,
  max: PropTypes.string,
  step: PropTypes.string,
  maxLength: PropTypes.string,
  minLength: PropTypes.string,
  autoFocus: PropTypes.bool,
  disabled: PropTypes.bool,
  readOnly: PropTypes.bool,
  required: PropTypes.bool,
  onFocus: PropTypes.func,
  onBlur: PropTypes.func,
  onKeyDown: PropTypes.func,
  onKeyPress: PropTypes.func,
  onKeyUp: PropTypes.func,
  onMouseEnter: PropTypes.func,
  onMouseOver: PropTypes.func,
  onMouseMove: PropTypes.func,
  onMouseDown: PropTypes.func,
  onMouseUp: PropTypes.func,
  onClick: PropTypes.func,
  onDoubleClick: PropTypes.func,
  onMouseLeave: PropTypes.func,
  onMouseOut: PropTypes.func,
  stopWriting: PropTypes.func,
  stopWritingTime: PropTypes.number
};

export default createField(Input);
