import { createField } from '../../fieldsRedux';
import { Chip, Grid, TextField } from '@material-ui/core';
import { Autocomplete } from '@material-ui/lab';
import { isArray } from 'lodash';
import PropTypes from 'prop-types';
import React, { Component } from 'react';

class InputTags extends Component {
  constructor(props) {
    super(props);
    const propValue = this.props.getField().value;
    if (isArray(propValue)) this.props.setField({ value: propValue });
    else {
      try {
        this.props.setField({ value: JSON.parse(propValue) });
      } catch (error) {
        this.props.setField({ value: [] });
      }
    }
  }

  handleChange = (_, value, reason) => {
    if (reason === 'create-option') {
      let text = value.pop();
      if (this.props.getField().value?.findIndex(item => item.text === text || item.id === text) === -1)
        this.props.setField({ value: [...value, { id: text, text }] });
    } else this.props.setField({ value });
  }

  handleDelete = i => () => {
    const tags = this.props.getField().value || [];
    this.props.setField({ value: tags.filter((tag, index) => index !== i) });
  }

  checkProps = () => {
    let tags = this.props.getField().value || [];
    if (typeof tags === 'string') {
      try {
        tags = JSON.parse(tags);
      } catch (e) { }
    }

    return { tags, handleChange: this.handleChange, handleDelete: this.handleDelete, ...this.props };
  }

  render() {
    const { handleChange, handleDelete, tags, handleInputChange, label, placeholder, suggestions, ...props } = this.checkProps();

    return (
      <Grid container spacing={1}>      
        <Grid item xs={12}>
          {tags.map((item, index) => (
            <Chip key={index} label={item.text} onDelete={handleDelete(index)} />
          ))}
        </Grid>    
        <Grid item xs={12}>
          <Autocomplete
            onInputChange={(_, value) => handleInputChange(value)}
            multiple
            freeSolo
            disableClearable
            value={tags}
            options={suggestions?.filter((searched) => tags.findIndex(item => item.id?.localeCompare(searched.id, undefined, { sensitivity: 'accent' }) === 0) < 0)}
            getOptionLabel={option => option.text || option}
            onChange={handleChange}
            noOptionsText='Não Encontrado'
            size='small'     
            renderTags={() => null}
            renderInput={params => (
              <TextField
                fullWidth
                margin='dense'
                color='secondary'
                label={label}
                placeholder={placeholder}
                {...params}
              />
            )}
            {...props}
          />
        </Grid>
      </Grid>
    );
  }
}

InputTags.propTypes = {
  field: PropTypes.string.isRequired,
  defaultValue: PropTypes.oneOfType([PropTypes.number, PropTypes.string, PropTypes.object, PropTypes.array]),
  fieldListener: PropTypes.func,
  destroyOnUnmount: PropTypes.bool
};

export default createField(InputTags);
