import { makeStyles, capitalize, alpha } from '@material-ui/core';
import MaterialButton from '@material-ui/core/Button';
import clsx from 'clsx';
import PropTypes from 'prop-types';
import React from 'react';

const useStyles = makeStyles(theme => ({
  containedInfo: {
    color: theme.palette.info.contrastText,
    backgroundColor: theme.palette.info.main,
    '&:hover': {
      backgroundColor: theme.palette.info.dark,
      '@media (hover: none)': {
        backgroundColor: theme.palette.info.main,
      },
    }
  },
  outlinedInfo: {
    color: theme.palette.info.main,
    border: `1px solid ${alpha(theme.palette.info.main, 0.5)}`,
    '&:hover': {
      border: `1px solid ${theme.palette.info.main}`,
      backgroundColor: alpha(theme.palette.info.main, theme.palette.action.hoverOpacity),
      '@media (hover: none)': {
        backgroundColor: 'transparent',
      },
    }
  },
  textInfo: {
    color: theme.palette.info.main,
    '&:hover': {
      backgroundColor: alpha(theme.palette.info.main, theme.palette.action.hoverOpacity),
      '@media (hover: none)': {
        backgroundColor: 'transparent',
      },
    }
  },
  containedSuccess: {
    color: theme.palette.success.contrastText,
    backgroundColor: theme.palette.success.main,
    '&:hover': {
      backgroundColor: theme.palette.success.dark,
      '@media (hover: none)': {
        backgroundColor: theme.palette.success.main,
      },
    }
  },
  outlinedSuccess: {
    color: theme.palette.success.main,
    border: `1px solid ${alpha(theme.palette.success.main, 0.5)}`,
    '&:hover': {
      border: `1px solid ${theme.palette.success.main}`,
      backgroundColor: alpha(theme.palette.success.main, theme.palette.action.hoverOpacity),
      '@media (hover: none)': {
        backgroundColor: 'transparent',
      },
    }
  },
  textSuccess: {
    color: theme.palette.success.main,
    '&:hover': {
      backgroundColor: alpha(theme.palette.success.main, theme.palette.action.hoverOpacity),
      '@media (hover: none)': {
        backgroundColor: 'transparent',
      },
    }
  },
  containedWarning: {
    color: theme.palette.warning.contrastText,
    backgroundColor: theme.palette.warning.main,
    '&:hover': {
      backgroundColor: theme.palette.warning.dark,
      '@media (hover: none)': {
        backgroundColor: theme.palette.warning.main,
      },
    }
  },
  outlinedWarning: {
    color: theme.palette.warning.main,
    border: `1px solid ${alpha(theme.palette.warning.main, 0.5)}`,
    '&:hover': {
      border: `1px solid ${theme.palette.warning.main}`,
      backgroundColor: alpha(theme.palette.warning.main, theme.palette.action.hoverOpacity),
      '@media (hover: none)': {
        backgroundColor: 'transparent',
      },
    }
  },
  textWarning: {
    color: theme.palette.warning.main,
    '&:hover': {
      backgroundColor: alpha(theme.palette.warning.main, theme.palette.action.hoverOpacity),
      '@media (hover: none)': {
        backgroundColor: 'transparent',
      },
    }
  },
  containedDanger: {
    color: theme.palette.error.contrastText,
    backgroundColor: theme.palette.error.main,
    '&:hover': {
      backgroundColor: theme.palette.error.dark,
      '@media (hover: none)': {
        backgroundColor: theme.palette.error.main,
      },
    }
  },
  outlinedDanger: {
    color: theme.palette.error.main,
    border: `1px solid ${alpha(theme.palette.error.main, 0.5)}`,
    '&:hover': {
      border: `1px solid ${theme.palette.error.main}`,
      backgroundColor: alpha(theme.palette.error.main, theme.palette.action.hoverOpacity),
      '@media (hover: none)': {
        backgroundColor: 'transparent',
      },
    }
  },
  textDanger: {
    color: theme.palette.error.main,
    '&:hover': {
      backgroundColor: alpha(theme.palette.error.main, theme.palette.action.hoverOpacity),
      '@media (hover: none)': {
        backgroundColor: 'transparent',
      },
    }
  },
}));

export default function Button({ color = 'default', variant = 'text', className, ...props }) {

  const classes = useStyles();

  function checarCor() {
    switch (color) {
      case 'primary':
        return 'primary';
      case 'secondary':
        return 'secondary';
      case 'inherit':
        return 'inherit';
      default:
        return 'default';
    }
  }

  return (
    <MaterialButton
      variant={variant}
      color={checarCor()}
      className={clsx(classes[variant], { [classes[`${variant}${capitalize(color)}`]]: color !== 'default' && color !== 'inherit' && color !== 'primary' && color !== 'secondary', }, className)}
      {...props}
    >
      {props.children}
    </MaterialButton>
  );
}

Button.propTypes = {
  children: PropTypes.node.isRequired,
  classes: PropTypes.object,
  className: PropTypes.string,
  color: PropTypes.oneOf(['default', 'inherit', 'primary', 'secondary', 'info', 'success', 'warning', 'danger']),
  component: PropTypes.elementType,
  disabled: PropTypes.bool,
  disableElevation: PropTypes.bool,
  disableFocusRipple: PropTypes.bool,
  disableRipple: PropTypes.bool,
  endIcon: PropTypes.node,
  focusVisibleClassName: PropTypes.string,
  fullWidth: PropTypes.bool,
  href: PropTypes.string,
  size: PropTypes.oneOf(['small', 'medium', 'large']),
  startIcon: PropTypes.node,
  type: PropTypes.string,
  variant: PropTypes.oneOf(['text', 'outlined', 'contained']),
};