import { createField } from '../../fieldsRedux';
import PropTypes from 'prop-types';
import React, { Component } from 'react';
import { FaCheck } from 'react-icons/fa';

/**
 * @deprecated 
 */
class MenuBar extends Component {
  state = {
    liWidths: [],
    guidePositionX: 0,
    guidePositionY: 0,
  }

  componentDidMount() {
    this.setLiWidths(() => {
      this.initSelectedGuide();
      this.updateWindowDimensions();
      window.addEventListener('resize', this.updateWindowDimensions);
    });
  }

  componentWillUnmount() {
    window.removeEventListener('resize', this.updateWindowDimensions);
  }

  componentDidUpdate(prevProps) {
    const { guides = [], getField, getPreviousField } = this.props;
    const { selectedGuide, selectedGuideRow } = getField();
    const { selectedGuide: selectedGuidePrev, selectedGuideRow: selectedGuideRowPrev } = getPreviousField(prevProps);

    if (selectedGuide !== selectedGuidePrev || selectedGuideRow !== selectedGuideRowPrev) {
      if (guides[selectedGuideRow]) {
        this.alterGuide(guides[selectedGuideRow].indexOf(selectedGuide));
      }
    }
  }

  initSelectedGuide = () => {
    const { guides = [], setField, getField } = this.props;
    const { selectedGuide, selectedGuideRow } = getField();

    if (guides.length > 0 && typeof selectedGuide !== 'undefined' && typeof selectedGuideRow !== 'undefined') {
      this.alterGuide(guides[selectedGuideRow].indexOf(selectedGuide));
    } else if (guides.length > 0) {
      setField({ selectedGuide: guides[0][0], selectedGuideRow: 0 });
    }
  }

  setLiWidths = (callback) => {
    const { guides = [] } = this.props;
    const liWidths = [];

    for (let g of guides) {
      liWidths.push(100 / g.length);
    }
    this.setState({ liWidths }, callback);
  }

  updateWindowDimensions = () => {
    const { guides = [], getField } = this.props;
    const { selectedGuide, selectedGuideRow } = getField();

    this.alterGuide(guides[selectedGuideRow].indexOf(selectedGuide));
  }

  alterGuide = (index) => {
    const { liWidths } = this.state;
    const { positionSpace = 16.5, getField } = this.props;
    const { selectedGuideRow } = getField();

    if (this.fpLineGuide) {
      const lineGuideWidth = this.fpLineGuide.offsetWidth;
      const guidePositionX = (((lineGuideWidth * liWidths[selectedGuideRow]) / 100) * index);

      let heightChildrens = 0;
      for (let i = 0; i < this.fpLineGuide.children.length; i++) {

        if (i === selectedGuideRow) {
          break;
        }

        heightChildrens += this.fpLineGuide.children[i].offsetHeight;
      }

      const guidePositionY = heightChildrens + selectedGuideRow * positionSpace;

      this.setState({ guidePositionX, guidePositionY });
    }
  }

  render() {
    const { liWidths, guidePositionX, guidePositionY } = this.state;
    const { movingTabColor, guides = [], getField, setField, style, className } = this.props;
    const { selectedGuide, selectedGuideRow, preenchidos = [] } = getField();

    return (
      <div ref={input => {
        this.fpLineGuide = input;
      }} style={style} className={`fp-container-line-guide ${className}`}>
        {guides.map((g, indexG) =>
          <ul key={indexG}>
            {g.map((item, index) => (
              <li key={index} style={{ width: `${liWidths[indexG]}%` }} onClick={() => {
                setField({ selectedGuide: item, selectedGuideRow: indexG }); this.alterGuide(index);
              }}>
                {item}
                {preenchidos.indexOf(item) >= 0 && <FaCheck className='fp-chck-g' />}
              </li>
            ))}
          </ul>
        )}
        <div className='fp-moving-tab'
          style={{
            backgroundColor: movingTabColor,
            boxShadow: movingTabColor ? `0 16px 26px -14px ${movingTabColor}` : undefined,
            width: `${liWidths[selectedGuideRow]}%`,
            transform: `translate3d(${guidePositionX}px, ${guidePositionY}px, 0px)`,
          }}>
          {selectedGuide}
          {preenchidos.indexOf(selectedGuide) >= 0 && <FaCheck className='fp-chck-w' />}
        </div>
      </div>
    );
  }
}

MenuBar.propTypes = {
  field: PropTypes.string.isRequired,
  guides: PropTypes.array.isRequired,
  defaultValue: PropTypes.oneOfType([PropTypes.number, PropTypes.string, PropTypes.object, PropTypes.array]),
  fieldListener: PropTypes.func,
  destroyOnUnmount: PropTypes.bool,
  movingTabColor: PropTypes.string,
  positionSpace: PropTypes.number,
};

export default createField(MenuBar);
