import { makeStyles } from '@material-ui/core';
import React, { useCallback, useEffect } from 'react';

const useStyles = makeStyles({
  container: {
    display: 'grid',
    placeItems: 'center'
  },
  circularProgress: {
    height: ({ size }) => size,
    width: ({ size }) => size,
    backgroundColor: ({ barBackgroundColor }) => barBackgroundColor,
    borderRadius: '50%',
    display: 'grid',
    placeItems: 'center',
    '&::before': {
      content: '""',
      position: 'absolute',
      height: ({ progressBarWidth }) => progressBarWidth,
      width: ({ progressBarWidth }) => progressBarWidth,
      borderRadius: '50%',
      backgroundColor: ({ backgroundColor }) => backgroundColor
    }
  },
  valueContainer: {
    position: 'relative',
    color: ({ color }) => color,
    fontSize: ({ fontSize }) => fontSize
  }
});

const PercentageDisplay = ({ 
  id,
  percentage = 0,
  size = 50,
  barWidth = 6,
  fontSize = '0.9rem',
  color = '#000',
  backgroundColor = '#fff',
  barColor = '#00995d',
  barBackgroundColor = '#00995d26'
}) => {

  const progressBarWidth = size - barWidth;
  const classes = useStyles({ color, backgroundColor, barColor, barBackgroundColor, size, progressBarWidth, fontSize });
  const percentageDisplayProgressId = `${id}-percentageDisplayProgress`;

  const createProgress = useCallback(() => {
    const porcentagem = parseInt(percentage);
    const progressBar = document.getElementById(percentageDisplayProgressId);
    if (!progressBar) return;
    progressBar.style.background = `conic-gradient(${barColor} ${porcentagem * 3.6}deg, ${barBackgroundColor} ${porcentagem * 3.6}deg)`;
  }, [barBackgroundColor, barColor, percentage, percentageDisplayProgressId]);
  useEffect(() => createProgress(), [percentage, createProgress]);

  return <div className={classes.container}>
    <div id={percentageDisplayProgressId} className={classes.circularProgress}>
      <div className={classes.valueContainer}>
        {`${percentage}%`}
      </div>
    </div>
  </div>;
};

export default PercentageDisplay;