import { makeStyles, Slide, Snackbar, Typography } from '@material-ui/core';
import React, { useEffect, useState } from 'react';
import { connect } from 'react-redux';

const useStyles = makeStyles(theme => ({
  container: {
    display: 'flex',
    overflow: 'hidden',
    borderRadius: 4,
    color: theme.palette.type === 'light' ? '#fff' : '#000',
    background: `linear-gradient(90deg, rgba(0,0,0,0) 0%, rgba(${theme.palette.type === 'light' ? '66,66,66' : '255,255,255' },1) 3%)`
  },
  messageDiv: {
    padding: '.5rem 1rem'
  },
  gradientStripe: {
    width: '1rem',
    background: 'linear-gradient(180deg, rgba(131,58,180,1) 0%, rgba(253,29,29,1) 50%, rgba(252,176,69,1) 100%)'
  }
}));


const SnackbarTransition = props => <Slide {...props} direction='up' />;

const PageCodeEasterEgg = ({ code, usuarioLogado }) => {
    
  const classes = useStyles();
  const [open, setOpen] = useState(false);
    
  function easterEggListener(e) {
    const teamMember = usuarioLogado?.idFornecedorInterno === 1; // Membro da TI
    if (teamMember && code && !open && e.ctrlKey && e.shiftKey && e.key === 'Z') setOpen(true);
  }
  
  useEffect(() => {
    document.addEventListener('keydown', easterEggListener, false);
    return () => document.removeEventListener('keydown', easterEggListener, false);
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, []);

  const handleClose = _ => setOpen(false);

  
  return <Snackbar 
    open={open} 
    onClose={handleClose}
    autoHideDuration={5000}
    TransitionComponent={SnackbarTransition}
    anchorOrigin={{ vertical: 'bottom', horizontal: 'left' }}
  >
    <div className={classes.container}>
      <div className={classes.gradientStripe} />
      <div className={classes.messageDiv}>
        <Typography variant='subtitle1'>Hi, dev!</Typography>
        <Typography variant='body1'>{`The page code is ${code}.`}</Typography>
      </div>
    </div>
  </Snackbar>;
};


const mapStateToProps = (state) => ({ 
  usuarioLogado: state.argous.usuarioLogado 
});

export default connect(mapStateToProps)(PageCodeEasterEgg);