import BarraFerramentas from './../BarraFerramentas';
import PageCodeEasterEgg from './PageCodeEasterEgg';
import * as argous from '../../actions';
import { getProgram, toggleFavorite } from '../../api';
import { Box, CircularProgress, Paper, Typography, useTheme } from '@material-ui/core';
import PropTypes from 'prop-types';
import React, { useEffect, useState } from 'react';
import { FaExclamationTriangle } from 'react-icons/fa';
import { connect } from 'react-redux';


const initialStateToolbarLoading = { 
  annotations: false 
};


function Programa({
  code,
  children,
  loading = false,
  threads = [],
  hint,
  title,
  panelTitle,
  hideTitle, 
  setCurrentProgram,
  programDidMount,
  favorites,
  setFavorites,
  bfHabNavegador,
  bfAbas = [],
  bfFerramentas = [],
  bfExecutaFuncao,
  bfNmFuncaoVoltar,
  bfNmCompartilhar,
  bfHabCrud,
  bfNmSalva,
  bfNmExclui,
  bfHabFavorito = true,
  bfTitulo, 
  // permissoes = { salvar: true, editar: true, excluir: true },
  linkCompartilhamento,
  className,
  rightSidebar,
  setRightSidebar,
  setAddProgramWidget,
  setProgramAnnotation,
  annotationParameters,
  openAnnotationParameters,
  fields: _,
  ...props
}) {

  const theme = useTheme();
  const [programa, setPrograma] = useState({});
  const [favoriteAnimationInsert, setFavoriteAnimationInsert] = useState(false);
  const [favoriteAnimationRemove, setFavoriteAnimationRemove] = useState(false);
  const [toolbarLoading/*, setToolbarLoading*/] = useState(initialStateToolbarLoading);


  useEffect(() => {
    (async () => {
      setCurrentProgram(code);
      const res = await getProgram(code);
      // TODO prever aqui tempo de bloqueio diferenciado
      if (res.pgmOk) setPrograma({ ...res.pgmDd, loaded: true, habWidget: res.habWidget });
      else setPrograma(p => ({ ...p, dsErro: res.dsErro, loaded: true }));
      programDidMount && programDidMount();
    })();
    return () => setCurrentProgram(undefined);
  // eslint-disable-next-line react-hooks/exhaustive-deps
  }, []);

  const openAnnotations = (event) => setProgramAnnotation({ ...programa, open: true, code, annotationParameters, openAnnotationParameters });

  const addFavorite = () => {
    const path = window.location.pathname;
    toggleFavorite({ link: path }).then(res => {
      setFavorites(res.fvrts);
      const existIn = res.fvrts.filter(item => item[3] === path);
      if (existIn.length > 0) {
        setFavoriteAnimationInsert(true);
        setTimeout(() => setFavoriteAnimationInsert(false), 300);
      } else {
        setFavoriteAnimationRemove(true);
        setTimeout(() => setFavoriteAnimationRemove(false), 500);
      }
    });
  };

  const addWidget = () => {
    if (!code) return;
    if (!rightSidebar) setRightSidebar && setRightSidebar(true);
    setAddProgramWidget && setAddProgramWidget(code);
  };

  const checkIfIsFavorite = () => {
    const path = window.location.pathname;
    const existIn = favorites.filter(item => item[3] === path);
    return existIn.length > 0 ? 'active' : '';
  };

  let { pgHint, pgTitle, pnlTitle, dsErro, loaded, habWidget, snAnotacaoPrograma } = programa;

  if (title !== undefined) pgTitle = title;
  if (hint !== undefined) pgHint = hint;
  if (panelTitle !== undefined) pnlTitle = panelTitle;
  if (bfTitulo === undefined) bfTitulo = pnlTitle;

  return (
    <Box fontFamily='Raleway, Helvetica, Arial, sans-serif' className={className} {...props}>
      {!hideTitle &&
        <span style={{ cursor: 'default', overflow: 'hidden', display: 'block', marginBottom: '2px', color: theme.palette.text.primary }}>
          <h2 className='reset-margin pull-left' style={{ color: theme.palette.text.primary }}>
            {pgTitle}
            <span className='text-muted' style={{ color: theme.palette.text.primary }} >&nbsp;|&nbsp;</span>
          </h2>
          <p id='pg-subtitulo' className='pull-left hw-program-subtitulo' style={{ color: theme.palette.text.primary }}>{pgHint}</p>
        </span>
      }
      <Paper variant='outlined' style={{ padding: 10, position: 'relative' }}>
        <BarraFerramentas
          titulo={bfTitulo}
          abas={bfAbas}
          habFavorito={bfHabFavorito}
          habNavegador={bfHabNavegador}
          executaFuncao={bfExecutaFuncao}
          nmFuncaoVoltar={bfNmFuncaoVoltar}
          nmCompartilhar={bfNmCompartilhar}
          habCrud={bfHabCrud}
          nmSalva={bfNmSalva}
          nmExclui={bfNmExclui}
          ferramentas={bfFerramentas}
          checkIfIsFavorite={checkIfIsFavorite}
          addFavorite={addFavorite}
          linkCompartilhamento={linkCompartilhamento}
          favoriteAnimationInsert={favoriteAnimationInsert}
          favoriteAnimationRemove={favoriteAnimationRemove}
          threads={threads}
          currentProgram={code}
          {...{ addWidget, habWidget, snAnotacaoPrograma, openAnnotations, toolbarLoading }}
        />
        {dsErro ? (
          <Box textAlign='center' >
            <FaExclamationTriangle style={{ color: '#ED1651', width: '90px', height: '90px' }} />
            <Typography variant='h6'>{dsErro}</Typography>
          </Box>
        ) : (!pnlTitle || loading || !loaded) ? (
          <Box position='relative' top={20} left='50%' z-index={20} height={80} width={0} marginLeft='-20px'>
            <CircularProgress disableShrink/>
          </Box>
        ) : children}
      </Paper>
      <PageCodeEasterEgg {...{ code }} />
    </Box>
  );
}

const mapStateToProps = (state) => ({ 
  fields: state.argous.fields, 
  favorites: state.argous.favorites, 
  rightSidebar: state.argous.rightSidebar, 
  programAnnotation: state.argous.programAnnotation 
});

const mapDispatchToProps = (dispatch) => ({ 
  setAddProgramWidget: data => dispatch(argous.setAddProgramWidget(data)), 
  setCurrentProgram: data => dispatch(argous.setCurrentProgram(data)), 
  setRightSidebar: data => dispatch(argous.setRightSidebar(data)), 
  setFavorites: data => dispatch(argous.setFavorites(data)), 
  setProgramAnnotation: data => dispatch(argous.setProgramAnnotation(data)) 
});

Programa.propTypes = {
  code: PropTypes.string.isRequired,
  loading: PropTypes.bool,
  children: PropTypes.oneOfType([PropTypes.node, PropTypes.element]),
  favorites: PropTypes.array,
  setFavorites: PropTypes.func,
  setCurrentProgram: PropTypes.func,
  programDidMount: PropTypes.func,
  threads: PropTypes.array,
  hideTitle: PropTypes.bool,  
  bfHabNavegador: PropTypes.bool,
  bfTitulo: PropTypes.string,
  bfFerramentas: PropTypes.array,
  bfAbas: PropTypes.array,
  bfExecutaFuncao: PropTypes.func,
  bfNmFuncaoVoltar: PropTypes.string,
  bfNmCompartilhar: PropTypes.string,
  bfHabCrud: PropTypes.bool,
  bfNmSalva: PropTypes.string,
  bfNmExclui: PropTypes.string,
  bfHabFavorito: PropTypes.bool,
  hint: PropTypes.string,
  linkCompartilhamento: PropTypes.string,
  title: PropTypes.string,
  panelTitle: PropTypes.string,
  // permissoes: PropTypes.object,
};

export default connect(mapStateToProps, mapDispatchToProps)(Programa);