import Input from './../Input';
import SimpleTable, { Action } from './../SimpleTable';
import { setField, getField } from '../../fieldsRedux';
import { serverAction } from '../../utils/api';
import { withTheme } from '@material-ui/core';
import PropTypes from 'prop-types';
import React, { Component } from 'react';
import { Scrollbars } from 'react-custom-scrollbars';

/**
 * @deprecated 
 */
class TablePicker extends Component {
  state = {
    dropOpen: false,
    typeMessage: 0,
    resultHeight: 0
  }
  changesAfterSelect = 2;

  componentWillUnmount() {
    clearTimeout(this.searchTimeout);
    clearTimeout(this.blurTimeout);
  }

  doSearchAction = (value) => {

    const recemSelecionado = this.resultRecemSelecionado();

    clearTimeout(this.searchTimeout);

    if (value.value && value.value.length > 2 && !recemSelecionado) {

      this.setState({ typeMessage: 2, resultHeight: 50 });

      this.searchTimeout = setTimeout(() => {

        serverAction(this.props.searchAction, { value: value.value }).then(res => {

          if (res.result) {

            const resultsFiltrados = this.filtraResults(res.result);
            const aux = resultsFiltrados.length > 0;

            this.setState({ typeMessage: aux ? 0 : 3, resultHeight: aux ? resultsFiltrados.length * 38 : 50 });

            setField(`results-${this.props.field}`, 'value', resultsFiltrados);

          } else {
            this.setState({ typeMessage: 3, resultHeight: 50 });
          }
        });
      }, 350);

    } else {

      const aux = value.value && value.value.length > 0;

      this.setState({ typeMessage: !recemSelecionado && aux ? 1 : 0, resultHeight: !recemSelecionado && aux ? 50 : 0 });

      setField(`results-${this.props.field}`, 'value', []);
    }
  }

  filtraResults = (results) => {
    const itensAlreadyAdded = getField(this.props.field, 'value');

    if (itensAlreadyAdded && itensAlreadyAdded.length > 0) {

      const resultsFiltrados = results.filter(r => {
        const exist = itensAlreadyAdded.filter(a => !this.compareObjectsWithAlias(a, r)).length > 0;

        return !exist;
      });

      return resultsFiltrados;
    }

    return results;
  }

  resultRecemSelecionado = () => {
    this.changesAfterSelect++;

    if (this.changesAfterSelect === 1) {
      return true;
    } else if (this.changesAfterSelect === 2) {
      setField(`search-${this.props.field}`, 'selectedResult', undefined);
    }

    return false;
  }

  selectResult = (item) => {
    const { field, alias } = this.props;

    setField(`search-${field}`, 'selectedResult', item);
    this.changesAfterSelect = 0;
    setField(`search-${field}`, 'value', this.createSearchSelectedValue(item, alias));
    setField(`results-${field}`, 'value', []);
    this.setState({ typeMessage: 0, resultHeight: 0 });

    this.add();
  }

  createSearchSelectedValue = (item, alias) => {
    let text = '';
    alias.map(m => text = text + (`${item[m]} `));
    return text;
  }

  add = () => {
    const itemToAdd = getField(`search-${this.props.field}`, 'selectedResult');

    if (itemToAdd) {
      if (this.props.addAction) {

        serverAction(this.props.addAction, { item: itemToAdd }).then(res => {
          if (res.pgmOk) {
            const itensAlreadyAdded = getField(this.props.field, 'value');
            setField(this.props.field, 'value', [itemToAdd, ...itensAlreadyAdded]);
            setField(`search-${this.props.field}`, 'value', '');
            setField(`search-${this.props.field}`, 'selectedResult', undefined);
          }
        });
      } else {
        const itensAlreadyAdded = getField(this.props.field, 'value');
        setField(this.props.field, 'value', [itemToAdd, ...itensAlreadyAdded]);
        setField(`search-${this.props.field}`, 'value', '');
        setField(`search-${this.props.field}`, 'selectedResult', undefined);
      }
    }
  }

  focusTypeMessage = () => {
    const resultsData = getField(`results-${this.props.field}`, 'value');

    if (resultsData.length === 0) {
      this.setState({ typeMessage: 1, resultHeight: 50 });
    } else {
      this.setState({ resultHeight: resultsData.length * 38 });
    }
  }

  blurTypeMessage = () => {
    this.blurTimeout = setTimeout(() => {
      this.setState({ typeMessage: 0, resultHeight: 0 });
    }, 250);
  }

  removeItem = (item) => {
    serverAction(this.props.removeAction, { item }).then(res => {
      if (res.pgmOk) {
        const itensAlreadyAdded = getField(this.props.field, 'value');
        const newItens = itensAlreadyAdded.filter(i => this.compareObjectsWithAlias(i, item));
        setField(this.props.field, 'value', newItens);
      }
    });
  }

  compareObjectsWithAlias = (a, b) => {
    let different = false;
    this.props.alias.map(i => {
      if (a[i] !== b[i]) {
        different = true;
      }
      return null;
    });
    return different;
  }

  render() {
    const { field, defaultValue, alias, head, rows, removeAction, fieldListener, placeholder } = this.props;
    const { typeMessage, resultHeight } = this.state;

    const value = getField(field, 'value');

    return (
      <form onSubmit={(e) => e.preventDefault()}>
        <table className='argous-groupselect'>
          <tbody>
            <tr className='head'>
              <td className='gs-search'>

                <Input placeholder={placeholder} onFocus={this.focusTypeMessage} onBlur={this.blurTypeMessage} field={`search-${field}`} fieldListener={this.doSearchAction} />

              </td>
            </tr>
            <tr>
              <td colSpan='2'>

                <div className='argous-list-dropdown' style={{ height: resultHeight }}>
                  <Scrollbars>
                    {typeMessage > 0 && (
                      <div className='padding-top-sm padding-bottom-sm color999 text-center'>
                        {typeMessage === 1 && 'Escreva para começar a buscar'}
                        {typeMessage === 2 && 'Pesquisando...'}
                        {typeMessage === 3 && 'Nenhum resultado foi encontrado'}
                      </div>
                    )}

                    <SimpleTable field={`results-${field}`} alias={alias} onClick={this.selectResult} className='reset-margin' />
                  </Scrollbars>
                </div>

                <SimpleTable field={field} defaultValue={defaultValue} alias={alias} head={head} rows={value && value.length > rows ? rows : undefined} fieldListener={fieldListener} className='reset-margin-bottom'>
                  {removeAction && <Action onClick={this.removeItem} glyphicon='glyph-icon icon-close' />}
                </SimpleTable>

                {(!value || (value && value.length < 1)) && (
                  <div className='padding-top-sm padding-bottom-sm text-center' style={{ backgroundColor: this.props.theme.palette.background.paper }}>
                    Pesquise acima para adicionar
                  </div>
                )}

              </td>
            </tr>
          </tbody>
        </table>
      </form>
    );
  }
}

TablePicker.propTypes = {
  field: PropTypes.string.isRequired,
  alias: PropTypes.array.isRequired,
  searchAction: PropTypes.string.isRequired,
  fieldListener: PropTypes.func,
  destroyOnUnmount: PropTypes.bool,
  addAction: PropTypes.string,
  placeholder: PropTypes.string,
  removeAction: PropTypes.string,
  head: PropTypes.array,
  rows: PropTypes.number
};

export default withTheme(TablePicker);
