import { ClickAwayListener, IconButton, makeStyles } from '@material-ui/core';
import React, { useState } from 'react';
import { RgbaColorPicker } from 'react-colorful';
import { RiAlignCenter, RiAlignJustify, RiAlignLeft, RiAlignRight, RiFontColor, RiFontSize } from 'react-icons/ri';


const ICON_SIZE = '2.5rem';
const BOX_SHADOW = 'rgba(0, 0, 0, 0.3) 0px 1px 12px -2px';

const useStyles = makeStyles(theme => ({
  container: {
    zIndex: 1,
    position: 'absolute',
    top: 0,
    right: 0,
    display: 'flex',
    alignItems: 'flex-start',
    justifyContent: 'flex-end',
    gap: ({ defaultSpacing }) => (defaultSpacing) ? defaultSpacing : ''
  },
  button: {
    padding: ({ defaultSpacing }) => (defaultSpacing) ? (defaultSpacing / 2) : '',
    color: `rgba(${(theme.palette.type === 'light') ? '0, 0, 0' : '255, 255, 255'}, .7)`,
    backgroundColor: `rgba(${(theme.palette.type === 'light') ? '255, 255, 255' : '0, 0, 0'}, .5)`,
    '&:hover': {
      backgroundColor: `rgba(${(theme.palette.type === 'light') ? '255, 255, 255' : '0, 0, 0'}, .7)`
    }
  },
  colorPickerContainer: {
    zIndex: 2,
    position: 'absolute',
    padding: '2rem'
  },
  colorPickerBoxShadow: {
    borderRadius: 8,
    webKitBoxShadow: BOX_SHADOW,
    mozBoxShadow: BOX_SHADOW,
    boxShadow: BOX_SHADOW
  }
}));


const initialStateOpen = {
  colorPicker: undefined
};

const initialState = {
  open: initialStateOpen
};


const TextStyles = ({ color, setColor, defaultSpacing, toggleClickAway, handleFontSizeChange, textAlign, handleTextAlignChange }) => {

  const classes = useStyles({ defaultSpacing });
  const [state, setState] = useState(initialState);
  const { open } = state;

  const logicSetState = (type, field, value) => setState(values => ({ 
    ...values, 
    [type]: (type && field) ? { ...values[type], [field]: value } : value 
  }));

  const handleColorPickerOpen = e => {
    toggleClickAway();
    const elRect = e.target.parentElement.getBoundingClientRect();
    logicSetState('open', 'colorPicker', elRect);
  };

  const handleColorPickerClose = e => {
    toggleClickAway();
    logicSetState('open', 'colorPicker', undefined);
  };


  return <div className={classes.container}>
    <IconButton onClick={handleTextAlignChange} onTouchStart={handleTextAlignChange} className={classes.button}>
      {(() => {
        switch (textAlign) {
          case 'start':
            return <RiAlignLeft size={ICON_SIZE} />;
          case 'end':
            return <RiAlignRight size={ICON_SIZE} />;
          case 'center':
            return <RiAlignCenter size={ICON_SIZE} />;
          case 'justify':
            return <RiAlignJustify size={ICON_SIZE} />;
          default:
            return null;
        }
      })()}
    </IconButton>
    <IconButton onClick={handleColorPickerOpen} onTouchStart={handleColorPickerOpen} className={classes.button}>
      <RiFontColor size={ICON_SIZE} />
    </IconButton>
    <IconButton onClick={handleFontSizeChange} onTouchStart={handleFontSizeChange} className={classes.button}>
      <RiFontSize size={ICON_SIZE} />
    </IconButton>
    {(!!open.colorPicker) && (
      <ClickAwayListener disableReactTree onClickAway={handleColorPickerClose}>
        <div className={classes.colorPickerContainer}>
          <div className={classes.colorPickerBoxShadow}>
            <RgbaColorPicker color={color} onChange={setColor} />
          </div>
        </div>
      </ClickAwayListener>
    )}
  </div>;
};


export default TextStyles;