import { TEXT_TO_HTML_ID, initialStateCharacter } from './TextToHtml';


export const DEAD_KEY = 'Dead';
export const SEARCH_PEOPLE_SYMBOL = '@';


const space = {
  'key': ' ',
  'keyCode': 32,
  'code': 'Space',
  'string': ' ',
  'dead': false,
  'shift': false
};


export const createCharactersWithMark = (string, mark) => {

  let characters = [];

  for (let i = 0; i < string?.length; i++) {
    characters.push({
      ...initialStateCharacter,
      string: string[i],
      mark
    });
  }
  characters.push(space);

  return { markedCharacters: characters };
};

export const getCharacterElement = ({ index }) => {

  let response = {
    nodeElement: undefined,
    rect: undefined
  };

  const parentElement = document.getElementById(TEXT_TO_HTML_ID);
  if (parentElement) {
    const childNodes = parentElement.childNodes[0].childNodes;
    const targetElement = childNodes[index];
    const targetElementRect = targetElement.getBoundingClientRect();
    response = {
      ...response,
      nodeElement: targetElement,
      rect: targetElementRect
    };
  }

  return response;
};


export const translateKey = ({ key, keyCode, code, shift, altGraph, capsLock, before }) => {

  let response = {
    string: '',
    dead: false,
    before: {
      dead: false,
      delete: false
    }
  };

  if (key === DEAD_KEY) {

    if (code === 'BracketLeft') {
      response.string = (shift) ? '`' : '´';
    } else if (code === 'Quote') {
      response.string = (shift) ? '^' : '~';
    } else if (code === 'Digit6') {
      response.string = (shift) ? '¨' : '';
    }
    response.before.dead = true;

  } else if (before?.dead) {

    const targetKey = KEYS(shift, before?.shift, altGraph, capsLock).filter((keyItem) => (((keyItem.key === key) || (keyItem.keyCode === keyCode)) && (keyItem.code === code)))?.[0];
    const specialKey = targetKey?.special.filter(special => special.string === before?.string)?.[0];
    const special = specialKey?.value;

    response.string = special ? special : targetKey?.key;
    response.before.delete = special ? true : false;
    response.before.dead = false;

  } else {
    const targetKey = KEYS(shift, before?.shift, altGraph, capsLock).filter((keyItem) => (((keyItem.key === key) || (keyItem.keyCode === keyCode)) && (keyItem.code === code)))?.[0];
    response.string = targetKey?.key;
  }

  return response;
};


const KEYS = (shift, beforeShift, altGraph, capsLock) => [
  {
    key: (shift || capsLock) ? 'W' : 'w', 
    keyCode: 87, 
    code: 'KeyW', 
    special: [
      {
        code: 'BracketLeft',
        key: 'Dead',
        keyCode: 219,
        string: (beforeShift) ? '`' : '´',
        value: (beforeShift) ? ((shift || capsLock) ? 'Ẁ' : 'ẁ') : ((shift || capsLock) ? 'Ẃ' : 'ẃ')
      },
      {
        code: 'Quote',
        key: 'Dead',
        keyCode: 222,
        string: (beforeShift) ? '^' : '~',
        value: (beforeShift) ? ((shift || capsLock) ? 'Ŵ' : 'ŵ') : ((shift || capsLock) ? '' : '')
      },
      {
        code: 'Digit6',
        key: 'Dead',
        keyCode: 54,
        string: (beforeShift) ? '¨' : '',
        value: (beforeShift) ? ((shift || capsLock) ? 'Ẅ' : 'ẅ') : ((shift || capsLock) ? '' : '')
      }
    ] 
  },
  {
    key: (shift || capsLock) ? 'E' : 'e', 
    keyCode: 69, 
    code: 'KeyE', 
    special: [
      {
        code: 'BracketLeft',
        key: 'Dead',
        keyCode: 219,
        string: (beforeShift) ? '`' : '´',
        value: (beforeShift) ? ((shift || capsLock) ? 'È' : 'è') : ((shift || capsLock) ? 'É' : 'é')
      },
      {
        code: 'Quote',
        key: 'Dead',
        keyCode: 222,
        string: (beforeShift) ? '^' : '~',
        value: (beforeShift) ? ((shift || capsLock) ? 'Ê' : 'ê') : ((shift || capsLock) ? 'Ẽ' : 'ẽ')
      },
      {
        code: 'Digit6',
        key: 'Dead',
        keyCode: 54,
        string: (beforeShift) ? '¨' : '',
        value: (beforeShift) ? ((shift || capsLock) ? 'Ë' : 'ë') : ((shift || capsLock) ? '' : '')
      }
    ] 
  },
  {
    key: (shift || capsLock) ? 'R' : 'r', 
    keyCode: 82, 
    code: 'KeyR', 
    special: [
      {
        code: 'BracketLeft',
        key: 'Dead',
        keyCode: 219,
        string: '´',
        value: (shift || capsLock) ? 'Ŕ' : 'ŕ'
      }
    ] 
  },
  {
    key: (shift || capsLock) ? 'Y' : 'y', 
    keyCode: 89, 
    code: 'KeyY', 
    special: [
      {
        code: 'BracketLeft',
        key: 'Dead',
        keyCode: 219,
        string: (beforeShift) ? '`' : '´',
        value: (beforeShift) ? ((shift || capsLock) ? 'Ỳ' : 'ỳ') : ((shift || capsLock) ? 'Ý' : 'ý')
      },
      {
        code: 'Quote',
        key: 'Dead',
        keyCode: 222,
        string: (beforeShift) ? '^' : '~',
        value: (beforeShift) ? ((shift || capsLock) ? 'Ŷ' : 'ŷ') : ((shift || capsLock) ? 'Ỹ' : 'ỹ')
      },
      {
        code: 'Digit6',
        key: 'Dead',
        keyCode: 54,
        string: (beforeShift) ? '¨' : '',
        value: (beforeShift) ? ((shift || capsLock) ? 'Ÿ' : 'ÿ') : ((shift || capsLock) ? '' : '')
      }
    ] 
  },
  {
    key: (shift || capsLock) ? 'U' : 'u', 
    keyCode: 85, 
    code: 'KeyU', 
    special: [
      {
        code: 'BracketLeft',
        key: 'Dead',
        keyCode: 219,
        string: (beforeShift) ? '`' : '´',
        value: (beforeShift) ? ((shift || capsLock) ? 'Ù' : 'ù') : ((shift || capsLock) ? 'Ú' : 'ú')
      },
      {
        code: 'Quote',
        key: 'Dead',
        keyCode: 222,
        string: (beforeShift) ? '^' : '~',
        value: (beforeShift) ? ((shift || capsLock) ? 'Û' : 'û') : ((shift || capsLock) ? 'Ũ' : 'ũ')
      },
      {
        code: 'Digit6',
        key: 'Dead',
        keyCode: 54,
        string: (beforeShift) ? '¨' : '',
        value: (beforeShift) ? ((shift || capsLock) ? 'Ü' : 'ü') : ((shift || capsLock) ? '' : '')
      }
    ] 
  },
  {
    key: (shift || capsLock) ? 'I' : 'i', 
    keyCode: 73, 
    code: 'KeyI', 
    special: [
      {
        code: 'BracketLeft',
        key: 'Dead',
        keyCode: 219,
        string: (beforeShift) ? '`' : '´',
        value: (beforeShift) ? ((shift || capsLock) ? 'Ì' : 'ì') : ((shift || capsLock) ? 'Í' : 'í')
      },
      {
        code: 'Quote',
        key: 'Dead',
        keyCode: 222,
        string: (beforeShift) ? '^' : '~',
        value: (beforeShift) ? ((shift || capsLock) ? 'Î' : 'î') : ((shift || capsLock) ? 'Ĩ' : 'ĩ')
      },
      {
        code: 'Digit6',
        key: 'Dead',
        keyCode: 54,
        string: (beforeShift) ? '¨' : '',
        value: (beforeShift) ? ((shift || capsLock) ? 'Ï' : 'ï') : ((shift || capsLock) ? '' : '')
      }
    ] 
  },
  {
    key: (shift || capsLock) ? 'O' : 'o', 
    keyCode: 79, 
    code: 'KeyO', 
    special: [
      {
        code: 'BracketLeft',
        key: 'Dead',
        keyCode: 219,
        string: (beforeShift) ? '`' : '´',
        value: (beforeShift) ? ((shift || capsLock) ? 'Ò' : 'ò') : ((shift || capsLock) ? 'Ó' : 'ó')
      },
      {
        code: 'Quote',
        key: 'Dead',
        keyCode: 222,
        string: (beforeShift) ? '^' : '~',
        value: (beforeShift) ? ((shift || capsLock) ? 'Ô' : 'ô') : ((shift || capsLock) ? 'Õ' : 'õ')
      },
      {
        code: 'Digit6',
        key: 'Dead',
        keyCode: 54,
        string: (beforeShift) ? '¨' : '',
        value: (beforeShift) ? ((shift || capsLock) ? 'Ö' : 'ö') : ((shift || capsLock) ? '' : '')
      }
    ] 
  },
  {
    key: (shift || capsLock) ? 'P' : 'p', 
    keyCode: 80, 
    code: 'KeyP', 
    special: [
      {
        code: 'BracketLeft',
        key: 'Dead',
        keyCode: 219,
        string: '´',
        value: (shift || capsLock) ? 'Ṕ' : 'ṕ'
      }
    ] 
  },
  {
    key: (shift || capsLock) ? 'A' : 'a', 
    keyCode: 65, 
    code: 'KeyA', 
    special: [
      {
        code: 'BracketLeft',
        key: 'Dead',
        keyCode: 219,
        string: (beforeShift) ? '`' : '´',
        value: (beforeShift) ? ((shift || capsLock) ? 'À' : 'à') : ((shift || capsLock) ? 'Á' : 'á')
      },
      {
        code: 'Quote',
        key: 'Dead',
        keyCode: 222,
        string: (beforeShift) ? '^' : '~',
        value: (beforeShift) ? ((shift || capsLock) ? 'Â' : 'â') : ((shift || capsLock) ? 'Ã' : 'ã')
      },
      {
        code: 'Digit6',
        key: 'Dead',
        keyCode: 54,
        string: (beforeShift) ? '¨' : '',
        value: (beforeShift) ? ((shift || capsLock) ? 'Ä' : 'ä') : ((shift || capsLock) ? '' : '')
      }
    ] 
  },
  {
    key: (shift || capsLock) ? 'S' : 's', 
    keyCode: 83, 
    code: 'KeyS', 
    special: [
      {
        code: 'BracketLeft',
        key: 'Dead',
        keyCode: 219,
        string: '´',
        value: (shift || capsLock) ? 'Ś' : 'ś'
      },
      {
        code: 'Quote',
        key: 'Dead',
        keyCode: 222,
        string: (beforeShift) ? '^' : '~',
        value: (beforeShift) ? ((shift || capsLock) ? 'Ŝ' : 'ŝ') : ((shift || capsLock) ? '' : '')
      }
    ] 
  },
  {
    key: (shift || capsLock) ? 'G' : 'g', 
    keyCode: 71, 
    code: 'KeyG', 
    special: [
      {
        code: 'BracketLeft',
        key: 'Dead',
        keyCode: 219,
        string: '´',
        value: (shift || capsLock) ? 'Ǵ' : 'ǵ'
      },
      {
        code: 'Quote',
        key: 'Dead',
        keyCode: 222,
        string: (beforeShift) ? '^' : '~',
        value: (beforeShift) ? ((shift || capsLock) ? 'Ĝ' : 'ĝ') : ((shift || capsLock) ? '' : '')
      }
    ] 
  },
  {
    key: (shift || capsLock) ? 'J' : 'j', 
    keyCode: 74, 
    code: 'KeyJ', 
    special: [
      {
        code: 'BracketLeft',
        key: 'Dead',
        keyCode: 219,
        string: '´',
        value: (shift || capsLock) ? 'J́' : 'j́'
      },
      {
        code: 'Quote',
        key: 'Dead',
        keyCode: 222,
        string: (beforeShift) ? '^' : '~',
        value: (beforeShift) ? ((shift || capsLock) ? 'Ĵ' : 'ĵ') : ((shift || capsLock) ? '' : '')
      }
    ] 
  },
  {
    key: (shift || capsLock) ? 'K' : 'k', 
    keyCode: 75, 
    code: 'KeyK', 
    special: [
      {
        code: 'BracketLeft',
        key: 'Dead',
        keyCode: 219,
        string: '´',
        value: (shift || capsLock) ? 'Ḱ' : 'ḱ'
      }
    ] 
  },
  {
    key: (shift || capsLock) ? 'L' : 'l', 
    keyCode: 76, 
    code: 'KeyL', 
    special: [
      {
        code: 'BracketLeft',
        key: 'Dead',
        keyCode: 219,
        string: '´',
        value: (shift || capsLock) ? 'Ĺ' : 'ĺ'
      }
    ] 
  },
  {
    key: (shift || capsLock) ? 'Ç' : 'ç', 
    keyCode: 186, 
    code: 'Semicolon', 
    special: [
      {
        code: 'BracketLeft',
        key: 'Dead',
        keyCode: 219,
        string: '´',
        value: (shift || capsLock) ? 'Ḉ' : 'ḉ'
      }
    ] 
  },
  {
    key: (shift || capsLock) ? 'Z' : 'z', 
    keyCode: 90, 
    code: 'KeyZ', 
    special: [
      {
        code: 'BracketLeft',
        key: 'Dead',
        keyCode: 219,
        string: '´',
        value: (shift || capsLock) ? 'Ź' : 'ź'
      },
      {
        code: 'Quote',
        key: 'Dead',
        keyCode: 222,
        string: (beforeShift) ? '^' : '~',
        value: (beforeShift) ? ((shift || capsLock) ? 'Ẑ' : 'ẑ') : ((shift || capsLock) ? '' : '')
      }
    ] 
  },
  {
    key: (shift || capsLock) ? 'C' : 'c', 
    keyCode: 67, 
    code: 'KeyC', 
    special: [
      {
        code: 'BracketLeft',
        key: 'Dead',
        keyCode: 219,
        string: '´',
        value: (shift || capsLock) ? 'Ć' : 'ć'
      },
      {
        code: 'Quote',
        key: 'Dead',
        keyCode: 222,
        string: (beforeShift) ? '^' : '~',
        value: (beforeShift) ? ((shift || capsLock) ? 'Ĉ' : 'ĉ') : ((shift || capsLock) ? '' : '')
      }
    ] 
  },
  {
    key: (shift || capsLock) ? 'V' : 'v', 
    keyCode: 86, 
    code: 'KeyV', 
    special: [
      {
        code: 'BracketLeft',
        key: 'Dead',
        keyCode: 219,
        string: (beforeShift) ? '`' : '´',
        value: (beforeShift) ? ((shift || capsLock) ? 'Ǜ' : 'ǜ') : ((shift || capsLock) ? 'Ǘ' : 'ǘ')
      },
      {
        code: 'Quote',
        key: 'Dead',
        keyCode: 222,
        string: (beforeShift) ? '^' : '~',
        value: (beforeShift) ? ((shift || capsLock) ? '' : '') : ((shift || capsLock) ? 'Ṽ' : 'ṽ')
      }
    ] 
  },
  {
    key: (shift || capsLock) ? 'N' : 'n', 
    keyCode: 78, 
    code: 'KeyN', 
    special: [
      {
        code: 'BracketLeft',
        key: 'Dead',
        keyCode: 219,
        string: (beforeShift) ? '`' : '´',
        value: (beforeShift) ? ((shift || capsLock) ? 'Ǹ' : 'ǹ') : ((shift || capsLock) ? 'Ń' : 'ń')
      },
      {
        code: 'Quote',
        key: 'Dead',
        keyCode: 222,
        string: (beforeShift) ? '^' : '~',
        value: (beforeShift) ? ((shift || capsLock) ? '' : '') : ((shift || capsLock) ? 'Ñ' : 'ñ')
      }
    ] 
  },
  {
    key: (shift || capsLock) ? 'M' : 'm', 
    keyCode: 77, 
    code: 'KeyM', 
    special: [
      {
        code: 'BracketLeft',
        key: 'Dead',
        keyCode: 219,
        string: (beforeShift) ? '`' : '´',
        value: (beforeShift) ? ((shift || capsLock) ? 'M̀' : 'm̀') : ((shift || capsLock) ? 'Ḿ' : 'ḿ')
      }
    ] 
  },
  {
    key: (shift || capsLock) ? 'Q' : 'q', 
    keyCode: 81, 
    code: 'KeyQ', 
    special: [
    ] 
  },
  {
    key: (shift || capsLock) ? 'T' : 't', 
    keyCode: 84, 
    code: 'KeyT', 
    special: [
    ] 
  },
  {
    key: (shift || capsLock) ? 'D' : 'd', 
    keyCode: 68, 
    code: 'KeyD', 
    special: [
    ] 
  },
  {
    key: (shift || capsLock) ? 'F' : 'f', 
    keyCode: 70, 
    code: 'KeyF', 
    special: [
    ] 
  },
  {
    key: (shift || capsLock) ? 'H' : 'h', 
    keyCode: 72, 
    code: 'KeyH', 
    special: [
      {
        code: 'Quote',
        key: 'Dead',
        keyCode: 222,
        string: (beforeShift) ? '^' : '~',
        value: (beforeShift) ? ((shift || capsLock) ? 'Ĥ' : 'ĥ') : ((shift || capsLock) ? '' : '')
      },
      {
        code: 'Digit6',
        key: 'Dead',
        keyCode: 54,
        string: (beforeShift) ? '¨' : '',
        value: (beforeShift) ? ((shift || capsLock) ? 'Ḧ' : 'ḧ') : ((shift || capsLock) ? '' : '')
      }
    ] 
  },
  {
    key: (shift || capsLock) ? 'X' : 'x', 
    keyCode: 88, 
    code: 'KeyX', 
    special: [
      {
        code: 'Digit6',
        key: 'Dead',
        keyCode: 54,
        string: (beforeShift) ? '¨' : '',
        value: (beforeShift) ? ((shift || capsLock) ? 'Ẍ' : 'ẍ') : ((shift || capsLock) ? '' : '')
      }
    ] 
  },
  {
    key: (shift || capsLock) ? 'B' : 'b', 
    keyCode: 66, 
    code: 'KeyB', 
    special: [
    ] 
  },
  {
    key: ' ', 
    keyCode: 32, 
    code: 'Space', 
    special: [
    ] 
  },
  {
    key: '1', 
    keyCode: 97, 
    code: 'Numpad1', 
    special: [
    ] 
  },
  {
    key: '2', 
    keyCode: 98, 
    code: 'Numpad2', 
    special: [
    ] 
  },
  {
    key: '3', 
    keyCode: 99, 
    code: 'Numpad3', 
    special: [
    ] 
  },
  {
    key: '4', 
    keyCode: 100, 
    code: 'Numpad4', 
    special: [
    ] 
  },
  {
    key: '5', 
    keyCode: 101, 
    code: 'Numpad5', 
    special: [
    ] 
  },
  {
    key: '6', 
    keyCode: 102, 
    code: 'Numpad6', 
    special: [
    ] 
  },
  {
    key: '7', 
    keyCode: 103, 
    code: 'Numpad7', 
    special: [
    ] 
  },
  {
    key: '8', 
    keyCode: 104, 
    code: 'Numpad8', 
    special: [
    ] 
  },
  {
    key: '9', 
    keyCode: 105, 
    code: 'Numpad9', 
    special: [
    ] 
  },
  {
    key: '0', 
    keyCode: 96, 
    code: 'Numpad0', 
    special: [
    ] 
  },
  {
    key: '/', 
    keyCode: 111, 
    code: 'NumpadDivide', 
    special: [
    ] 
  },
  {
    key: '*', 
    keyCode: 106, 
    code: 'NumpadMultiply', 
    special: [
    ] 
  },
  {
    key: '-', 
    keyCode: 109, 
    code: 'NumpadSubtract', 
    special: [
    ] 
  },
  {
    key: '+', 
    keyCode: 107, 
    code: 'NumpadAdd', 
    special: [
    ] 
  },
  {
    key: '.', 
    keyCode: 110, 
    code: 'NumpadComma', 
    special: [
    ] 
  },
  {
    key: ',', 
    keyCode: 108, 
    code: 'NumpadDecimal', 
    special: [
    ] 
  },
  {
    key: (shift) ? '"' : "'", 
    keyCode: 192, 
    code: 'Backquote', 
    special: [
    ] 
  },
  {
    key: (shift) ? '!' : '1', 
    keyCode: 49, 
    code: 'Digit1', 
    special: [
    ] 
  },
  {
    key: (shift) ? '@' : '2', 
    keyCode: 50, 
    code: 'Digit2', 
    special: [
    ] 
  },
  {
    key: (shift) ? '#' : '3', 
    keyCode: 51, 
    code: 'Digit3', 
    special: [
    ] 
  },
  {
    key: (shift) ? '$' : '4', 
    keyCode: 52, 
    code: 'Digit4', 
    special: [
    ] 
  },
  {
    key: (shift) ? '%' : '5', 
    keyCode: 53, 
    code: 'Digit5', 
    special: [
    ] 
  },
  {
    key: '6', 
    keyCode: 54, 
    code: 'Digit6', 
    special: [
    ] 
  },
  {
    key: (shift) ? '&' : '7', 
    keyCode: 55, 
    code: 'Digit7', 
    special: [
    ] 
  },
  {
    key: (shift) ? '*' : '8', 
    keyCode: 56, 
    code: 'Digit8', 
    special: [
    ] 
  },
  {
    key: (shift) ? '(' : '9', 
    keyCode: 57, 
    code: 'Digit9', 
    special: [
    ] 
  },
  {
    key: (shift) ? ')' : '0', 
    keyCode: 48, 
    code: 'Digit0', 
    special: [
    ] 
  },
  {
    key: (shift) ? '_' : '-', 
    keyCode: 189, 
    code: 'Minus', 
    special: [
    ] 
  },
  {
    key: (altGraph) ? '§' : ((shift) ? '+' : '='), 
    keyCode: 187, 
    code: 'Equal', 
    special: [
    ] 
  },
  {
    key: (shift) ? '<' : ',', 
    keyCode: 188, 
    code: 'Comma', 
    special: [
    ] 
  },
  {
    key: (shift) ? '>' : '.', 
    keyCode: 190, 
    code: 'Period', 
    special: [
    ] 
  },
  {
    key: (shift) ? ':' : ';', 
    keyCode: 191, 
    code: 'Slash', 
    special: [
    ] 
  },
  {
    key: (altGraph) ? '°' : ((shift) ? '?' : '/'), 
    keyCode: 191, 
    code: 'IntlRo', 
    special: [
    ] 
  },
  {
    key: (altGraph) ? 'º' : ((shift) ? '}' : ']'), 
    keyCode: 220, 
    code: 'Backslash', 
    special: [
    ] 
  },
  {
    key: (altGraph) ? 'ª' : ((shift) ? '{' : '['), 
    keyCode: 221, 
    code: 'BracketRight', 
    special: [
    ] 
  }
];