import { recebeThread, recebeInfoThread, server } from '../../api';
import ProgressBar from '../../components/ProgressBar';
import SimpleTable from '../../components/SimpleTable';
import { setField } from '../../fieldsRedux';
import { getConfig } from '../../utils/config';
import { sqlDateToReadableDate } from '../../utils/date';
import { IconButton } from '@material-ui/core';
import PropTypes from 'prop-types';
import React, { Component } from 'react';
import { FaAngleLeft, FaPrint } from 'react-icons/fa';
import { TiCog } from 'react-icons/ti';
import Spinner from 'react-spinkit';

/**
 * @deprecated 
 */
class ThreadLog extends Component {
  state = {
    mouseOver: false,
    open: false,
    loading: false,
    selectPanelClicks: 0,
    infoProcesso: {
      open: false,
      info: {},
      confs: {},
      current: {}
    }
  }

  componentWillUnmount() {
    window.removeEventListener('click', this.closePanel);
    clearTimeout(this.timeoutInfoProcesso);
  }

  mouseOver = (boolean) => {
    this.setState({ mouseOver: boolean });
  }

  openDetails = () => {
    const { open, infoProcesso } = this.state;
    this.setState({ open: !open, selectPanelClicks: 0 });
    window.addEventListener('click', this.closePanel);
    if (!open) {
      const { thread, qt = 15, nrPg = 0 } = this.props;
      this.setState({ loading: true });
      if (infoProcesso.open) this.buscaInfoProcesso(infoProcesso.info[0]);
      recebeThread({ cdThread: thread.thread, qt, nrPg }).then(res => {
        const processos = res.processos.map(item => {
          switch (item[3]) {
            case 'P': item[3] = 'Pronto'; break;
            case 'R': item[3] = 'Executando'; break;
            default: item[3] = 'Finalizado';
          }
          switch (item[4]) {
            case 'N': item[4] = 'Normal'; break;
            case 'C': item[4] = 'Cancelado'; break;
            case 'E': item[4] = 'Erro'; break;
            default: item[4] = '';
          }
          const inicio = sqlDateToReadableDate(item[6]);
          const fim = sqlDateToReadableDate(item[7]);
          item[6] = `${inicio.hour} - ${inicio.date}`;
          item[7] = `${fim.hour} - ${fim.date}`;
          return item;
        });
        this.setState({ loading: false });
        setField(`${this.props.currentProgram}-threads`, 'value', processos);
      });
    }
  }

  visualizaProcesso = (processo) => {
    this.setState({ loading: true });
    this.buscaInfoProcesso(processo[0]);
  }

  buscaInfoProcesso = (idProcesso) => {
    recebeInfoThread({ idProcesso }).then(res => {
      const confs = JSON.parse(res.infoProcesso[6]);
      const arr = [];
      for (let key of Object.keys(confs)) arr.push([key, confs[key].toString()]);
      if (res.currentStatus) {
        this.timeoutInfoProcesso = setTimeout(() => {
          this.buscaInfoProcesso(idProcesso);
        }, 3000);
      }
      this.setState({
        infoProcesso: {
          open: true,
          info: res.infoProcesso,
          confs: arr,
          current: res.currentStatus
        },
        loading: false
      });
    });
  }

  closeInfoProcesso = () => {
    clearTimeout(this.timeoutInfoProcesso);
    this.setState({
      infoProcesso: {
        open: false,
        info: {},
        confs: {},
        current: {}
      }
    });
  }

  closePanel = () => {
    const { selectPanelClicks } = this.state;
    if (selectPanelClicks > 0) {
      this.setState({ open: false });
      window.removeEventListener('click', this.closePanel);
    }
    this.setState({ selectPanelClicks: selectPanelClicks + 1 });
  }

  render() {
    const { open, loading, infoProcesso } = this.state;
    const { currentProgram, thread } = this.props;
    return (
      <React.Fragment>
        <IconButton title={thread.nome} onClick={this.openDetails}><TiCog key={'leave'} /></IconButton>
        {open && (
          <div onClick={e => e.stopPropagation()}>
            <h3 className='reset-margin-top bg-blue-alt'>
              {infoProcesso.open && <span className='pointer' onClick={this.closeInfoProcesso}><FaAngleLeft /></span>}
              {infoProcesso.open ? infoProcesso.info[2] : thread.nome}
            </h3>
            {loading && (
              <div className='loading-default margin-top-md margin-bottom-md'><Spinner name='ball-scale-multiple' fadeIn='half' /></div>
            )}
            {!loading && (
              <div>
                {infoProcesso.open && (
                  <div>
                    {infoProcesso.current &&
                      <div className='padding-sm'>
                        <span>{infoProcesso.current[7]}</span>
                        <span className='color999'> - {infoProcesso.current[6]}</span>
                        <ProgressBar value={parseInt(((infoProcesso.current[5] * 100) / infoProcesso.current[4]).toFixed(2), 10)} />
                        <div className='over-hidden'>
                          <h6 className='pull-left'><span className='color999'>Executando há:</span> {infoProcesso.current[1]}</h6>
                          <h6 className='pull-right'><span className='color999'>Tempo estimado:</span> {infoProcesso.current[3]}</h6>
                        </div>
                      </div>
                    }
                    <SimpleTable field={`${currentProgram}-thread-info`} className='reset-margin-bottom' defaultValue={infoProcesso.confs} destroyOnUnmount />
                    {infoProcesso.info[9].relatorios.length > 0 &&
                      <div>
                        <hr className='reset-margin' />
                        <div className='text-right padding-sm'>
                          {infoProcesso.info[9].relatorios.map((item, index) =>
                            <a key={index} className='ag-threadlog-print' href={`${getConfig().ssl ? 'https' : 'http'}${server()}}r${infoProcesso.info[0]}s${item[0]}.agrel`} target='_blank' rel='noreferrer'>
                              {item[1]}
                              {item[2] > 0 && <span className='agtl-size'>- {item[2]} kb</span>}
                              <span className='agtl-print bg-blue-alt'>
                                <FaPrint />
                              </span>
                            </a>
                          )}
                        </div>
                      </div>
                    }
                  </div>
                )}
                {!infoProcesso.open && (
                  <SimpleTable field={`${currentProgram}-threads`} rows={5} className='process-threads reset-margin-bottom' onClick={this.visualizaProcesso} head={['Descrição', 'Status', 'Finalização', 'Início', 'Fim']} alias={[2, 3, 4, 6, 7]} />
                )}
              </div>
            )}
          </div>
        )}
      </React.Fragment>
    );
  }
}

ThreadLog.propTypes = {
  thread: PropTypes.object,
  currentProgram: PropTypes.string,
  qt: PropTypes.number,
  nrPg: PropTypes.number
};

export default ThreadLog;
