import { ArgousAvatar } from '../';
import { setAnotherProgramField } from '../../fieldsRedux';
import { useStore } from '../../utils';
import { serverAction } from '../../utils/api';
import { Card, Typography, Grid, Button, Portal, Popper } from '@material-ui/core';
import React, { useState, useEffect, useRef, useCallback } from 'react';
import { IoIosCall } from 'react-icons/io';
import { MdMail, MdMessage } from 'react-icons/md';


const initialState = {
  local: {
    anchor: null,
    userId: null
  },
  data: {},
  show: false
};


function UserCard({ anchor, userId }) {

  const time = useRef();
  const clear = useRef(true);
  const allowShow = useRef(false);
  const { usuarioLogado: { idUsuario: loggedUserId } } = useStore();

  const [state, setState] = useState({
    ...initialState, 
    local: {
      anchor: (anchor || null),
      userId: (userId || null)
    } 
  });
  const { local, show, data } = state;

  const logicSetState = ({ type, field, value }) => setState(states => ({ 
    ...states, 
    [type]: type && field ? { ...states[type], [field]: value } : value
  }));

  useEffect(() => {
    if (anchor && userId) {
      logicSetState({ type: 'local', value: { anchor, userId } });
      allowShow.current = true;
    } else {
      clearTimeout(time.current);
      allowShow.current = false;
      setTimeout(() => {
        if (!clear.current) return;
        setState(initialState);
      }, 200);
    }
  }, [anchor, userId, show]);

  const search = useCallback(async () => {
    if (!local.anchor || !local.userId) {
      logicSetState({ type: 'show', value: false });
      clearTimeout(time.current);
      return;
    }
    const response = await serverAction('getInfoUsuario', { idUsuario: local.userId }, 'Argous.MntUsuarios', true);
    time.current = setTimeout(() => {
      if (allowShow.current) {
        setState(states => ({
          ...states,
          data: response,
          show: true
        }));
      }
    }, 500);
  }, [local]);
  useEffect(() => void(search()), [search]);

  const returnUser = () => {
    return { idUsuario: userId, titulo: data.dsNomeUsuario, subTitulo: data.dsFuncaoUsuario, msgsNaoLidas: 0, online: false };
  };

  function mouseLeave() {
    setState(initialState);
    clearTimeout(time.current);
    clear.current = true;
  }

  function mouseEnter() {
    clearTimeout(clear.current);
    clear.current = false;
  }


  if (local.userId && show) return (
    <Portal>
      <Popper 
        open={((!!anchor || !!local.anchor) && !!data.dsNomeUsuario)} 
        anchorEl={anchor || local.anchor} 
        onClick={e => e.stopPropagation()} 
        onMouseLeave={mouseLeave} 
        onMouseEnter={mouseEnter} 
        style={{ zIndex: 1300 }}
      >
        <Card variant='outlined' style={{ padding: '10px', maxWidth: '350px' }}>
          <Grid container spacing={1}>
            <Grid item xs='auto'>
              <ArgousAvatar style={{ width: '50px', height: '50px', margin: '10px' }} nome={data.dsNomeUsuario} idUsuario={userId || local.userId} />
            </Grid>
            <Grid item xs='auto'>
              <Typography variant='h6' style={{ overflow: 'hidden', width: '250px', textOverflow: 'ellipsis', whiteSpace: 'nowrap' }}>{data.dsNomeUsuario}</Typography>
              <Typography variant='body2' style={{ width: '250px' }}>{data.dsFuncaoUsuario}</Typography>
              <Typography variant='caption' style={{ width: '250px' }}>{data.dsSetor}</Typography>
            </Grid>
            {(data.dtExclusaoUsuario) ? <Typography variant='h6'>Usuário excluido em {data.dtExclusaoUsuario}</Typography> : (
              ((userId && (loggedUserId !== userId)) || (local.userId && (loggedUserId !== local.userId))) && (
                <React.Fragment>
                  {(data.nrRamalUsuario) && (
                    <Grid item xs>
                      <Button fullWidth variant='contained' href={`tel: ${data.nrRamalUsuario}`} title={`Ramal: ${data.nrRamalUsuario}`}><IoIosCall/>&nbsp;{data.nrRamalUsuario}</Button>
                    </Grid>
                  )}
                  <Grid item xs>
                    <Button fullWidth variant='contained' href={`mailto: ${data.dsEmailUsuario}`} title={data.dsEmailUsuario}><MdMail/>&nbsp;Email</Button>
                  </Grid>
                  <Grid item xs>
                    <Button fullWidth variant='contained' onClick={() => setAnotherProgramField('janelaChatOpen', returnUser(), 'argousSystem')} title='Abrir chat de conversa'><MdMessage/>&nbsp;Mensagem</Button>
                  </Grid>
                </React.Fragment>
              )
            )}
          </Grid>
        </Card>
      </Popper>
    </Portal>
  );
  return null;
}


export default UserCard;