import { formataNome } from '../../utils';
import { serverAction } from '../../utils/api';
import { TextField } from '@material-ui/core';
import { Autocomplete } from '@material-ui/lab';
import { isArray, isObject, isString } from 'lodash';
import React, { useState, useEffect, useCallback } from 'react';
// import PropTypes from 'prop-types';

const retornaType = type => {
  switch (type) {
    case 'beneficiarios':
      return ({ 
        label: 'Beneficiário', 
        action: 'getBeneficiarios', 
        busca: 'dsBusca',
        res: 'Beneficiarios', 
        getLabel: (option, lista) => isArray(option) ? option.length ? formataNome(option[1]) : '' : option ? lista.filter(i => i[0] === option[0])[0][1] : '' 
      });
    case 'empresas':
      return ({ 
        label: 'Empresa', 
        action: 'getEmpresas', 
        busca: 'dsBusca',
        res: 'Empresas', 
        getLabel: (option, lista) => isArray(option) ? option.length ? `${option[2]} - ${option[3]}` : '' : option ? lista.filter(i => i[0] === option)[0][1] : '' 
      });
    case 'gruposUnimed':
      return ({ 
        label: 'Grupos Unimed', 
        action: 'getGruposUnimed', 
        busca: 'dsBusca',
        res: 'Unimeds', 
        getLabel: (option, lista) => isArray(option) ? option.length ? option[1] : '' : option ? lista.filter(i => i[0] === option)[0][1] : ''
      });
    case 'estados':
      return ({ 
        label: 'Estado', 
        action: 'getEstados', 
        busca: 'dsBusca',
        res: 'Estados', 
        getLabel: (option, lista) => isArray(option) ? option.length ? `${option[0]} - ${option[1]}` : '' : option ? lista.filter(i => i[0] === option)[0][1] : ''
      });
    case 'cidades':
      return ({ 
        label: 'Cidade', 
        action: 'getCidades', 
        busca: 'dsCidade',
        res: 'cidades', 
        getLabel: (option, lista) => isArray(option) ? option.length ? `${formataNome(option[1]?.trim())} - ${option[2]?.trim()}` : '' : option ? lista.filter(i => i[0] === option)[0][1] : ''
      });
    case 'unimeds':
      return ({ 
        label: 'Unimed', 
        action: 'getUnimeds', 
        busca: 'dsBusca',
        res: 'Unimeds', 
        getLabel: (option, lista) => isArray(option) ? option.length ? `${option[0]} - ${option[1]}` : '' : option ? lista.filter(i => i[0] === option)[0][1] : ''
      });
    case 'prestadores':
      return ({ 
        label: 'Prestadores', 
        action: 'getPrestadores', 
        busca: 'dsBusca',
        res: 'Prestadores', 
        getLabel: (option, lista) => isArray(option) ? option.length ? `${option[1]} - ${formataNome(option[2])}` : '' : option ? lista.filter(i => i[0] === option)[0][1] : ''
      });
    default:
      return ({});
  }
};

const UtilAutoComplete = ({ type = '', label, value, textFieldProps = {}, onChange = e => e, excluidos, idEmpresa, cdEstado, cdUnimed, getOptionLabel, pgCode = 'BuscasHelper', action, busca, resposta, searchParams, filterFunction, forceUpdate, readOnly, ...props }) => {
  const [open, setOpen] = useState(false);
  const [lista, setLista] = useState([]);
  const [inputValue, setInputValue] = useState('');

  const buscar = useCallback(async valor => {
    const res = await serverAction(action || retornaType(type).action, { [busca || retornaType(type)?.busca || 'value']: valor || inputValue, excluidos, idEmpresa, cdEstado, cdUnimed, ...searchParams }, true, pgCode);
    setLista(res[resposta || retornaType(type)?.res || 'result'] || []);
  // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [inputValue, type, searchParams]);

  useEffect(() => {
    if (isString(value)) buscar(value);    
  // eslint-disable-next-line react-hooks/exhaustive-deps
  }, []);

  useEffect(() => {
    // if (inputValue.length < 2) return;
    buscar();
  }, [inputValue, buscar]);

  return (
    <Autocomplete
      open={open}
      onOpen={() => !readOnly && setOpen(true)}
      onClose={() => setOpen(false)}
      disableClearable={readOnly}
      options={isObject(value) ? lista.filter(v => filterFunction ? filterFunction(v, value) : true || v[0] !== value[0]) : lista}
      getOptionSelected={(o, v) => o[0] === v}
      getOptionLabel={getOptionLabel ? (option => getOptionLabel(option, lista)) : (option => retornaType(type).getLabel(option, lista)) }
      noOptionsText={inputValue.length < 2 ? 'Digite para pesquisar' : 'Não Encontrado'}
      clearText='Limpar'
      openText='Abrir'
      onInputChange={e => setInputValue(e?.target?.value || '')}
      filterSelectedOptions
      autoComplete
      autoHighlight
      {...{ value, onChange }}
      {...props}
      renderInput={params => (
        <TextField
          margin='dense'
          variant='outlined'
          label={label || retornaType(type).label}
          {...params}
          {...textFieldProps}
          InputProps={{ ...textFieldProps.InputProps, ...params.InputProps }}
          inputProps={{ ...textFieldProps.inputProps, ...params.inputProps, readOnly }}
        />
      )}
    />
  );
};

// UtilAutoComplete.propTypes = {
//   type: PropTypes.oneOf(['beneficiarios', 'empresas', 'gruposUnimed', 'estados', 'cidades', 'unimeds', 'prestadores']).isRequired,
//   value: PropTypes.array,
//   textFieldProps: PropTypes.object,
//   onChange: PropTypes.func,
//   excluidos: PropTypes.bool,
//   idEmpresa: PropTypes.string,
//   cdEstado: PropTypes.string,
//   cdUnimed: PropTypes.string,
//   getOptionLabel: PropTypes.func
// }

export default UtilAutoComplete;