import clsx from 'clsx';
import React, { useCallback, useEffect, useState } from 'react';
import './style.css';


const Zoom = ({ id, style, children }) => {

  const figureId = (id) ? `${id}-zoom-figure` : 'zoom-figure';
  const [active, setActive] = useState(false);

  const copyMediaUrl = useCallback(() => {
    const figureElement = document.getElementById(figureId);
    const mediaElement = figureElement.children?.[0];
    if (mediaElement) figureElement.style.backgroundImage = "url('" + mediaElement.src + "')";
  }, [figureId]);
  useEffect(() => setTimeout(() => copyMediaUrl(), 500), [copyMediaUrl]);

  function zoom(e, activeParameter) {
    if (active || activeParameter) {
      let offsetX, offsetY, x, y;
      const figureElement = document.getElementById(figureId);

      if (figureElement) {
        if (e?.nativeEvent?.offsetX) offsetX = e.nativeEvent.offsetX;
        else offsetX = e?.touches?.[0].pageX;
        if (e?.nativeEvent?.offsetY) offsetY = e.nativeEvent.offsetY;
        else offsetX = e?.touches?.[0].pageX;

        x = ((offsetX / figureElement.offsetWidth) * 100);
        y = ((offsetY / figureElement.offsetHeight) * 100);

        figureElement.style.backgroundPosition = `${x}% ${y}%`;
      }
    }
  }

  const handleActive = e => {
    const state = !active;
    setActive(state);
    (state) && zoom(e, state);
  };


  return <figure 
    {...{ style }} 
    id={figureId} 
    onMouseMove={zoom} 
    onTouchMove={zoom} 
    onClick={handleActive} 
    className={clsx('zoom-figure', { 'zoom-figure-active': active })}
  >
    {children}
  </figure>;
};


export default Zoom;