import { initField, fieldChangeListener, clearField, setField, getField, fieldUpdatesActionHandler, waitFieldsData, setServerValueToField, setFieldValueToServer, getPreviousField } from './fieldsRedux';
import PropTypes from 'prop-types';
import React, { Component } from 'react';
import { connect } from 'react-redux';

/**
  * @deprecated
  * @param {*} WrappedComponent - 
  * @param {*} opts -
  * @returns {*} -
  */
const createField = (WrappedComponent, opts = { disableInitField: false, disableFieldChangeListener: false }) => {

  class HOCComponent extends Component {

    componentDidMount() {

      if (!opts.disableInitField) {
        const { field, defaultValue } = this.props;
        initField(field, defaultValue);
      }
    }

    componentDidUpdate(prevProps) {

      if (!opts.disableFieldChangeListener) {
        const { field, fieldListener } = this.props;
        fieldChangeListener(field, fieldListener, prevProps);
      }
    }

    componentWillUnmount() {
      const { field, destroyOnUnmount } = this.props;

      if (destroyOnUnmount) {
        clearField(field);
      }
    }

    handleUpdates = (value) => {
      const { field, ajax, onFieldSuccess, action, receiveUpdates = [], sendUpdates = [], onAllUpdated, dados, onActionSuccess } = this.props;

      setField(field, 'value', value, ajax).then(res => {
        if (onFieldSuccess) {
          onFieldSuccess(value);
        }

        fieldUpdatesActionHandler(sendUpdates, receiveUpdates, action, dados, onActionSuccess, onAllUpdated, value);
      });
    }

    getField = (field, key) => {
      return field ? getField(field, key) : getField(this.props.field);
    }

    getPreviousField = (p1, p2, p3) => {
      if (typeof p1 === 'object') {
        return getPreviousField(this.props.field, p1, p2);
      } 
      return getPreviousField(p1, p2, p3);
      
    }

    setField = (p1, p2, p3) => {
      if (typeof p1 === 'object') {
        return setField(this.props.field, p1);
      } else if (typeof p2 === 'object') {
        return setField(p1, p2);
      } else if (!p3) {
        return setField(this.props.field, p1, p2);
      } 
      return setField(p1, p2, p3);
      
    }

    waitFieldsData = (wFields = [], wDatas = [], rejectMilliseconds = 5000, milliseconds = 100) => {
      return waitFieldsData(wFields, wDatas, rejectMilliseconds, milliseconds);
    }

    setServerValueToField = (field) => {
      return field ? setServerValueToField(field) : setServerValueToField(this.props.field);
    }

    setFieldValueToServer = (field) => {
      return field ? setFieldValueToServer(field) : setFieldValueToServer(this.props.field);
    }

    clearField = (field) => {
      field ? clearField(field) : clearField(this.props.field);
    }

    render() {
      return <WrappedComponent
        handleUpdates={this.handleUpdates}
        getField={this.getField}
        getPreviousField={this.getPreviousField}
        setField={this.setField}
        waitFieldsData={this.waitFieldsData}
        setServerValueToField={this.setServerValueToField}
        setFieldValueToServer={this.setFieldValueToServer}
        clearField={this.clearField}
        {...this.props} />;
    }
  }

  HOCComponent.propTypes = {
    field: PropTypes.string.isRequired,
    defaultValue: PropTypes.oneOfType([PropTypes.number, PropTypes.string, PropTypes.object, PropTypes.array]),
    fieldListener: PropTypes.func,
    destroyOnUnmount: PropTypes.bool
  };

  const mapStateToProps = (state) => {
    return { fields: state.argous.fields };
  };

  return connect(mapStateToProps)(HOCComponent);
};

export default createField;
