import { getConfig } from './config';
import { getStore } from './store';
import { action, agGet, agSet } from '../api';
import { setAnotherProgramField } from '../fieldsRedux';

const getCurrentProgram = async (n = 0) => {
  const state = getStore().getState();
  const { currentProgram } = state.argous;
  if (!currentProgram) {
    return new Promise((resolve, reject) => {
      if (n >= 10) reject(console.error('getServerValue, setServerValue ou serverAction chamado antes do mount do Programa. Use programDidMount().'));
      setTimeout(async () => {
        resolve(await getCurrentProgram(n++));
      }, 100);
    });
  }
  return currentProgram;
};

/**
 * @deprecated
 */
export const getServerValue = async (field:string): Promise<any> => {
  const currentProgram = await getCurrentProgram();

  const res = await agGet({ pgCode: currentProgram, field });
  if (res.pgmOk) {
    let value = res[field];
    try {
      value = typeof value === 'object'
        ? value : JSON.parse(value);
    } catch (err) { }
    return value;
  }
  return '';
};

/**
 * @deprecated
 * 
 * @param {string} field - String
 * @param {any} value - Valor
 * 
 * @returns {Promise<any>} retorno 
 */
export const setServerValue = async (field:string, value:any): Promise<any> => {
  const currentProgram = await getCurrentProgram();

  const res = await agSet({ pgCode: currentProgram, [field]: value });
  return !!res.pgmOk;
};

let debuging = false;

export const enableDebugger = () => {
  debuging = true;
};

export const serverAction = async (metodo:string, dados = {}, pgCode?:string|boolean, tratarErro?:string|boolean, token?:string) => {

  const code = typeof pgCode === 'string' ? pgCode : typeof tratarErro === 'string' ? tratarErro : undefined;
  const tErro = typeof tratarErro === 'boolean' ? tratarErro : typeof pgCode === 'boolean' ? pgCode : false;

  const currentProgram = code || await getCurrentProgram();

  const data = {
    pgCode: currentProgram,
    action: metodo,
    dados
  };

  const res = await action(data, token);
  if (debuging && process.env.NODE_ENV !== 'production') {
    console.log('-------- Server Action ---------');
    console.log('pgCode: ', currentProgram);
    console.log('action: ', metodo);
    console.log('dados: ', dados);
    console.log('res: ', res);
    if (res.dsErro) console.error(res.dsErro);
    console.log('--------------------------------');
  }
  if (res.dsErro && tErro) setError(res.dsErro);
  if (res.chTkn) document.cookie = `lt=${res.chTkn};path=/;SameSite=Lax;${getConfig().ssl ? 'secure' : ''}`;
  if (res.blkTkn) {
    setInfo(res.blkTkn, 'Justificativa de Bloqueio', 0);
    window.localStorage.setItem('blkUs', JSON.stringify(getStore().getState().argous.usuarioLogado));
    getStore().dispatch({ type: 'setUsuarioLogado', usuarioLogado: {} });
    document.cookie = `lt=;path=/;SameSite=Lax;${getConfig().ssl ? 'secure' : ''}`;
    getStore().dispatch({ type: 'setBlockUser', block: true });
  }
  if (res.vldPsw) getStore().dispatch({ type: 'setVldPsw', vldPsw: true });
  return res;
};

export function setAlert(mensagem: string, type: 'error' | 'success' | 'info' | 'warning', title: string, time = 5000) {
  setTimeout(() => {
    setAnotherProgramField('systemTopAlert', { mensagem, type, title, time }, 'argousSystem');
  }, 0);
}

export function setError(mensagem:string, title = 'Erro') {
  setTimeout(() => {
    setAnotherProgramField('systemTopAlert', { mensagem, type: 'error', title }, 'argousSystem');
  }, 0);
}

export function setSuccess(mensagem:string, title:string, time = 5000) {
  setTimeout(() => {
    setAnotherProgramField('systemTopAlert', { mensagem, type: 'success', title, time }, 'argousSystem');
  }, 0);
}

export function setInfo(mensagem:string, title:string, time = 5000) {
  setTimeout(() => {
    setAnotherProgramField('systemTopAlert', { mensagem, type: 'info', title, time }, 'argousSystem');
  }, 0);
}

export function setWarning(mensagem:string, title:string, time = 5000) {
  setTimeout(() => {
    setAnotherProgramField('systemTopAlert', { mensagem, type: 'warning', title, time }, 'argousSystem');
  }, 0);
}

export function sendNotification({ id, mensagem, title, type, link, dsLabel }:{ id:string|number, mensagem:string, title:string, type:string, link:string, dsLabel:string }) {
  setTimeout(() => {
    setAnotherProgramField('systemTopAlert', { type: 'notification', id, mensagem, title, link, dsLabel, icon: type }, 'argousSystem');
  }, 0);
}

const getUrlVars = () => {
  var vars:any = {};
  window.location.href.replace(/[?&]+([^=&]+)=([^&]*)/gi, (_, key, value):string => {
    vars[key] = value;
    return '';
  });
  return vars;
};

const deleteURLParameters = () => {
  let params = window.location.href;
  let pos = params.indexOf('?');
  if (pos > 0) params = params.substring(0, pos); 
  window.history.replaceState('object or string', 'Title', params);
};

export const getUrlParameters = async (pgCode?:string) => {
  const urlPrms = getUrlVars();
  if (urlPrms.ctrl) {
    deleteURLParameters();
    return await serverAction('getURLParameters', { ctrl: urlPrms.ctrl }, pgCode, true);
  }
};